# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import toolz

class BoatyardBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        self.show_ship = False
        self.show_gold = False  # нужно показать золото при выходе за экран
        self.gold_shown = False # показано ли золото
        self.show_unit = False  # показывать ли юнита на верфи
        self.is_blocked = False
        base_building.BaseBuilding.__init__(self, cdata)
        self.SetIsHome(False)
        self.hack_for_upgrade = True
        self.faked_upgrade_complete = False
        EMgr.add_event_listener(self)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_event_listener(self)
# ============================================================
    # @override
    def switch_animation(self):
        base_building.BaseBuilding.switch_animation(self)
        self.Drawer.ShowImages('unit', self.show_unit)
        if self.show_ship:
            self.Drawer.ShowImages('ship', True)
        self.Drawer.ShowImages('gold', self.gold_shown)
        if self.is_blocked:
            self.Drawer.ShowImages('button', False)
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        '''Обработчик всех обычных сообщений (которые не были обработаны в on_msg)'''
        if msg_type == 'hammer_event':
            self.PlaySound('SND_HAMMER_HIT_1')
# ============================================================
    # @override
    def on_create(self):
        base_building.BaseBuilding.on_create(self)
        _uid = self.GetAttrStr('unit_linked_uid')
        self.linked_unit = BMgr.get_item(_uid)
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == 'questman_complete':            # юнит получил золото
            self.show_gold = True
            if self.is_offscreen:
                self.gold_shown = True
            self.switch_animation()
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'offscreen':
            if self.is_offscreen: return    # чтобы не дублировать
            self.is_offscreen = True
            if self.faked_upgrade_complete and not self.has_next_level:  # достигнут максимальный вариант...
                if self.show_unit and self.linked_unit:
                    self.linked_unit.ready_to_show(self)
                    self.linked_unit.Signals('try_to_show', 1)
            if self.show_gold:
                self.gold_shown = True
                self.switch_animation()
        elif signal == 'onscreen':
            self.is_offscreen = False
        elif signal == 'faked_upgrade_complete':
            #self.switch_level_after_fake()
            self.SM.SetState('normal')
            #self.show_unit = False
            self.switch_level('boat_build')
            self.faked_upgrade_complete = True
            # ## затычка, чтобы только после постройки лодки считать квест выполненным
            # оставим на всякий случай:
            building_info = {'type':self.get('type'), 'subtype':self.get('stype')}
            EMgr.send_event( 'build_repair_boat_complete', self, building_info )
            if self.is_offscreen:        # если уже за экраном
                if not self.has_next_level:  # достигнут максимальный вариант...
                    if self.linked_unit:
                        self.linked_unit.ready_to_show(self)
                    #self.switch_level_to_ship()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    #@override
    def on_state_change(self, cur_state, next_state):
        base_building.BaseBuilding.on_state_change(self, cur_state, next_state)
        # ---
        if next_state == 'normal':
            if cur_state in ['build', 'upgrade']:
                self.is_blocked = True  # отныне и впредь
        if cur_state != next_state:
            if next_state == 'upgrade':       # затычка для особого поведения верфи, там анимация должна проигрываться
                self.Drawer.RestartEffect('build')
# ============================================================
    #@override
    def on_building_upgrade(self):
        base_building.BaseBuilding.on_building_upgrade(self)
        self.is_deactivated = self.GetAttrBool('deactivated')
# ============================================================
    #@override
    def switch_level_to_ship(self):
        '''Здесь происходит "апгрейд" здания, смена варианта. Здесь должны (!) проверяться все необходимые требования, кроме кол-ва ресурсов'''
        var_name = "level_2"
        if var_name in self.variants_switch:
            next_var_name = self.variants_switch[var_name]
            trace('switch_level_to_ship:')
            trace(' switch_level_to_ship:', self.get('name'), self.get('uid'))
            self.SwitchLevel(next_var_name)   # переключаемся на следующий вариант
            self.update_next_res()
            self.show_ship = True
            self.show_unit = False
            self.switch_animation()
            self.unreserve_action_cells()
            #if self.linked_unit:
                #self.linked_unit.show_as_possible()
        self.update_res_enough_state()
# ============================================================
    #@override
    def switch_level_to_goldship(self):
        '''Здесь происходит "апгрейд" здания, смена варианта. Здесь должны (!) проверяться все необходимые требования, кроме кол-ва ресурсов'''
        var_name = "level_3"
        if var_name in self.variants_switch:
            next_var_name = self.variants_switch[var_name]
            trace('switch_level_to_goldship:')
            trace(' switch_level_to_goldship:', self.get('name'), self.get('uid'))
            self.SwitchLevel(next_var_name)   # переключаемся на следующий вариант
            self.update_next_res()
            self.show_ship = True
            self.switch_animation()
            self.unreserve_action_cells()
        self.update_res_enough_state()
# ============================================================
    def hide_unit(self):
        self.switch_level_to_ship()
# ============================================================
    #@override
    def scale_bump(self):
        if not self.show_unit:
            base_building.BaseBuilding.scale_bump(self)
# ============================================================
    #@override
    def _switch_animation_extra(self, place):
        if place == 'upgrade':
            self.Drawer.ShowImages('build')     # для верфи
            self.Drawer.SetProgressedAnimation('build')
            self.Drawer.ShowImages('repair')     # для верфи
            #TMgr.add_task( task_callworker.CallWorkerTask(self) )
            #b.Drawer.SetFramesProgress("build", b.build_progress/b.build_time)
# ============================================================
    #@override
    def on_worker_in(self, unit):
        '''We need to start faked upgrade'''
        base_building.BaseBuilding.on_worker_in(self, unit)
        # #self.Drawer.SetProgressedAnimation('build')
        self.Drawer.PlayAnimationOnce('build', self.build_time)
        self.Signals.Restart('faked_upgrade_complete', self.build_time)
        self.SM.SetState('upgrade')
        self.PlaySound('SND_BOATYARD_WORK')
        self.Drawer.SetAnimationEvent('unit', 50, 'hammer_event')
        self.show_unit = True
# ============================================================
    #@override
    def is_active(self):
        return base_building.BaseBuilding.is_active(self) and not self.is_blocked
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        if place == 'before_name':
            if self.is_mouse_over_zone == 'ship':
                result.append( ('text', 'COLOR_HEADER') )
                ship_text = self.GetAttrStr('ship_text')
                result.append( ('text', ship_text) )
                return False
        return True
# ============================================================
    #@override
    def Get_tip_position(self, tip_size, camera_pos, window_rect, zoom):
        '''Возвращает координаты типса от левого-верхнего угла картинки типса'''
        if self.is_mouse_over_zone == 'ship':
            return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect, 'notip_ship_rect')
        else:
            return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect)
# ============================================================
