# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building

class FarmBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.SetIsHome(False)
        self.to_building_subtype = "bakery"
        self.fields = [None, None, None, None]
        self.pre_fields = [None, None, None, None]  # для апгрейдов
        self.field_cells = ['field1_cell', 'field2_cell', 'field3_cell', 'field4_cell']
        self.grown_stack = []
        self.is_unit_working = False
        self.has_places_for_fields = True
        # зарезервируем места для полей:`
        for i in range(0,4):
            c_name = self.field_cells[i]
            _cell = self.GetAttrCellFixed(c_name)
            if self.IsPlaceFreeFor(_cell, 'farm_field1', True):
                self.ReservePlaceFor(_cell, 'farm_field1')
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    def on_res_gen_started(self):
        self.PlaySound('SND_FARM_WORK')
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'field_grown':
            self.on_field_grown()
# ============================================================
    # @override
    def on_repair_complete(self):
        '''Для обработки ситуации окончания repair'''
        self.is_generating_res = False
        self.is_unit_working = False
        for field in self.fields:
            if field:
                if field.get('variant_name') == 'broken_1': # если поле было "сломано"
                    field.reinit_field_effectiveness_params(self)
                    field.switch_level()
                field.start_wheat_growth()
# ============================================================
    def on_build_started(self):
        for field in self.fields:
            if field:
                field.switch_level()
# ============================================================
    def on_cancel_build(self):
        for field in self.fields:
            if field:
                BMgr.RemoveItemFromMap(field)
        self.pre_fields = []
# ============================================================
    def has_free_field_places(self):
        index = 0
        for field in self.fields:
            if not field:
                _cell_name = self.field_cells[index]
                _cell = self.GetAttrCellFixed(_cell_name)
                if self.IsPlaceFreeFor(_cell, 'farm_field1'):
                    self.has_places_for_fields = True
                    return True
            else:
                if field.status != 'normal':# затычка от только что начавшейся стройки
                    self.has_places_for_fields = True
                    return True
            index += 1
        self.has_places_for_fields = False
        return False
# ============================================================
    #@override
    def get_raw_button_frame(self):
        _frame = base_building.BaseBuilding.get_raw_button_frame(self)
        if _frame >= 0:
            #if self.status == 'normal' and (self.is_generated_res or self.is_generating_res):
            _can_build = self.has_free_field_places()
            if not _can_build:
                _frame = -1
        return _frame
# ============================================================
    #@override
    def is_active(self):
        '''Возможны ли действия над объектом в данный момент. Метод может быть переопределен'''
        _is_active = base_building.BaseBuilding.is_active(self)
        if _is_active:
            if not self.has_places_for_fields:
                return False
            return True
        else:
            return False
# ============================================================
    #@override
    def start_resource_generation(self):
        res = self.GetRes("normal_gives_1")
        _is_res_empty = True
        if res: _is_res_empty = res.Empty()
        if _is_res_empty:     # если ресурс не задан, то здесь делать нечего
            return
        res_name = res.GetSingleName()
        if self.has_generating_res_progress() or not res_name or not self.res_generation_time or self.status != "normal":
            return
        self.is_generating_res = True
        self.on_res_gen_started()
        self.Drawer.ShowIn('unit')
        self.switch_animation()
        #for field in self.fields:
        #    if field:
        #        field.start_wheat_growth(self.res_generation_time)
# ============================================================
    #@override
    def on_upgrade_complete(self):
        for field in self.pre_fields:
            if field:
                field.reinit_field_effectiveness_params(self)
                field.on_farm_build_complete()
        self.pre_fields = []
# ============================================================
    #@override
    def on_upgrade_started(self):
        for field in self.pre_fields:
            if field:
                field.switch_level()
# ============================================================
    #@override
    def on_build_complete(self, building_info):
        for field in self.fields:
            if field:
                field.reinit_field_effectiveness_params(self)
                field.on_farm_build_complete()
        self.pre_fields = []
# ============================================================
    #@override
    def update_effectiveness(self):
        '''У самой фермы эффективности нет и она не падает'''
        pass
# ============================================================
    #@override
    def give_resource(self):
        res = base_building.BaseBuilding.give_resource(self)
        self.check_grown_stack()
        return res
# ============================================================
    #@override
    def on_create(self):
        base_building.BaseBuilding.on_create(self)
        if self.status == 'broken':
            self.append_field('broken_1')
        elif self.status == 'place':
           self.append_field('place')
        else:
            self.append_field()
# ============================================================
    #@override
    def on_before_building_upgrade(self):
        base_building.BaseBuilding.on_before_building_upgrade(self)
        if self.status == 'normal':
            self.append_field('place')
            for field in self.fields:
                if field:
                    if field.status in ['place', '__none__']:
                        self.pre_fields.append(field)
# ============================================================
    #@override
    def on_upgrade_cancel(self):
        if self.status == 'normal':
            for field in self.pre_fields:
                BMgr.RemoveItemFromMap(field)
        self.pre_fields = []
# ============================================================
    def append_field(self, field_variant = 'level_1'):
        result = []
        _need_fields = 0
        if self.status == 'normal':
            _need_fields = self.variants_indexes[self.get('variant_name')] + 1
        elif self.status in ['broken', 'place']:
            _need_fields = self.variants_indexes[self.get('variant_name')] + 1
        _total = 0
        for i in range(0,4):
            field = self.fields[i]
            if field: _total += 1
        if _total >= _need_fields:
            #mbox(self.status + '-' + str(_total)+' '+str(_need_fields))
            return result
        _need_fields -= _total
        for i in range(0,4):
            if not self.fields[i]:  # если данное поле еще не создано
                c_name = self.field_cells[i]
                _cell = self.GetAttrCellFixed(c_name)
                if self.IsPlaceFreeFor(_cell, 'farm_field1', True):
                    field = BMgr.AddItemToMap('farm_field1', field_variant, _cell, True)
                    result.append(field)
                    self.fields[i] = field
                    field.set_parent_farm(self) # передадим полю ссылку на ферму
                    if self.status == 'normal':
                        #field.reinit_field_effectiveness_params(self)
                        #if not self.is_deactivated:
                        #    field.start_wheat_growth()
                        #else:
                        if self.is_deactivated:
                            field.is_deactivated = True
                    _need_fields -= 1
                    if not _need_fields:
                        break
        return result
# ============================================================
    #@override
    def activate(self):
        base_building.BaseBuilding.activate(self)
        if self.status == 'normal':
            for field in self.fields:
                if field:
                    field.reinit_field_effectiveness_params(self)
                    field.activate()
                    field.start_wheat_growth()
# ============================================================
    def on_field_growth_complete(self, field):
        '''Данный метод вызывается полем'''
        self.grown_stack.append(field)  # добавим поле в стек
        self.SendMsg('field_grown')
# ============================================================
    def on_field_grown(self):
        '''Данный метод вызывается по событию о созревании одного из полей'''
        self.check_grown_stack()
# ============================================================
    def check_grown_stack(self):
        if not self.is_generated_res:   # если ресурса перед зданием нет
            _status = self.GetAttrStr('status')
            if _status == 'normal':
                if self.grown_stack and not self.is_unit_working:
                    field = self.grown_stack.pop(0) # стек все-таки
                    if field:
                        field.start_unit_gather()   #@ пока время берем то же самое
                        self.is_unit_working = True
                        self.switch_animation()
# ============================================================
    def on_field_gather_complete(self, field):
        '''Вызывается полем, когда завершен сбор урожая'''
        self.generate_resource()
        field.start_wheat_growth()  # тут же запустим на поле рост урожая
        self.is_unit_working = False
        self.switch_animation()
# ============================================================
    #@override
    def _switch_animation_extra(self, tag):
        if tag == 'end':
            state = self.SM.GetCurrentState()
            if state in self._normal__empty:
                self.Drawer.ShowImages('anim', self.is_generating_res)
            else:
                self.Drawer.ShowImages('anim', False)
            if not self.worker_units or state == 'upgrade':
                self.Drawer.ShowImages('sleep_unit', False)
            else:
                self.Drawer.ShowImages('sleep_unit', not self.is_unit_working)
        pass
# ============================================================
    #@override
    def on_action_hit_me(self):
        for field in self.fields:
            if field:
                field.Drawer.Blink('', 1, 0xffff8888, 0.5)
# ============================================================
    #@override
    def on_action_hits_me_complete(self):
        base_building.BaseBuilding.on_action_hits_me_complete(self)
        for field in self.fields:
            if field:
                field.break_me()
# #######################################################
# #######################################################
# #######################################################
