# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import building_helper


# ============================================================
# ============================================================
# ============================================================
class HelperAction(object):
    def __init__(self, name, params):
        self.name = name
        self.params = params
# ============================================================
# ============================================================
# ============================================================
class FarmField(base_building.BaseBuilding):
    field_group = ['wheat' + str(i) for i in range(1,10)]
    unit_group = ['unit' + str(i) for i in range(1,10)]
    field_shaded_layers = ['floor','default','unit','resources','upgrading','build','anim']

    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.SetIsHome(False)
        self.bhelper = building_helper.BuildingHelper() # помощник в смене групп изображений
        self.bhelper.initialize(self.Drawer)
        self.is_process_started = False
        self.growth_percent = 0.5     # отношение времени созревания к суммарному времени сбора+созревания
        self.farm = None
        if self.is_shaded:
            self.shade_layers()
        for group in self.field_group:   # чистим поля
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
    def set_parent_farm(self, farm):
        self.farm = farm
# ============================================================
    def shade_layers(self):
        self.Drawer.SetShaded(self.field_shaded_layers, True)
        self.Drawer.SetShaded(self.field_group, True)
        self.Drawer.SetShaded(self.unit_group, True)
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'mower_event':
            self.PlaySound('SND_FARM_MOWER')
# ============================================================
    #@override
    def switch_animation(self):
        base_building.BaseBuilding.switch_animation(self)
        if not self.Drawer.IsPartiallyInFog():
            self.Drawer.ShowImages(self.field_group, (self.status == 'normal'))
            for field in self.field_group:
                self.Drawer.SetTreeMode(field, 62, 3, 0.3)
            #if self.is_generating_res:
            if self.is_process_started:
                self.bhelper.do_subactions()    # чтобы не потерлись действия, если еще раз вызвана эта функция
            else:
                for group in self.field_group:
                    self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    # @override
    def create_building_units(self):
        pass
# ============================================================
    # @override
    def build_road_on_upgrade(self, prev_status):
        pass
# ============================================================
    # @override
    def start_resource_generation(self):
        pass
# ============================================================
    #@override
    def on_building_upgrade(self):
        base_building.BaseBuilding.on_building_upgrade(self)
        if self.is_shaded:
            self.shade_layers()
        for group in self.field_group:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', self.get_effective_gen_time())
            self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
#    def on_create(self):
#        base_building.BaseBuilding.on_create(self)
# ============================================================
    #def get_effective_gen_time(self):
    #    '''Возвращает время генерации ресурса с учетом эффективности здания'''
    #    _need_res = self.need_base_resource()
    #    if _need_res or self.subtype in self.has_no_effectiveness:
    #        return self.res_generation_time
    #    if self.need_max_time and self.max_res_gen_time:
    #        return self.max_res_gen_time    # если требуется максимальное время генерации
    #    coeff = self.get_effectiveness_coeff()
    #    gen_time = self.res_generation_time
    #    max_gen_time = self.max_res_generation_time
    #    tim = gen_time + ((1 - coeff)/0.9)*(max_gen_time - gen_time)
    #    return tim
# ============================================================
# ============================================================
    def init_grow(self):
        '''Инициализация процесса роста'''
        # запустим рост:
        _anim_time = 2 * self.GetAnimationTime('EP_B_PERS_FARM_FARMER')
        _full_time = self.get_effective_gen_time()
        _growth_time = _full_time - _anim_time*9
        self.growth_percent = _growth_time / _full_time
        if _growth_time < 0: _growth_time = 1
        _growth_times = []
        for i in range(0,9):
            _growth_times.append(tools.Rand(_growth_time*7/10, _growth_time))   # 70%-100%
        i = 0
        for group in self.field_group:
            self.Drawer.PlayAnimationOnce(group, _growth_times[i])
            i += 1
        self.Signals.Restart('growth_complete', _growth_time)
# ============================================================
    def init_gather(self):
        '''Инициализация процесса сбора урожая'''
        _anim_time = 2 * self.GetAnimationTime('EP_B_PERS_FARM_FARMER')
        # запустим сбор урожая:
        _gather_time = _anim_time*9
        _time_per_bush = _gather_time / 9
        _time = 0
        _actions = []
        _act = []
        _act.append(HelperAction( 'reset_anim_time', {'images':'unit1'} ))
        _act.append(HelperAction( 'show', {'images':'unit1'} ))
        _mower_percent = 50
        _act.append(HelperAction( 'set_event', {'image':'unit1', 'percent':_mower_percent, 'event':'mower_event'} ))
        _actions.append( {'time':_time, 'actions':_act} )
        for i in range(2,10):
            _time += _time_per_bush
            _a = []
            _a.append( HelperAction( 'hide', {'images':'unit'+str(i-1)} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'wheat'+str(i-1), 'time':0} ) )
            _unit = 'unit' + str(i)
            _act.append( HelperAction( 'reset_anim_time', {'images':_unit} ) )
            _a.append( HelperAction( 'show', {'images':_unit} ) )
            _a.append( HelperAction( 'set_event', {'image':_unit, 'percent':_mower_percent, 'event':'mower_event'} ) )
            _actions.append({'time':_time, 'actions':_a})
        _time = _gather_time - 1
        _a = []
        _a.append( HelperAction( 'hide', {'images':'unit9'} ) )
        _a.append( HelperAction( 'stop_animation', {'image':'wheat9', 'time':0} ) )
        _actions.append({'time':_time, 'actions':_a})
        self.bhelper.set_actions(self.Signals, _actions)
        self.Signals.Restart('gather_complete', _gather_time)   # само событие нужно только для измерения времени
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'bhelper_signal':
            if not self.bhelper.do_next_action():
                self.is_process_started = False # если выполнили все действия
                self.update_effectiveness()
                self.farm.on_field_gather_complete(self)
        elif signal == 'growth_complete':
            # отправить сообщение ферме
            self.farm.on_field_growth_complete(self)
        elif signal == 'init_grow':
            self.init_grow()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    def start_wheat_growth(self):
        _status = self.GetAttrStr('status')
        if _status == 'normal':
            #self.is_process_started = True
            self.Signals.Restart('init_grow', 1)
# ============================================================
    def start_unit_gather(self):
        _status = self.GetAttrStr('status')
        if _status == 'normal':
            self.is_process_started = True
            self.init_gather()
# ============================================================
    def on_farm_build_complete(self):
        if self.status == 'place':
            self.switch_level()
        if self.status in ['place', 'normal']:
            self.start_wheat_growth()
# ============================================================
#    def _get_tips_extra(self, place, result, by_click):
#        return True
# ============================================================
    #@override
    def _append_genbar(self, result):
        '''Бар производства на типсе'''
        _has_growth = self.Signals.Has('growth_complete')
        if not self.is_process_started and not _has_growth:
            return
        _progress = 0#self.Signals.GetProgress('resource_generated')
        if _has_growth:
            _progress = self.Signals.GetProgress('growth_complete') * self.growth_percent
        else:
            _has_gather = self.Signals.Has('gather_complete')
            if _has_gather:
                _progress = self.growth_percent + self.Signals.GetProgress('gather_complete') * (1-self.growth_percent)
            else:
                _progress = self.growth_percent # пока ожидаем освобождения юнита
        if _progress < 0:
            return
        result.append('newline')
        result.append( ('text', 'COLOR_LINES') )
        result.append( ('text', 'TIP_RES_GENERATION') )
        result.append('newline')
        _bar_images = {'left':'TIP_GENBAR_L', 'right':'TIP_GENBAR_R', 'full':'TIP_GENBAR_FULL', 'empty':'TIP_GENBAR_EMPTY'}
        self._append_bar(result, _progress, _bar_images)
# ============================================================
    def reinit_effectiveness_params(self):
        pass
# ============================================================
    def reinit_field_effectiveness_params(self, farm):
        '''Переинициализирует параметры эффективности здания'''
        if not self.has_effectivness_inited:
            self.has_effectivness_inited = True
            self.res_generation_time = farm.GetAttrInt('res_gen_time')
            _new_max_res_generation_time = farm.GetAttrInt('res_gen_time_max')
            _new_max_res_gen_time = farm.GetAttrInt('gen_max_time')
            if _new_max_res_generation_time == self.max_res_generation_time and\
                _new_max_res_gen_time == self.max_res_gen_time:
                    return  # если параметры не изменились, то значит значения сохраняем
            self.res_gen_first_times = farm.GetAttrInt('res_gen_first_times')
            self.max_res_gen_time = farm.GetAttrInt('res_gen_maxtime')
            if self.current_effectiveness == 100 and self.res_gen_first_times > 0:  # если первый отсчет еще
                self.effectiveness_degradation_count = self.res_gen_first_times
            else:
                self.effectiveness_degradation_count = farm.GetAttrInt('res_gen_ef_count')   # через сколько генераций ресурсов эффективность падает на 10%
            if self.effectiveness_degradation_count == 0:
                self.effectiveness_degradation_count = 3    # дефолтное значение
            self.max_res_generation_time = farm.GetAttrInt('res_gen_time_max')
            if self.max_res_generation_time < self.res_generation_time:
                self.max_res_generation_time = self.res_generation_time + 1 # +1 чтобы избежать деления на ноль
            self.generation_count = 0               # сколько раз уже генерировали ресурс
            self.current_effectiveness = 100        # текущая эффективность (проценты)
# ============================================================
    def break_me(self):
        self.show_arrow(False)
        #self.energy = 100   # на всякий случай восстановим энергию
        trace('field_break_me:')
        trace(' field_break_me:', self.get('name'), self.get('uid'))
        self.SwitchLevel('broken_1')   # переключаемся на сломанный вариант
        self.bhelper.reset()
        self.is_process_started = False
        self.update_next_res()
        self.Drawer.ShowImages(self.field_group, False)
        self.Drawer.ShowImages(self.unit_group, False)
        self.switch_animation()
        self.on_switch_level_complete()
        self.update_res_enough_state()
        self.Signals.Cancel('growth_complete')
        self.Signals.Cancel('init_grow')
        self.Signals.Cancel('gather_complete')
        #self.Signals.Cancel('growth_complete')
# ============================================================
    def on_minimal_effectiveness(self):
        EMgr.AppendLetter([('text','LETTER_EFFECTIVENESS_10')], self.get('uid'))
        UI.show_top_text('MSG_EFFECTIVENESS_FARM_FIELD', self.get('uid'))
# ============================================================
# ============================================================
