# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import task_qm

class QuarryBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.to_building_subtype = "stonecutter"
        self.SetIsHome(False)
        self.is_internal_free_units = True
        self.has_no_more_stones = False
        self.is_unit_inside = False
        self.around_granites_count = 0
        self.granites_lasts = 0
        #EMgr.add_event_listener(self)
# ============================================================
    # @override
    #def dispose(self):
    #    EMgr.remove_event_listener(self)
# ============================================================
    # @override
    def need_base_resource(self):
        return False
# ============================================================
    # @override
    def on_building_upgrade(self):
        base_building.BaseBuilding.on_building_upgrade(self)
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _granites = BMgr.GetItemsInRadius(self, action_cell, r, 'granite')
        self.around_granites_count = 0
        for g in _granites:
            _contains_percent = g.GetAttrInt('contains_percent')
            _count = _contains_percent / 10
            self.around_granites_count += _count
        self.granites_lasts = self.around_granites_count
# ============================================================
    #def on_event(self, name, obj, extra_dict):
    #    '''Обработчик игровых событий'''
# ============================================================
    # @override
    def take_resource(self, resource):
        _single_value = resource.GetSingleValue()
        if _single_value > 0:
            self.SendMsg('resource_received')
# ============================================================
    def on_switch_level_complete(self):
        self.unit_boost = self.GetAttrInt('unit_boost')
# ============================================================
    # @override
    def start_resource_generation(self):
        '''У нас ресурс не генерится, но нужно отправлять юнита'''
        self._send_quarryman_later()
        self.on_res_gen_started()
# ============================================================
    def need_to_show_unit(self):
        '''Нужно ли показывать юнита в здании'''
        return False
# ============================================================
    def need_pers_no_action(self):
        '''Нужно ли показывать юнита в здании'''
        return self.is_unit_inside and not self.is_under_attack
# ============================================================
    def _send_quarryman_later(self, time = 500):
        self.Signals.Restart('send_quarryman', time)
# ============================================================
    def _send_quarryman(self):
        '''Отправляет лесоруба за древесиной, попутно проверяя необходимые условия'''
        if not self.is_generated_res and self.is_under_attack == 0 and self.status != 'broken':
            _internal_count = len(self._internal_free_units)
            if _internal_count:
                _unit = self._internal_free_units[0]
                result = self.get_nearest_stone()
                if result:
                    self.show_clock(False)  # песочные часы ожидания
                    _stone, _path = result
                    # путь тоже нужно передавать, чтобы не считать лишний раз
                    _unit.work_boost = self.unit_boost / 100
                    TMgr.add_task( task_qm.QuarryManTask(_stone, _unit, _path) )
                    self.is_unit_inside = False
                else:
                    _has_granite = self.has_more_items()
                    if not _has_granite:
                        # показать замок
                        self.show_lock()  # замок, если нет доступных деревьев
                        self.has_no_more_stones = True   #@ нужно отслеживать, вдруг появились новые деревья!
                                                        #@ тогда нужно сбросить переменную
                                                        #@ сделать show_lock(False)
                                                        #@ вызвать работника
                                                        #@ убрать замок
                        # отправить юнита в главное здание, если это допустимо
                        self.return_worker_home()
                        self.switch_to_blocked_level()  # здание брошено
                        letter = [ ('text', 'MSG_BUILDING_NO_STONES') ]
                        EMgr.AppendLetter(letter, self.get('uid'))
                        UI.show_top_text('MSG_BUILDING_NO_STONES', self.get('uid'))
                    else:
                        self._send_quarryman_later(2000)   # через две секунды попробуем еще раз - вдруг туман откроется
                self.switch_animation()
# ============================================================
    def has_more_items(self):
        '''Проверяем - есть ли еще гранит вокруг, учитываем даже в тумане'''
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _granites = BMgr.GetItemsInRadius(self, action_cell, r, 'granite')
        _around_granites_count = len(_granites)
        return (_around_granites_count > 0)
# ============================================================
    #@override
    def update_effectiveness(self):
        pass
# ============================================================
    #@override
    def reinit_effectiveness_params(self):
        '''Хитрый хак'''
        _cur_eff = self.current_effectiveness
        base_building.BaseBuilding.reinit_effectiveness_params(self)
        self.current_effectiveness = _cur_eff
# ============================================================
    def return_worker_home(self):
        has_workers = len(self.worker_units)
        if has_workers:
            unit = self.worker_units.pop(0)
            #self._internal_free_units.pop(0)    #
            UMgr.remove_building_worker()  # уменьшим число занятых зданиями работников
            unit.start_work()       # освобождаем юнита от домика
            unit.set_current_subtype('simple')
            unit.show()       # освобождаем юнита от домика
            UMgr.add_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            self.is_unit_inside = False
            self.switch_animation()
            self.PlaySound('SND_BUILDING_ABANDONED')
        #unit.go_home()      # отправим его в ближайший дом
# ============================================================
    def get_nearest_stone(self):
        _action_cell = self.FixAddCell(self.cell_pos, self.GetAttrCell('action_cell'))
        result = self.GetNearestTreeInRadius('different', 'granite', _action_cell, self.GetAttrInt('action_r'))
        return result
# ============================================================
    #@override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type in ['upgrade_complete', 'resource_was_taken']:
            if self.status == 'normal':
                if self.worker_units:
                    self._send_quarryman_later()
        elif msg_type == 'resource_received':       # когда юнит принес ресурс
            self.granites_lasts -= 1       # уменьшим число собранных деревьев
            self.current_effectiveness = 100* self.granites_lasts / self.around_granites_count
            self.generate_resource()
            self.is_unit_inside = True
            self.switch_animation()
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'send_quarryman':
            self._send_quarryman()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
            if signal == 'show_zone':
                action_cell = self.GetAttrCellFixed('action_cell')
                r = self.GetAttrInt('action_r')
                LBoard.HighlightRadius(self, 'different','granite', action_cell, r)
# ============================================================
    #@override
#    def on_mouse_in(self, over_zone):
#        base_building.BaseBuilding.on_mouse_in(self, over_zone)
# ============================================================
    #@override
    def on_mouse_out(self):
        base_building.BaseBuilding.on_mouse_out(self)
        LBoard.HighlightRadius(False)
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        if place == 'after_name':
            if self.SM.GetState() not in self._normal__empty:
                self._append_pbar(result)   # добавим прогрессбар, если нужно
                return False
        return True
# #######################################################
# #######################################################
# #######################################################
