# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import task_wc

class WoodcutterBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.to_building_subtype = "sawmill"
        self.SetIsHome(False)
        self.is_internal_free_units = True
        self.has_no_more_trees = False
        self.is_unit_inside = False
        self.unit_boost = self.GetAttrInt('unit_boost')
        self.around_trees_count = 0
        self.trees_lasts = 0
        EMgr.add_event_listener(self)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_event_listener(self)
# ============================================================
    # @override
    def on_building_upgrade(self):
        base_building.BaseBuilding.on_building_upgrade(self)
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _trees = BMgr.GetItemsInRadius(self, action_cell, r, 'tree')
        self.around_trees_count = len(_trees)
        self.trees_lasts = self.around_trees_count
# ============================================================
    # @override
    def need_base_resource(self):
        return False
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == 'tree_grown':            # если выросло новое дерево
            self._send_woodcutter_later()
# ============================================================
    # @override
    def take_resource(self, resource):
        _single_value = resource.GetSingleValue()
        #self.base_resource_count += _single_value
        if _single_value > 0:
            self.SendMsg('resource_received')
# ============================================================
    def on_switch_level_complete(self):
        self.unit_boost = self.GetAttrInt('unit_boost')
# ============================================================
    # @override
    def start_resource_generation(self):
        '''У нас ресурс не генерится, но нужно отправлять юнита'''
        self._send_woodcutter_later()
        self.on_res_gen_started()
# ============================================================
    def need_to_show_unit(self):
        '''Нужно ли показывать юнита в здании'''
        return False
# ============================================================
    def need_pers_no_action(self):
        '''Нужно ли показывать юнита в здании'''
        return self.is_unit_inside and not self.is_under_attack
# ============================================================
    def _send_woodcutter_later(self, time = 500):
        self.Signals.Restart('send_woodcutter', time)
# ============================================================
    def _send_woodcutter(self):
        '''Отправляет лесоруба за древесиной, попутно проверяя необходимые условия'''
        if not self.is_generated_res and self.is_under_attack == 0 and self.status != 'broken':
            _int_units_count = len(self._internal_free_units)
            if _int_units_count > 0:
                _unit = self._internal_free_units[0]
                result = self.get_nearest_tree()
                if result:
                    self.show_clock(False)  # песочные часы ожидания
                    _tree, _path = result
                    # путь тоже нужно передавать, чтобы не считать лишний раз
                    _unit.work_boost = self.unit_boost / 100
                    TMgr.add_task( task_wc.WoodCutterTask(_tree, _unit, _path) )
                    self.is_unit_inside = False
                else:
                    _has_trees = self.has_more_items()
                    if not _has_trees:
                        # показать замок
                        self.show_lock()  # замок, если нет доступных деревьев
                        self.has_no_more_trees = True   #@ нужно отслеживать, вдруг появились новые деревья!
                                                        #@ тогда нужно сбросить переменную
                                                        #@ сделать show_lock(False)
                                                        #@ вызвать работника
                                                        #@ убрать замок
                        # отправить юнита в главное здание, если это допустимо
                        #if not BMgr.IsBuildableOnLevel('forester1'):
                        self.return_worker_home()
                        self.switch_to_blocked_level()  # здание брошено
                        letter = [ ('text', 'MSG_BUILDING_NO_TREES') ]
                        EMgr.AppendLetter(letter, self.get('uid'))
                        UI.show_top_text('MSG_BUILDING_NO_TREES', self.get('uid'))
                        #else:
                        #    mbox('Is it first level???')
                    else:
                        self._send_woodcutter_later(2000)   # через две секунды попробуем еще раз - вдруг туман откроется
                self.switch_animation()
# ============================================================
    def has_more_items(self):
        '''Проверяем - есть ли еще деревья вокруг, учитываем даже в тумане'''
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _trees = BMgr.GetItemsInRadius(self, action_cell, r, 'tree')
        _around_trees_count = len(_trees)
        return (_around_trees_count > 0)
# ============================================================
    def return_worker_home(self):
        has_workers = len(self.worker_units)
        if has_workers:
            unit = self.worker_units.pop(0)
            #self._internal_free_units.pop(0)    #
            UMgr.remove_building_worker()  # уменьшим число занятых зданиями работников
            unit.start_work()       # освобождаем юнита от домика
            unit.set_current_subtype('simple')
            unit.show()       # освобождаем юнита от домика
            UMgr.add_free_unit(unit)
            self.is_unit_inside = False
            self.switch_animation()
            self.PlaySound('SND_BUILDING_ABANDONED')
        #unit.go_home()      # отправим его в ближайший дом
# ============================================================
    def get_nearest_tree(self):
        _action_cell = self.GetAttrCellFixed('action_cell')
        # _trees = Map.GetItemsInRadius(_action_cell, self.GetAttrInt('action_r'), 'tree', 'tree')
        result = self.GetNearestTreeInRadius('tree', 'tree', _action_cell, self.GetAttrInt('action_r'))
        return result
# ============================================================
    #@override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type in ['upgrade_complete', 'resource_was_taken']:
            if self.status == 'normal':
                if self.worker_units:
                    self._send_woodcutter_later()
        elif msg_type == 'resource_received':       # когда юнит принес ресурс
            self.trees_lasts -= 1       # уменьшим число собранных деревьев
            self.current_effectiveness = 100 * (self.trees_lasts / self.around_trees_count)
            self.generate_resource()
            self.is_unit_inside = True
            self.switch_animation()
        elif msg_type == 'tree_grown': #@ должно вызываться извне, когда вырастет новое дерево, желательно сразу проверить - в досягаемости ли оно для здания
            pass
# ============================================================
    #@override
    def update_effectiveness(self):
        pass
# ============================================================
    #@override
    def reinit_effectiveness_params(self):
        '''Хитрый хак'''
        _cur_eff = self.current_effectiveness
        base_building.BaseBuilding.reinit_effectiveness_params(self)
        self.current_effectiveness = _cur_eff
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'send_woodcutter':
            self._send_woodcutter()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
            if signal == 'show_zone':
                action_cell = self.GetAttrCellFixed('action_cell')
                r = self.GetAttrInt('action_r')
                LBoard.HighlightRadius(self, 'tree','tree', action_cell, r)
# ============================================================
    #@override
#    def on_mouse_in(self, over_zone):
#        base_building.BaseBuilding.on_mouse_in(self, over_zone)
# ============================================================
    #@override
    def on_mouse_out(self):
        base_building.BaseBuilding.on_mouse_out(self)
        LBoard.HighlightRadius(False)
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        if place == 'after_name':
            if self.SM.GetState() not in self._normal__empty:
                self._append_pbar(result)   # добавим прогрессбар, если нужно
                return False
        return True
# #######################################################
# #######################################################
# #######################################################
