# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_walkg
import task_walkw
import task_classes
import toolz


class DiffItem(DifferentItem):
    resource_names = ['food', 'bricks', 'boards', 'coins', 'gold']  #, 'gem_red', 'gem_green', 'gem_blue']
    res_images = {
        "food": "IMG_I_TIP_FOOD",
        "bricks": "IMG_I_TIP_BRICKS",
        "boards": "IMG_I_TIP_BOARDS",
        "coins": "IMG_I_TIP_COINS",
        "gold": "IMG_I_TIP_GOLD"
        }
# ============================================================
    item_layers = ['floor','default','unit','resources','upgrading','build','anim','cover']
    bump_layers = ['default']
    hlight_layers = ['default','cover']
# ============================================================
    def __init__(self, cdata):
        DifferentItem.__init__(self, cdata)

        self.subtype = self.get('stype')
        self.action_time = 1
        self.action_progress = 0
        self.cell_pos = self.get('cell')        # initializing position member
        self.res_action_cost = self.get_action_cost_res()
        self.action_was_started = False
        self.action_hits_last = self.GetAttrInt('action_hits_count')
        self.action_hits_count = self.action_hits_last
        self.action_started = False
        self.task = None    # затычка, чтобы не падало на объектах, у которых нет своего класса
        self.is_mouse_over = False
        self.completeGivesRes = self.GetRes('gives') # ресурсы, которые отдает объект после окончания работы с ним
        self.is_mouse_over_zone = ''
        self.has_check = False      # установлена ли галка участия в задании
        self.can_show_uncheck = False   # можно ли показывать иконку отмены (это затрагивает только визуальную часть)
        self.need_attention_effect = False
        self.need_born_effect = False
        self.arrow_shown = False
        self.activate_show_arrow = self.GetAttrBool('activate_show_arrow')
        self.clock_shown = False
        self.action_enable_complete = False
        self.activate_uid = self.GetAttrStr('activate_uid')
        self.activated_focus = self.GetAttrBool('activated_focus')
        self.had_action = False     # было ли хоть одно действие над объектом?
        self.extra_event = self.GetAttrStr('extra_event')   # нужно в самом начале прочитать эти данные, чтобы вариант не перетер
        self.tut_text_id = self.GetAttrStr('tutorial_text')
        self.is_secret = self.GetAttrBool('is_secret')
        self.need_to_remove_offscreen = False   # нужно ли удалить юнита при выходе за пределы экрана (используется индейцем после окончания квеста)
        self.is_offscreen = True
        #self.secret_tips_shown = False
        self.is_shaded = self.GetAttrBool('shaded')
        if self.is_shaded:
            self.Drawer.SetShaded(self.item_layers, True)
        self.is_deactivated = self.GetAttrBool('deactivated')
        self.non_selectable = self.GetAttrBool('non_selectable')
        if self.non_selectable:
            self.SetMouseActive(False)
        #self.on_building_upgrade()
# ============================================================
    def dispose(self):
        pass
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'click':
            _has_access = self.HasAccess()
            if _has_access:
                self.on_click(msg_param1)
            else: LBoard.CrossNoToLastClick()
        elif msg_type != 'on_create':
            self.on_message(msg_type, msg_param1, msg_param2, msg_dict)
        elif msg_type == 'on_create':
            self.on_create()
        #elif msg_type == 'variant_change':
        #    self.on_variant_change(msg_param1)
# ============================================================
    def on_msg_now(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'mouse_in':
            self.on_mouse_in(msg_param1)
            #if self.is_secret:
                #if self.is_active():
                    #self.Drawer.ShowImages('secret')
                    #self.secret_tips_shown = True
        elif msg_type == 'mouse_out':
            #if self.is_secret:
                #self.Drawer.ShowImages('secret', False)
                #self.secret_tips_shown = False
            self.on_mouse_out()
# ============================================================
    # interaction interface for objects
    def on_interact(self, sender, cmd, additional):
        if cmd == "building":
            return
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        pass
# ============================================================
    def on_signal(self, signal):
        if signal == 'attention_signal':
            if self.is_active():
                self.show_attention_effect()
            self.add_attention_signal() # сразу перезапустим сигнал
        elif signal == 'offscreen':
            if self.is_offscreen: return    # чтобы не дублировать
            self.is_offscreen = True
            if self.need_to_remove_offscreen:   # если нужно удалить после выхода за экран
                BMgr.RemoveItemFromMap(self)    # удалим объект
                return
            #elif not self.is_shown:
            #    if self.need_to_show:
            #        self.show_as_possible()
        elif signal == 'onscreen':
            self.is_offscreen = False
        elif signal == 'kill_self':
            BMgr.RemoveItemFromMap(self)    # удаляем себя
            return
        elif signal == 'got_access':
            _seq = 0
            if not self.is_green_state(): _seq = 1
            self.Drawer.SetSequence('arrow', _seq)
            # показ подсказок на объекты:
            self.show_tutorial_text()
            self.switch_animation()
            # #if self.GetAttrStr('debug') == '123':
            # #    mbox(':' + str(_seq))
        elif signal == 'no_access':
            self.Drawer.SetSequence('arrow', 1)    # в любом случае красная будет
            self.switch_animation()
            # #if self.GetAttrStr('debug') == '123':
            # #    mbox('-1-')
        pass
# ============================================================
    def show_tutorial_text(self):
        '''Один раз показывает подсказку генерала'''
        if self.tut_text_id:
            UI.show_tut_text(self.tut_text_id, False, self.get('uid'))
            self.tut_text_id = ''
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == "enter":
            self.on_state_enter(state_name, additional)
            return
        elif type == "exit":
            self.on_state_exit(state_name, additional)
            return
        elif type == "change":
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_enter(self, state_name, additional):
        self.switch_animation()
        pass
# ============================================================
    def on_state_exit(self, state_name, additional):
        pass
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def on_click(self, btn_name):
        '''Вызывается при клике по объекту'''
        #_is_selectable = not self.non_selectable
        #if _is_selectable:
        #    _action_not_started = not self.action_was_started# and not self.action_started
        #    if _action_not_started:
        #        if not self.is_deactivated:
        _is_active = self.is_active()
        if _is_active:
                _is_enough_params = self.is_enough_params()
                if _is_enough_params:
                    if not self.task:
                        _item_actor = self.GetAttrStr('item_actor')
                        if _item_actor == 'general':
                            #if UMgr.general and not UMgr.general.task:   # чтобы на генерала вешалось максимум одно задание
                                self.task = task_walkg.WalkGeneralTask(self)
                                TMgr.add_task( self.task )
                                self.show_arrow(False)
                                return
                        else:
                            _res = self.res_action_cost     # сколько стоит действие
                            if _res:
                                _is_res_enough = UStats.IsResEnough(_res)
                            if _is_res_enough:    # если ресурсов достаточно
                                self.task = task_walkw.WalkWorkerTask(self)
                                TMgr.add_task( self.task )
                                self.show_arrow(False)
                                return
                    elif not self.task.is_action_started:
                        self.task.cancel()
                        self.task = None
                        return
                    #elif not self.action_was_started:
                    #    self.task.cancel()
                    #    self.task = None
                    #    return
                else:
                    _param_name = self._get_not_enough_param_name()
                    UI.BlinkGem(3, _param_name)
        #else:
        #LBoard.MoveCameraToLastClick()
        LBoard.CrossNoToLastClick()
# ============================================================
    def on_mouse_in(self, over_zone):
        _is_active = self.is_active()
        if not _is_active:
            self.is_mouse_over = False
            self.is_selected = False
            self.Drawer.ShowImages('select', False)
            return
        if _is_active:
            self.is_mouse_over = True
            self.is_mouse_over_zone = over_zone
            self.Drawer.HighlightImages(self.hlight_layers, True)
            self.Drawer.ShowImages('select', True)
            check_frame = 0
            if self.need_show_uncheck(): check_frame = 1
            self.Drawer.SetFrame('check', check_frame)
# ============================================================
    def on_mouse_out(self):
        _is_active = self.is_active()
        if not _is_active:
            self.is_mouse_over = False
            self.is_selected = False
            return
        self.can_show_uncheck = True    # теперь крестик показывать можно
        self.is_mouse_over = False
        self.is_mouse_over_zone = ''
        self.Drawer.HighlightImages(self.hlight_layers, False)
        self.Drawer.ShowImages('select', False)
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        self.Drawer.SetFrame('check', check_frame)
# ============================================================
    def on_variant_change(self, new_variant_name):
        '''Вызывается после смены варианта в этом же кванте'''
        self.res_action_cost = self.get_action_cost_res()
        self.action_hits_last = self.GetAttrInt('action_hits_count')
        self.action_hits_count = self.action_hits_last
        if self.is_shaded:
            self.Drawer.SetShaded(self.item_layers, True)
        self.non_selectable = self.GetAttrBool('non_selectable')
        if self.non_selectable:
            self.SetMouseActive(False)
        self.switch_animation()
# ============================================================
    def remove_me_later(self):
        ''' Удаляет тестовый объект с поля, считаем его "взятым" '''
        #BMgr.RemoveItemFromMap(self)
        self.Signals.Add('kill_self', 100)
        self.Drawer.Hide(True)
        self.deactivate()
        self.SetMouseActive(False)
# ============================================================
    #def on_building_upgrade(self):
    #    pass
# ============================================================
    def on_create(self):
        # деревья переопределяют данный метод!
        self.arrow_shown = self.GetAttrBool('arrow_shown')
        self.switch_animation()
        _seq = 0
        if not self.is_green_state(): _seq = 1
        self.Drawer.SetSequence('arrow', _seq)
        _has_access = self.IsPassable()
        if _has_access:     # если проходимость есть при старте
            self.show_tutorial_text()
        # #if self.GetAttrStr('debug') == '123':
        # #    mbox(str(_seq))
# #######################################################
# #######################################################
# #######################################################
    def switch_animation(self):
        #state_name = self.SM.GetCurrentState()
        if self.Drawer.IsPartiallyInFog():
            self.Drawer.ShowImages('', False)
            return
        self.Drawer.ShowImages('', False)
        #if self.is_active():
        #    self.Drawer.ShowImages('secret', self.secret_tips_shown)
        self.Drawer.ShowImages(['floor','icons','shadow'])
        self._switch_animation_extra('begin')
        image_category = 'default'
        #dict = {}
        #if state_name in dict:
        #    image_category = dict[state_name]
        self.Drawer.ShowImages(image_category, True)
        self.Drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated and not self.action_started)
        self.Drawer.ShowImages('check', self.has_check)
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        self.Drawer.SetFrame('check', check_frame)
        self.Drawer.ShowImages('arrow', self.arrow_shown)
        self.Drawer.ShowImages('clock_wait', self.clock_shown)
        self.Drawer.ShowImages('born', self.need_born_effect)
        self.Drawer.ShowImages('attention', self.need_attention_effect)
        self.Drawer.ShowImages('extra_1', True)
        self._switch_animation_extra('end')
# ============================================================
    def _switch_animation_extra(self, place):
        '''Метод для переопределения в подклассах
        place: точка вызова метода'''
        pass
# ============================================================
    def need_show_uncheck(self):
        return self.is_mouse_over and self.can_show_uncheck
# ============================================================
    def Invalidate(self):
        '''перерисовка объекта, вызывается из C++'''
        self.switch_animation()
# ============================================================
    def switch_variant(self, variant_name):
        self.SwitchVariant(variant_name)
        self.on_variant_change(variant_name)
        pass
# ============================================================
    def set_check(self, visible):
        '''Ставит/убирает галку над объектом'''
        if self.has_check == visible:
            return
        self.has_check = visible
        if self.get('stype') != 'tree':
            if visible: self.PlaySoundG('SND_CLICK_TASK')
        if self.has_check:
            self.can_show_uncheck = False   # чтобы крестик не показывался, пока курсор не выйдет за пределы объекта
        self.Drawer.SetFrame('check', 0)
        self.switch_animation()
# ============================================================
# воздействие юнита:
# ============================================================
    def is_action_complete(self):
        '''Действие [юнитом] над данным объектом уже было выполнено'''
        return self.action_hits_last <= 0
# ============================================================
#    def get_hits_last(self):
#        '''Сколько ударов осталось сделать для того, чтобы разбить/отремонтировать/построить данный объект'''
#        return self.action_hits_last
# ============================================================
    def show_secret_dialog(self):
        _secret_desc = self.GetAttrStr('secret_desc')
        self.PlaySoundG('SND_SECRET_FOUND')
        Camera.MoveTo(self)
        dlg.Open('SecretMenu', 'dlg_secret', {'text':_secret_desc, 'showin_effect':'secret_fly_in', 'pause':str(400)})
# ============================================================
    def action_hit_me(self, count=1):
        '''По объекту сделан удар
        count: сколько ударов за раз'''
        if self.is_action_complete():     # на всякий случай, чтобы объект нельзя было разрушить дважды
            return
        self.action_was_started = True  # чтобы уже нельзя было отменить
        self.action_hits_last -= count
        self.action_progress = 1 - self.action_hits_last / self.action_hits_count
        self.on_action_hit_me()        # чтобы можно было переопределить
        if self.action_hits_last <= 0:
            self.action_was_started = False # чтобы объект работал, если у него сложная логика
            effect_digged_name = self.GetAttrStr('effect_complete_name')
            if effect_digged_name:
                self.AddEffect(effect_digged_name, self.get('position'))
            if self.is_secret:
                UStats.AddLevelValue('secrets', 1)
                self.show_secret_dialog()
            self.on_action_hits_me_complete()
            self.activate_on_action_complete()
# ============================================================
    def on_action_hit_me(self):
        pass
# ============================================================
    def on_action_hits_me_complete(self):
        pass        # need to override
# ============================================================
    def on_before_action_started(self):
        # отключим селекты перед началом действия
        self.is_selected = False
        self.Drawer.HighlightImages(self.hlight_layers, False)
        self.Drawer.ShowImages('select', False)
# ============================================================
    def start_comet(self):
        '''Запускает комету, если указана в атрибутах, иначе просто дает сигнал на завершение удаления'''
        _comet_effect_name = self.GetAttrStr('comet_name')
        if _comet_effect_name:
            LBoard.StartComet(self, _comet_effect_name, 'comet_fly_complete')
            self.PlaySoundG('SND_ITEM_FLIGHT')
        else:
            self.Signals.Restart('comet_fly_complete', 1)
# ============================================================
# ============================================================
    def get_subtask(self, parent_task, actor):
        return task_classes.Subtask(parent_task, actor)
# ============================================================
    def get_unit_subtype(self):
        _unit_subtype = self.GetAttrStr('actor_subtype')
        if not _unit_subtype:
            return 'simple'
        else:
            return _unit_subtype
# ============================================================
    def show_arrow(self, show = True):
        self.arrow_shown = show
        self.switch_animation()
# ============================================================
    def is_active(self):
        '''Возможны ли действия над объектом в данный момент. Метод может быть переопределен'''
        _not_deactivated = not self.is_deactivated
        _is_selectable = not self.non_selectable
        _action_not_started = not self.action_was_started# and not self.action_started
        if _not_deactivated:
            if _is_selectable:
                if _action_not_started:
                    return True
        return False
# ============================================================
    def deactivate(self):
        self.is_deactivated = True
        self.switch_animation()
# ============================================================
    def activate(self):
        if self.is_deactivated:
            if self.activate_show_arrow:
                self.show_arrow()
                self.activate_show_arrow = False
        self.is_deactivated = False
        self.switch_animation()
# ============================================================
    def get_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        return 'action_cell'
# ============================================================
    def _filter_available_cells(self, cells_names):
        '''Выбирает из списка имен ячеек-атрибутов те, которые доступны для подхода'''
        _cells_names_result = []
        for _cell_name in cells_names:
            _cell = self.GetAttrCellFixed(_cell_name)
            _is_free = self.IsCellFree(_cell)
            if _is_free:
                _is_accessible = self.IsCellAccessible(_cell)
                if _is_accessible:
                    _cells_names_result.append(_cell_name)
        return _cells_names_result
# ============================================================
    def is_green_state(self):
        '''Цвет типса/селекта'''
        _has_access = self.HasAccess()
        if self.is_deactivated:
            if _has_access:
                return True
            else:
                return False
        if not _has_access:
            return False
        if not self.is_enough_params():
            return False
        if self.res_action_cost:
            _is_res_enough = UStats.IsResEnough(self.res_action_cost)
            if not _is_res_enough:   # если хватает ресурсов на действие над объектом
                return False
        _is_active = self.is_active()
        if _is_active:
            return True
        return True
# ============================================================
    def Get_tip_image(self):
        '''Возвращает ID изображения, которое используется для бэка типса'''
        # чтобы обновить кадр селекта:
        _frame = 0
        if not self.is_green_state(): _frame = 1
        self.Drawer.SetFrame('select', _frame)
        self.Drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        # ---
        if self.is_green_state():
            return "IMG_I_TIP_BG"
        return "IMG_I_TIP_BG_RED"
# ============================================================
    def get_item_name(self):
        return self.GetAttrStr('item_name')
# ============================================================
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        result = []
        if not self._get_tips_extra('before_name', result, by_click): return result
        item_name = self.get_item_name()
        if self.is_secret and not by_click:
            return [('image', 'IMG_I_TIP_SECRET')]
        if not item_name or item_name == 'NO_TIPS':
            return result
        if not by_click:
            result.append( ('text', 'COLOR_HEADER') )
            result.append( ('text', item_name) )
            if not self.is_active():
                return result
            #if self.SM.GetCurrentState() in ["", "normal"]:
            if not self.has_check:
                res = self.res_action_cost
                _is_res_empty = True
                if res: _is_res_empty = res.Empty()
                if not _is_res_empty:
                    result.append( 'newline' )
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ('text', 'NEED_FOR_ACTION') )
                    result.append( 'space' )
                    res_numbers = self._append_res_values(res)
                    result.extend( res_numbers )
                    if not UStats.IsResEnough(res):
                        diff_res = UStats.GetNotEnoughRes(res)
                        if diff_res:
                            result.append( 'newline' )
                            result.append( ('text', 'NOT_ENOUGH') )
                            result.append( 'space' )
                            res_numbers = self._append_res_values(diff_res)
                            result.extend( res_numbers )
                self._append_need_gems(result)
            # теперь опишем что дает данный элемент
            self._get_gives(result)
            if not self._get_tips_extra('end', result, by_click): return result
        else:       # by click
            if not self._get_tips_extra('before_click', result, by_click): return result
            _has_access = self.HasAccess()
            if not _has_access:
                result = []
                result.append( ('text', 'COLOR_RED') )
                result.append( ('text', 'NO_PATH') )
                return result
            if not self._get_tips_extra('before_res', result, by_click): return result
            _is_active = self.is_active()
            if not _is_active:
                return []
            res = self.res_action_cost
            _is_res_empty = True
            if res: _is_res_empty = res.Empty()
            if not _is_res_empty:
                if not UStats.IsResEnough(res):
                    diff_res = UStats.GetNotEnoughRes(res)
                    if diff_res:
                        result.append( ("text", "COLOR_RED") )
                        result.append( ("text", "NOT_ENOUGH") )
                        result.append( 'space' )
                        res_numbers = self._append_res_values(diff_res)
                        result.extend( res_numbers )
                    #else:
                    #    result = [('digit', 1)]
            self._append_not_enough_gems(result)
            if not self._get_tips_extra('end_click', result, by_click): return result
            if not result:
                return self.Get_tip_text(False) # если по клику нет типса, то вернем его как без клика
        return result
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
        '''Предназначается для переопределения в потомках
        place: метка позиции вставки типса
        result: список значений, в который нужно добавлять данные
        by_click: типс по клику или нет
        returns: False - для завершения типса, т.е. после него ни одной строки не прибавится'''
        return True
# ============================================================
    def _append_need_gems(self, result):
        _need_power = self.GetAttrInt('need_power')
        _need_oratory = self.GetAttrInt('need_oratory')
        if _need_oratory > 1:
            result.append( ("text", "COLOR_LINES") )
            result.append('newline')
            result.append(('text', 'TO_SPEAK'))
            result.append( 'space' )
            result.append(('text', 'TIPS_ORATORY'))
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
            result.append( 'space' )
            result.append( ('digit', _need_oratory) )
        if _need_power > 1:
            result.append( ("text", "COLOR_LINES") )
            result.append('newline')
            result.append(('text', 'TO_SPEAK'))
            result.append( 'space' )
            result.append(('text', 'TIPS_POWER'))
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
            result.append( 'space' )
            result.append( ('digit', _need_power) )
# ============================================================
    def _append_not_enough_gems(self, result):
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        # здесь нужно указать чего и сколько нужно (по клику)
        _cur_res = UStats.GetCurRes()
        _ne_oratory = _need_oratory - _cur_res.Get('gem_green')
        _ne_power = _need_power - _cur_res.Get('gem_blue')
        if _ne_oratory > 0:
            result.append( ("text", "COLOR_RED") )
            result.append(('text', 'ATTR_NOT_ENOUGH'))
            result.append( 'space' )
            result.append(('text', 'TIPS_ORATORY'))
            result.append( 'space' )
            result.append( ('digit', _ne_oratory) )
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
        if _ne_power > 0:
            result.append( ("text", "COLOR_RED") )
            result.append(('text', 'ATTR_NOT_ENOUGH'))
            result.append( 'space' )
            result.append(('text', 'TIPS_POWER'))
            result.append( 'space' )
            result.append( ('digit', _ne_power) )
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
# ============================================================
    def _get_gives(self, result):
        '''Возвращает текст для части типса GIVES'''
        _gives = self.completeGivesRes
        if _gives:
            if not _gives.Empty():
                _gives = _gives.Clone()
                _gives.FixFood()
                if not _gives.MainResEmpty():
                    result.append( 'newline' )
                    result.append( ("text", "COLOR_LINES") )
                    result.append( ("text", "GIVES") )
                    result.append( 'space' )
                    res_numbers = self._append_res_values(_gives)
                    result.extend( res_numbers )
# ============================================================
    def _append_res_values(self, resources):
        '''Для типса. Генерирует список со значениями переданного ресурса, с соответствующими изображениями'''
        result = []
        first_res = True
        for res_name in self.resource_names:    # выводим значения ресурсов...
            #trace('res: ', resources, res_name)
            res_val = resources.Get(res_name)
            if res_val > 0:     # на всякий случай
                if not first_res:
                    result.append( 'comma' )
                    result.append( 'space' )
                first_res = False
                result.append( ("digit", resources.Get(res_name)) )
                result.append( ("image", self.res_images[res_name]) )
        return result
# ============================================================
    def Get_tip_position(self, tip_size, camera_pos, window_rect, zoom):
        '''Возвращает координаты типса от левого-верхнего угла картинки типса'''
        return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect)
# ============================================================
#    def apply_image_subtype(self, img_group):
#        self.Drawer
# ============================================================
# ============================================================
    def get_unpacked_content(self, content_string):
        _preparts = content_string.split(';')
        _result = []
        for prepart in _preparts:
            if prepart:             # если не пустой текст
                parts = prepart.split(':')
                if parts[0]:        # если не пустой текст
                    _result.append(parts)
        return _result
# ============================================================
    def get_free_gift_cell(self):
        _cell_pos = self.get('cell')
        for i in range(1,6):
            _cell_name = 'gift' + str(i) + '_cell'
            _cell = self.GetAttrCell(_cell_name)
            _cell = self.FixAddCell(_cell_pos, _cell)
            _is_free = self.IsCellFree(_cell)
            if _is_free:
                return _cell
        mbox(to_str("ERROR: all cells are busy. UID:", str(self.uid), 'Name:', self.get('name'), 'Cell:', _cell_pos))
        return None
# ============================================================
    def publish_contents(self, contents, cell = None):
        '''Выкладывает на поле объекты'''
        #mbox(to_str(contents))
        if not contents:
            return []
        content_template, content_variant = '', ''
        result = []
        for content in contents:
            part0 = content[0]      # tinypy bug workaround
            content_parts = len(content)
            if content_parts == 2:
                content_template, content_variant = content
            elif content_parts == 1:
                content_template, content_variant = part0, ''
            else:
                continue
            if not cell:
                _cell = self.get_free_gift_cell()
            else:
                _cell = cell
            # - create item and place item on map in cell_pos
            if _cell:   # если есть такая ячейка
                _new_item = BMgr.AddItemToMap(content_template, content_variant, _cell)        # hack - BuildingsManager может добавлять на карту не только здания...
                if _new_item:
                    _pos = _new_item.GetAttrPoint('center_item_pos')
                    _new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
                    _new_item.play_content_sound(self)
                    result.append(_new_item)
        return result
# ============================================================
    def publish_contents_by_one(self, contents, cell = None):
        '''Выкладывает на поле объекты для показа по очереди (сам метод объекты не показывает, а только выкладывает)'''
        #mbox(to_str(contents))
        if not contents:
            return []
        content_template, content_variant = '', ''
        result = []
        for content in contents:
            part0 = content[0]      # tinypy bug workaround
            content_parts = len(content)
            if content_parts == 2:
                content_template, content_variant = content
            elif content_parts == 1:
                content_template, content_variant = part0, ''
            else:
                continue
            if not cell:
                _cell = self.get_free_gift_cell()
            else:
                _cell = cell
            # - create item and place item on map in cell_pos
            if _cell:   # если есть такая ячейка
                _new_item = BMgr.AddItemToMap(content_template, content_variant, _cell)        # hack - BuildingsManager может добавлять на карту не только здания...
                if _new_item:
                    #_pos = _new_item.GetAttrPoint('center_item_pos')
                    #_new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
                    _new_item.Drawer.Hide(True)  # скроем объекты
                    result.append(_new_item)
        return result
# ============================================================
    def play_content_sound(self, generator):
        '''Звук выпадающего из чего-либо объекта, определяется у самого объекта
        generator: объект, из которого выпадает ресурс'''
        pass
# ============================================================
    def show_attention_effect(self):
        '''Запускает эффект, привлекающий внимание к объекту'''
        self.need_attention_effect = True
        self.Drawer.RestartEffect('attention')
        self.switch_animation()
# ============================================================
    def show_born_effect(self):
        '''Запускает эффект, привлекающий внимание к объекту'''
        self.need_born_effect = True
        self.Drawer.RestartEffect('born')
        self.switch_animation()
# ============================================================
    def add_attention_signal(self, _from = 3000, _till = 7000):
        _time = tools.Rand(_from, _till)
        if _time < 300:
            mbox('attention_signal too often')  # dbg
        self.Signals.Restart('attention_signal', _time)
# ============================================================
    def stop_attention_signal(self):
        self.Signals.Cancel('attention_signal')
# ============================================================
    def is_enough_params(self):
        '''Достаточно ли красноречия и силы'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory or _have_power < _need_power:
            return False
        else:
            return True
# ============================================================
    def _get_not_enough_param_name(self):
        '''Название характеристики, которо не хватает'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory:
            return 'oratory'
        elif _have_power < _need_power:
            return 'power'
        else:
            return ''
# ============================================================
    def get_action_text(self, for_task_slide = False):
        return ''
# ============================================================
    def append_letter_resource(self, res_gives, res_text_id):
        '''Добавляет сообщение о подъеме ресурса генералом'''
        pass
        #_rcount = res_gives.GetSingleValue()
        #letter = []
        #letter.append( ('text', 'MSG_TAKEN_RES') )
        #letter.append('space')
        #letter.append( ('digit', _rcount) )
        #letter.append('space')
        #letter.append( ('text', res_text_id) )
        #EMgr.AppendLetter(letter, '')
# ============================================================
    def play_hit_sound(self):
        pass
# ============================================================
    def play_complete_sound(self):
        pass
# ============================================================
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.2)
# ============================================================
    def send_extra_event(self):
        if self.extra_event:
            BMgr.SendGlobalMsg('extra_event', self.extra_event)
            self.extra_event = ''
# ============================================================
    def get_action_cost_res(self):
        has_resname = self.HasAttr('acost_res_name')
        acost_res_name = 'action_cost'
        if has_resname:
            acost_res_name = self.GetAttrStr('acost_res_name')
        return self.GetRes(acost_res_name)
# ============================================================
    def activate_on_action_complete(self):
        if self.activate_uid:
            _item = BMgr.get_item(self.activate_uid)
            if _item:
                _item.activate()
                if self.activated_focus:
                    LBoard.MoveCameraToUID(_item.get('uid'), True)
            self.activate_uid = ''  # чтобы повторно нельзя было активировать
# ============================================================
    def do_action_enable(self):
        if not self.action_enable_complete:
            self.action_enable_complete = True
            self.switch_variant('enabled')
# ============================================================
    def Force_highlight(self, highlight):
        self.Drawer.HighlightImages(self.hlight_layers, highlight)
        #self.switch_animation()
# ============================================================
    def Force_colorize(self, color, highlight):
        '''color - without alpha part, only RGB. Alpha always 0xFF'''
        if not highlight:
            self.Drawer.ColorizeImages(self.hlight_layers, 0xffffff, False)
        else:
            self.Drawer.ColorizeImages(self.hlight_layers, color, False)
# ============================================================

