import glob
import md5
import os.path

import sys
if not "tinypy" in sys.version:
    from boot import *

import tokenize,parse,encode


CORE = ['tokenize','parse','encode','py2bc']

def build_bc(opt=False):
    out = []
    for mod in CORE:
	out.append("""unsigned char tp_%s[] = {"""%mod)
	fname = mod+".tpc"
	data = open(fname,'rb').read()
	cols = 16
	for n in xrange(0,len(data),cols):
	    out.append(",".join([str(ord(v)) for v in data[n:n+cols]])+',')
	out.append("""};""")
    out.append("")
    f = open('bc.c','wb')
    f.write('\n'.join(out))
    f.close()

def _compile(s,fname):
    tokens = tokenize.tokenize(s)
    t = parse.parse(s,tokens)
    r = encode.encode(fname,s,t)
    return r


files_md5 = {}
md5_filename = 'md5.txt'
def read_md5_file(path):
    global md5_filename
    global files_md5
    db = open(path + md5_filename, 'r')
    lines = db.readlines()
    db.close()
    for line in lines:
        _parts = line.split(' ')
        name, size, _md5 = _parts
        _size = int(size)
        files_md5[name] = (name, _size, _md5.strip())

def write_md5_file(path):
    global md5_filename
    global files_md5
    db = open(path + md5_filename, 'w')
    for k,v in files_md5.items():
        db.write('%s %d %s\n' % v)
    db.close()

def calc_file_md5(filepath):
    _file = open(filepath, 'rb')
    content = _file.read()
    _md5 = md5.new(content)
    _size = len(content)
    _hash = _md5.hexdigest()
    _file.close()
    return (filepath, _size, _hash)

#def create_md5_file(path, files):
#    global md5_filename
#    global files_md5
#    files_md5 = {}
#    db = open(path + md5_filename, 'w')
#    for filename in files:
#        _parts = calc_file_md5(filename)
#        db.write('%s %s %s\n' % _parts)
#        files_md5[filename] = _parts  # save md5 in dict
#    db.close()

def main():
    global md5_filename
    global files_md5
    #path = './'
    path = './../'
    files = glob.glob(path + '*.py')
    if os.path.exists(path + md5_filename): ## if no MD5 file, then create it
        read_md5_file(path)
        #create_md5_file(path, files)
    #else:
    #    read_md5_file(path)                     # else read it
    print 'Compiling started:'
    for module in files:
        if module.find("py2bc_all") == -1:      # skip self if started in same folder
            new_name = module[:-3] + '.tpc'
            _parts = calc_file_md5(module)
            if os.path.exists(module):          # if compiled module exist
                _fname, _size, _hash = _parts
                if module in files_md5:         # if file has calculated previous hash
                    _sname, _ssize, _shash = files_md5[module]
                    if _ssize == _size and _shash == _hash: # if hash the same - using old file
                        # print module, '--> NO CHANGES'
                        continue        # to next file
            files_md5[module] = _parts  # saving hash
            print module, "-->", new_name
            s = load(module)
            r = _compile(s, module)
            save(new_name, r)
    print 'Complete'
    write_md5_file(path)            # saving hash db
    from datetime import datetime
    now = datetime.now()
    print 'Time - %02d:%02d:%02d' % (now.hour, now.minute, now.second)
    print 'Creating bc.h'
    build_bc()
    
    return

if __name__ == '__main__':
    main()
