# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes


class BreakPotSubtask(task_classes.Subtask):
    def __init__(self, parent_task, actor):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.actor_action_animation = 'hit'

    def start_action(self, item):
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True
# ============================================================
# ============================================================
class Pot(base_diff_item.DiffItem):
    '''Горшок'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.task = None
        self.like_tree_mode = self.GetAttrBool('like_tree')
        if self.like_tree_mode:
            self.Drawer.SetTreeMode('default', 100)
        self.like_bush_mode = self.GetAttrBool('like_bush')
        if self.like_bush_mode:
            self.Drawer.SetTreeMode(['default','growth'], 50)
        self.like_stalagmite = self.GetAttrBool('like_stalagmite')
        self.hit_sound = self.GetAttrStr('hit_sound')
        self.broken_sound = self.GetAttrStr('broken_sound')
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        content_template_name, variant_name = self.get_content_ex()
        if not content_template_name:
            content_template_name = self.get_content()      # имя шаблона объекта внутри горшка
            variant_name = self.GetAttrStr('contains_variant_name')
        self.switch_variant( 'broken_' + str(tools.Rand(1,3)) )    # broken_1, broken_2, broken_3
        self.action_hits_last = self.GetAttrInt('action_hits_count')   # уже после смены варианта
        pot_info = {"type":self.get('type'), "subtype":self.get('subtype')}
        EMgr.send_event( 'pot_broken', self, pot_info )
        self.publish_content(content_template_name, variant_name)
        if self.broken_sound:
            self.PlaySound(self.broken_sound)
        elif self.like_tree_mode:
            self.PlaySound('SND_CHOP_TREE_FALL')
        elif self.like_bush_mode:
            self.PlaySound('SND_CHOP_BUSH_DISAPPEAR')
        elif self.like_stalagmite:
            self.PlaySound('SND_STALACTITE_CRASH')

        if not self.like_tree_mode:
            if not self.like_bush_mode:
                if not self.like_stalagmite:
                    if self.subtype == 'burrel':
                        UStats.AddLevelValue('broken_barrels', 1)
                        if not self.broken_sound: self.PlaySound('SND_BARREL_CRASH')
                    else:
                        UStats.AddLevelValue('broken_pots', 1)
                        if not self.broken_sound: self.PlaySound('SND_POT_CRASH')
        self.send_extra_event()
        BMgr.remove_uid(self.get('uid'))
# ============================================================
    def get_content_ex(self):
        '''Контент из "content"'''
        _content = self.get_unpacked_content( self.GetAttrStr('content') )  # строго до смены варианта
        if _content:
            _item = _content[0]
            _part0 = _item[0]
            _item_parts = len(_item)
            if _item_parts == 1:
                return _part0, ''
            else:
                return _item
        return (None, None)
# ============================================================
    def get_content(self):
        '''Возвращает имя шаблона содержимого горшка, если в нем что-то есть'''
        content_template_name = self.GetAttrStr('contains_template')
        if not content_template_name:
            if self.GetAttrBool('contains_gold_small'):
                return 'gold_small'
            elif self.GetAttrBool('contains_gold_big'):
                return 'gold_big'
        return content_template_name
# ============================================================
    def publish_content(self, content_template_name, variant_name):
        '''Выкладывает на поле объект'''
        if not content_template_name:
            return
        cell_pos = self.get('cell')
        # - create item and place item on map in cell_pos
        _new_item = BMgr.AddItemToMap(content_template_name, variant_name, cell_pos)        # hack - BuildingsManager может добавлять на карту не только здания...
        if _new_item:
            _pos = _new_item.GetAttrPoint('center_item_pos')
            _new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
            _new_item.play_content_sound(self)
# ============================================================
    def get_subtask(self, parent_task, actor):
        return BreakPotSubtask(parent_task, actor)
# ============================================================
    def play_hit_sound(self):
        if self.hit_sound:
            self.PlaySound(self.hit_sound)
        elif self.like_tree_mode:
            self.PlaySound('SND_CHOP_TREE')
        elif self.like_bush_mode:
            self.PlaySound('SND_CHOP_BUSH')
        elif self.like_stalagmite:
            self.PlaySound('SND_STALACTITE_HIT')
        elif self.subtype == 'burrel':
            self.PlaySound('SND_BARREL_HIT')
        else:
            self.PlaySound('SND_POT_HIT')
# ============================================================
    #@override
    def get_action_text(self, for_task_slide = False):
        return 'DLG_TASK_BREAK_POT'
# ============================================================
    def scale_bump(self):
        if not self.like_tree_mode:
            base_diff_item.DiffItem.scale_bump(self)
        else:
            _pos = self.GetAttrPoint('center_item_pos')
            self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.1)
# ============================================================
    #@override
    def get_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        _cell_names = []
        for i in range(1,9):
            _name = 'action_cell' + str(i)
            _has_attr = self.HasAttr(_name)
            if _has_attr:
                _cell_names.append(_name)
        _cell_names = self._filter_available_cells(_cell_names)
        _len = len(_cell_names)
        if _len > 0:
            _rnd = tools.Rand(0, _len-1)
            return _cell_names[_rnd]
        else:
            return 'action_cell'
# ============================================================
# #######################################################
# #######################################################
# #######################################################
