# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import dialog
import toolz

class BuildingsDlg(dialog.Dialog):
    '''Диалог постройки зданий'''
    resource_names = ["food", "bricks", "boards", "coins"]
    res_images = {
        "food": "IMG_I_TIP_FOOD",
        "bricks": "IMG_I_TIP_BRICKS",
        "boards": "IMG_I_TIP_BOARDS",
        "coins": "IMG_I_TIP_COINS",
        "gold": "IMG_I_TIP_GOLD"
        }
# ============================================================
    def __init__(self, cdata, params={}):
        dialog.Dialog.__init__(self, cdata, params)
        self.size = 'big'
        self.has_bridge = False
        self.templates = []
        self.names_templates = {}
        self.stats = None
        self.was_click = False  # чтобы менять описание по клику для недоступных для строительства
        self.res_on_create = None   # ресурсы, которые были при открытии диалога
        self.build_task_template = ''
        self.selected_building_tag = ''
        # если нужно, то доп. параметры можно вытащить из params
# ============================================================
    def on_signal(self, signal):
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == "enter":
            pass
        elif type == "exit":
            pass
        elif type == "change":
            self.on_state_change(additional.prev_state, additional.next_state)
        pass
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def on_dlg_command(self, tag, command):
        pass
# ============================================================
    def on_command(self, name, command, param):
        if command == 'keycode':
            if param == 'escape':
                self.ShowFakeBG( False )
                DLG.Close_dialog(self)
# ============================================================
    sort_list = ['garden1', 'fisherman1', 'smokery1', 'farm1', 'bakery1', 'woodcutter1',\
                'sawmill1', 'watchtower1', 'warehouse1', 'house1', 'granitemine1',\
                'stonecutter1', 'quarry1', 'silvermine1', 'mint1',\
                'bridge_1', 'bridge_2', 'bridge_3', 'bridge_4',\
                'bridge_sus_1', 'bridge_sus_2', 'bridge_sus_3', 'bridge_sus_4' ]
    def on_create(self):
        '''Метод, вызывается для инициализации диалога данными'''
        tools.PlaySound('SND_WINDOW_OPEN')
        self.ShowFakeBG( True )
        self.stats = self.GetFullStats()
        self.res_on_create = DLG.GetCurrentRes()
        self.build_task_template = self.GetBuildTaskTemplate()
        self.SetImage('idNoBuildingImage', 'IMG_I_BUILDING_BACK_EMPTY')
        _templates = DLG.GetBuildableList()
        # >>> сортируем список зданий =========
        self.names_templates = {}
        templates = []
        for item in _templates: self.names_templates[item.name] = item
        for name in self.sort_list:
            if name in self.names_templates:
                templates.append(self.names_templates[name])
        if not self.sort_list: templates = _templates
        # <<< =================================

        _index = 0
        _is_flipped_item = False
        for item in templates:
            if item.is_visible:
                if item.subtype == 'bridge':
                    if self.has_bridge: continue
                    else: self.has_bridge = True
                _list_name = 'idList1'
                if _index >= 8:
                    _list_name = 'idList2'
                    _is_flipped_item = True
                # нужно прилепить эффект: ========= >>>
                if item.name == self.build_task_template:   # если это шаблон квеста
                    _arrow_index = _index
                    _p = {'x':0, 'y':89}
                    _p.x += self.GetRTdx();
                    _p.y += self.GetRTdy();
                    if _arrow_index < 8:
                        _p.x += 52
                        _p.y += 60 + _arrow_index * 52 + 25
                        self.ShowXmlEffect('arrow_right', _p)
                    else:
                        _arrow_index -= 8
                        _p.x += 629
                        _p.y += 60 + _arrow_index * 52 + 25
                        self.ShowXmlEffect('arrow_right', _p)
                # <<< =================================
                _can_build = item.can_build
                _is_prior = self.ULI_CanBuildPriority(item.name)
                item.cannot = ''
                if _can_build:
                    #if 'barbarian_attack' in self.stats:
                    #    if self.stats.barbarian_attack > 0:
                    #        item.cannot = 'BARBARIAN'
                    #        _can_build = False
                    # #_has_priorities = self.ULI_HasPriorities()
                    # #if _has_priorities:
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    if item.depends_from:
                        if _can_build:
                            for dep in item.depends_from:
                                _has = self.HasActiveBuilding(dep)
                                _can_build = False
                                item.cannot = 'NO_DEPENDANT'
                                if _has:
                                    item.cannot = ''
                                    _can_build = True
                                    break
                    if _can_build:
                        #_is_exception = self.IsException(item.name)
                        _has_builders = self.HasEnoughBuilders(item.name)
                        _has_mplaces = self.HasActiveMapPlaces(item.name, item.subtype)
                        if not _has_builders:# and not _is_exception:
                            _can_build = False
                            item.cannot = 'NO_BUILDERS'
                        elif not _has_mplaces:
                            _can_build = False
                            _has_deactive_mplaces = self.HasOnlyDeactivatedMapPlaces(item.name, item.subtype)
                            item.cannot = ('NO_MAP_PLACE', 'NO_ACTIVE_MAP_PLACE')[_has_deactive_mplaces == True]
                        if item.cannot == '':
                            if item.name == 'quarry1':
                                _has_granite = self.HasGranite()
                                if not _has_granite:
                                    _can_build = False
                                    item.cannot = 'NO_GRANITE'
                else:
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    else:
                        item.cannot = 'CANNOT_BUILD'    # from C++
                trace(_list_name, item.name, item.cannot, _can_build)
                self.ListAddBuilding(_list_name, item.name, _can_build, _is_flipped_item)   # здания, которые можно строить
                self.templates.append(item)
                _index += 1
        #HasActiveBuilding('name')
# ============================================================
    def on_button(self, id):
        if id == "idCancelBtn":
            self.close_me()
        elif id == "idBuildBtn":
            if self.TryReserveRes(self.selected_building_tag, 'level_1', 'normal_needs_1'):
                DLG.Allow_single_dialogs(False)
                DLG.SendMsg('switch_to_build_mode', self.selected_building_tag)
                self.ShowFakeBG(False)
                DLG.Close_dialog(self)
        pass
# ============================================================
    def close_me(self):
        self.ShowFakeBG( False )
        DLG.Close_dialog(self)
# ============================================================
    def on_list_click(self, id, tag, index, extra):
        '''Вызывается при клике по элементу списка'''
        #mbox(str(extra['is_res_enough']))
        #can_build_now = extra.can_build_now
        is_res_enough = extra.is_res_enough
        if tag in self.names_templates:
            item = self.names_templates[tag]
            _cannot_empty = (len(item.cannot) == 0)
            _is_default = True
            if not is_res_enough: _is_default = False
            if not _cannot_empty: _is_default = False
            if _is_default:
                self.selected_building_tag = tag
                    #if self.TryReserveRes(tag, 'level_1', 'normal_needs_1'):    # сразу же резервируем стоимость здания
                    #DLG.Allow_single_dialogs(False)                 # чтобы можно было строить строить здание, а диалог ждал
                    #DLG.SendMsg('switch_to_build_mode', tag)                # просим переключиться в режим строительства
                    #self.ShowFakeBG( False )
            #DLG.Close_dialog(self)                                   # ну и закрываем свой диалог
            else:
                self.was_click = True
                self.SetImage('idRedBGImage', 'IMG_I_BUILDING_NB_BACK')
                self.SetTextByTemplate('idItemName', tag, 'item_name', 'COLOR_YELLOW')
                if item.cannot == 'OTHER_PRIORITY':
                    self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_CANNOT_BUILD_YET')])
                elif item.cannot == 'NO_BUILDERS':
                    self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_WORKERS')])
                #elif item.cannot == 'BARBARIAN':
                #    self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','TIPS_CANNOT_BUILD_BARB')])
                elif item.cannot == 'CANNOT_BUILD_YET':
                    self.SetText('idItemDesc', [('text', 'COLOR_WHITE'), ('text', 'SHOP_DLG_CANNOT_BUILD_YET')])
                elif item.cannot == 'NO_MAP_PLACE':
                    if item.subtype in ['watchtower', 'fisherman', 'bridge']:
                        self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_MAP_PLACES')])
                    else:
                        self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_MAP_PLACES')])
                elif item.cannot == 'NO_ACTIVE_MAP_PLACE':
                    if item.subtype in ['watchtower', 'fisherman', 'bridge']:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_MAP_PLACES')])
                        self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_ACTIVE_MAP_PLACES')])
                    else:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_MAP_PLACES')])
                        self.SetText('idItemDesc', [('text','SHOP_DLG_NO_ACTIVE_MAP_PLACES')])
                elif item.cannot == 'NO_DEPENDANT':
                    dependant_names = []
                    need_or = False
                    for dep in item.depends_from:
                        if need_or:
                            dependant_names.append('space')
                            dependant_names.append( ('text', 'SHOP_DLG_OR') )
                            dependant_names.append('space')
                        name = DLG.GetTemplateAttr(dep, 'item_name')
                        dependant_names.append(('text', name))
                        need_or = True
                    _result = [('text','COLOR_DESCRIPTION'), ('text','SHOP_DLG_NEED'),\
                                                'newline',\
                                                ('text','COLOR_WHITE')]
                    _result.extend(dependant_names)
                    self.SetText('idItemDesc', _result)
                elif not is_res_enough:
                    # нужно перенести в on_create, и кол-во ресурсов считать только один раз
                    need_res = DLG.GetTemplateRes(item.name, 'normal_needs_1')
                    if need_res:
                        diff_res = self.res_on_create.GetNotEnoughRes(need_res)
                        _result = [('text','COLOR_DESCRIPTION'), ('text','SHOP_DLG_NEED_RES'),'newline']
                        not_enough_res = self._append_res_values(diff_res)
                        _result.append( ('text','COLOR_WHITE') )
                        _result.extend(not_enough_res)
                        self.SetText('idItemDesc', _result)
                    else:
                        self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NEED_RES')])   # чтобы не упало
                elif item.cannot == 'NO_GRANITE':
                    self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_GRANITE')])
                else:
                    _result = [('text','_cannot_empty'), ('text','SHOP_DLG_CANNOT_BUILD_YET')]
                    #_result.append( ('text','COLOR_WHITE') )
                    self.SetText('idItemDesc', _result)
# ============================================================
    def set_cannot_data(self, tag, index, extra, result):
        '''Выставляет данные для объекта, который нельзя строить
        returns: False if build possible'''
        is_res_enough = True
        #can_build_now = False
        if 'is_res_enough' in extra:
            is_res_enough = extra.is_res_enough
        #if 'can_build_now' in extra: can_build_now = extra.can_build_now
        if tag in self.names_templates:
            item = self.names_templates[tag]
            _cannot_empty = (len(item.cannot) == 0)
            _is_default = True
            if not is_res_enough: _is_default = False
            if not _cannot_empty: _is_default = False
            if _is_default:
                #if self.TryReserveRes(tag, "level_1", "normal_needs_1"):    # сразу же резервируем стоимость здания
                #    DLG.Allow_single_dialogs(False)                 # чтобы можно было строить строить здание, а диалог ждал
                #    DLG.SendMsg("switch_to_build_mode", tag)                # просим переключиться в режим строительства
                #    DLG.Close_dialog(self)                                   # ну и закрываем свой диалог
                return False
            else:
                self.SetImage('idRedBGImage', 'IMG_I_BUILDING_NB_BACK')
                #self.SetTextByTemplate('idItemName', tag, 'item_name', 'COLOR_BROWN')
                result.append('newline')
                if item.cannot == 'OTHER_PRIORITY':
                    #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_CANNOT_BUILD_YET')])
                    result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_CANNOT_BUILD_YET')])
                elif item.cannot == 'NO_BUILDERS':
                    result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_WORKERS')])
                #elif item.cannot == 'BARBARIAN':
                #    result.extend([('text','COLOR_WHITE'), ('text','TIPS_CANNOT_BUILD_BARB')])
                elif item.cannot == 'CANNOT_BUILD_YET':
                    result.extend([('text', 'COLOR_WHITE'), ('text', 'SHOP_DLG_CANNOT_BUILD_YET')])
                elif item.cannot == 'NO_MAP_PLACE':
                    if item.subtype in ['watchtower', 'fisherman', 'bridge']:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_MAP_PLACES')])
                        result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_MAP_PLACES')])
                    else:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_MAP_PLACES')])
                        result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_MAP_PLACES')])
                elif item.cannot == 'NO_ACTIVE_MAP_PLACE':
                    if item.subtype in ['watchtower', 'fisherman', 'bridge']:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_MAP_PLACES')])
                        result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_OTHER_ACTIVE_MAP_PLACES')])
                    else:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_MAP_PLACES')])
                        result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_ACTIVE_MAP_PLACES')])
                elif item.cannot == 'NO_DEPENDANT':
                    dependant_names = []
                    need_or = False
                    for dep in item.depends_from:
                        if need_or:
                            dependant_names.append('space')
                            dependant_names.append( ('text', 'SHOP_DLG_OR') )
                            dependant_names.append('space')
                        name = DLG.GetTemplateAttr(dep, 'item_name')
                        dependant_names.append(('text', name))
                        need_or = True
                    _result = [('text','COLOR_DESCRIPTION'), ('text','SHOP_DLG_NEED'),\
                                                'newline',\
                                                ('text','COLOR_WHITE')]
                    _result.extend(dependant_names)
                    #self.SetText('idItemDesc', result)
                    result.extend(_result)
                elif not is_res_enough:
                    # нужно перенести в on_create, и кол-во ресурсов считать только один раз
                    need_res = DLG.GetTemplateRes(item.name, 'normal_needs_1')
                    if need_res:
                        diff_res = self.res_on_create.GetNotEnoughRes(need_res)
                        _result = [('text','COLOR_DESCRIPTION'), ('text','SHOP_DLG_NEED_RES'),'newline']
                        not_enough_res = self._append_res_values(diff_res)
                        _result.append( ('text','COLOR_WHITE') )
                        _result.extend(not_enough_res)
                        #self.SetText('idItemDesc', result)
                        result.extend(_result)
                    else:
                        #self.SetText('idItemDesc', [('text','COLOR_WHITE'), ('text','SHOP_DLG_NEED_RES')])   # чтобы не упало
                        result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NEED_RES')])    # чтобы не упало
                elif item.cannot == 'NO_GRANITE':
                    result.extend([('text','COLOR_WHITE'), ('text','SHOP_DLG_NO_GRANITE')])
                else:
                    _result = [('text','COLOR_WHITE'), ('text','SHOP_DLG_CANNOT_BUILD_YET')]
                    result.extend(_result)
            return True
        return False
# ============================================================
    def on_list_selection_change(self, id, new_tag, new_index, old_index, extra):
        if new_index >= 0:
            _groups = ['floor', 'default', 'anim', 'unit']
            self.was_click = False  # при смене элемента сбросим
            self.SetImage('idRedBGImage', '')
            self.SetImage('idNoBuildingImage', '')
            _bush = []
            if new_tag == 'garden1':
                _bush = ['bush'+str(i) for i in range(1,7)]
                _groups.extend(_bush)
                _groups.append('gather3')
            elif new_tag == 'watchtower1':
                _groups.append('worker_inside')
            elif new_tag == 'farm1':
                _groups.append('sleep_unit')
            if new_tag in self.names_templates:
                item = self.names_templates[new_tag]
                is_res_enough = True
                if 'is_res_enough' in extra: is_res_enough = extra.is_res_enough
                self.SetTemplate('idBuildingImage', new_tag, '', _groups, _bush, 90)
                if is_res_enough and len(item.cannot) == 0:
                    self.SetTextByTemplate('idItemName', new_tag, 'item_name', 'COLOR_YELLOW')
                    #self.SetTextByTemplate('idItemDesc', new_tag, 'item_description', 'COLOR_DESCRIPTION')
                    result = [('text', 'COLOR_DESCRIPTION')]
                    result.append( ('text', self.GetTextIDFromTemplate(new_tag, 'item_description')) )
                else:
                    self.SetTextByTemplate('idItemName', new_tag, 'item_name', 'COLOR_YELLOW')
                    result = [('text', 'COLOR_DESCRIPTION')]
                    result.append( ('text', self.GetTextIDFromTemplate(new_tag, 'item_description')) )
                    self.set_cannot_data(new_tag, new_index, extra, result)
                self.SetText('idItemDesc', result)
        else:
            self.SetTemplate('idBuildingImage', '', '')
            self.SetText('idItemName', '')
            self.SetText('idItemDesc', '')
            self.was_click = False  # при выходе за список сбросим тоже
            self.SetImage('idRedBGImage', '')
            self.SetImage('idNoBuildingImage', 'IMG_I_BUILDING_BACK_EMPTY')
        if new_index >= 0:
            tools.PlaySound('SND_BTN_OVER_PAPER')
        pass
# ============================================================
    def on_update(self, ms):
        pass
# ============================================================
    def _append_res_values(self, resources):
        '''Для типса. Генерирует список со значениями переданного ресурса, с соответствующими изображениями'''
        result = []
        if not resources: return result
        first_res = True
        for res_name in self.resource_names:    # выводим значения ресурсов...
            res_val = resources.Get(res_name)
            if res_val > 0:     # на всякий случай
                if not first_res:
                    result.append( 'comma' )
                    result.append( 'space' )
                first_res = False
                result.append( ('digit', resources.Get(res_name)) )
                result.append( ('image', self.res_images[res_name]) )
        return result
# ============================================================
# ============================================================
# ============================================================
# ============================================================
