# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

class GameHelper(object):
    #resource_names = ["food", "bricks", "boards", "coins"]
    #res_images = {
    #    "food": "IMG_I_TIP_FOOD",
    #    "bricks": "IMG_I_TIP_BRICKS",
    #    "boards": "IMG_I_TIP_BOARDS",
    #    "coins": "IMG_I_TIP_COINS"
    #    }
    PAUSE_TIME = 10000
# ============================================================
    def __init__(self):
        self.helper_item_uid = ''        # UID последнего сгенерированного объекта
        self.helper_bricks_item_uid = ''        # UID последнего сгенерированного объекта
        self.units_timer_started = False
        self.units_timer_time = 0
        self.units_msg_shown_timer = 0
        self.ticks = 0
        self.level_settings = None
        self.repeat_time = Globals.GInt('REPEAT_UNITS_MSG_TIME')
        self.gold_enough = False
        self.pause_time = self.PAUSE_TIME     # время, на которое паузится подбрасывание древесины
        self.bricks_pause_time = self.PAUSE_TIME     # время, на которое паузится подбрасывание кирпичей
# ============================================================
    def on_create(self):
        self.level_settings = BMgr.GetLevelSettings()
        trace('Level settings:', self.level_settings)
# ============================================================
    def on_update(self, ms):
        '''Вызывается каждый второй quant! См. BaseManager.cpp::Update -> mgr_t.on_update'''
        if self.ticks > 1000000: self.ticks = 0
        self.ticks += 1
        if self.helper_item_uid == '':
            if self.pause_time > 0: self.pause_time -= ms   # для паузы генерации досок
        if self.helper_bricks_item_uid == '':
            if self.bricks_pause_time > 0: self.bricks_pause_time -= ms   # для паузы генерации досок
        if not (self.ticks % 15):   #! проверяем каждые 0.333 секунды (10)
            if self.pause_time <= 0:
                if self.helper_item_uid == '':
                    self.try_generate_boards()
        if not (self.ticks % 15):   #! проверяем каждые 0.333 секунды (10)
            if self.bricks_pause_time <= 0:
                if self.helper_bricks_item_uid == '':
                    self.try_generate_bricks()
        # проверяем кол-во юнитов:
        if not (self.ticks % 10):       #! проверяем каждые 0.2 секунды (4)
            self.check_units_count(ms*10)
        if not (self.ticks % 50):   # раз в секунду
            _is_main_level = LBoard.Core.is_main_level()
            if _is_main_level:
                if not self.gold_enough:
                    if self.level_settings != None:
                        res = UStats.GetCurRes()
                        _gold = res.Get('gold')
                        _need_gold = int(self.level_settings.need_gold)
                        if _gold >= _need_gold and _need_gold > 0:
                            self.gold_enough = True
                            letter = [ ('text', 'MSG_GOLD_GATHERED') ]
                            EMgr.AppendLetter(letter, '')
        pass
# ============================================================
    def check_units_count(self, ms):
        '''Проверяет - достаточно ли юнитов, если нет, то выводит подсказку о домиках'''
        if BMgr.CanBeBuilded('house1'):     # если домики уже можно строить
            if not UMgr.is_free_real_units_enough():
                self.units_timer_started = True
            else:
                self.units_timer_started = False
                self.units_timer_time = 0
                self.units_msg_shown_timer = 0
            if self.units_timer_started:
                self.units_timer_time += ms
            if self.units_timer_time >= 17000:    # 17 сек (полсекунды, чтобы не ноль)
                if self.units_msg_shown_timer > 0 and self.units_msg_shown_timer < self.repeat_time:
                    self.units_msg_shown_timer += ms
                elif self.units_msg_shown_timer <= 0:
                    # показать подсказку
                    if self.level_settings:
                        has_house_in_building = BMgr.Has_buildings_in_build('house1')
                        if not has_house_in_building:
                            if 'level_type' in self.level_settings:
                                if self.level_settings.level_type != 'underlevel':
                                    UI.show_top_text('HELPER_UNITS', '', True)
                            else:
                                UI.show_top_text('HELPER_UNITS', '', True)
                    self.units_msg_shown_timer += ms
                else:
                    self.units_msg_shown_timer = 0
                    self.units_timer_time = 0
# ============================================================
    def on_event(self, event, extra):
        if event == 'unit_res_taken':
            if self.helper_item_uid == extra.uid:
                self.helper_item_uid = ''
            elif self.helper_bricks_item_uid == extra.uid:
                self.helper_bricks_item_uid = ''
# ============================================================
    wood_help_shown = False     # была ли подсказка на первые подброшенные дрова?
    def try_generate_boards(self):
        #if not BMgr.CanBeBuilded('woodcutter1'):    # если нельзя его строить, то
        #    return                                 # сразу выходим

        #if self.helper_item_uid:
        #    _item = BMgr.get_item(self.helper_item_uid)
        #    if _item:    # если сгенерированный элемент еще на поле
        #        return
        # ---
        self.helper_item_uid = ''
        _woodcutters = BMgr.get_buildings_simple(['woodcutter'])
        for b in _woodcutters:
            _has_access = b.HasAccess() # если есть доступные/незаброшенные лесорубы
            if _has_access:   # если здание доступно
                if not b.lock_shown:
                    return
        self.pause_time = self.PAUSE_TIME
        _total = UStats.GetCurRes()
        _total_boards = _total.Get('boards')    # сколько всего древесины
        if _total_boards < 15:
            self.generate_boards()  # генерим ресурс
# ============================================================
    def get_maximum_index_item(self, _generators):
        pass
# ============================================================
    def generate_boards(self):
        self.pause_time = self.PAUSE_TIME
        _generators = BMgr.get_items_by_subtype('boards_helper', False, True)
        _gen_count = len(_generators)
        if _gen_count > 0:
            _max_index = -1
            _generator = None
            for _g in _generators:
                _index = _g.GetAttrInt('index')
                if _index > _max_index:
                    _max_index = _index
                    _generator = _g
            #index = tools.Rand(0, _gen_count-1)
            #_generator = _generators[index]
            _items = _generator.generate_res()
            if _items:
                _item = _items[0]
                self.helper_item_uid = _item.get('uid')
                if not self.wood_help_shown:
                    if LBoard.episode <= 1:
                        UI.show_top_text('WOOD_HELPER_GENERATED', _generator.get('uid'))
                    elif LBoard.episode == 2:
                        UI.show_top_text('WOOD_HELPER_GENERATED_2', _generator.get('uid'))
                    self.wood_help_shown = True
                    Camera.MoveTo(_item)
                    _item.show_arrow()
# ============================================================
    tower_states = {'build':0, 'upgrade':0, 'repair':0, 'awaiting':0}
    def _has_upgrading_towers(self):
        for tower in BMgr.all_buildings:
            _stype = tower.get('stype')
            if _stype == 'watchtower':
                if tower.status != 'map_place':
                    _is_active_raw = tower.is_active_raw()
                    if _is_active_raw:
                        #if tower.status == 'broken':
                        if tower.clock_shown:
                            return True
                        t_state = tower.SM.GetState()
                        if t_state in self.tower_states:
                            return True
        return False
# ============================================================
    bricks_help_shown = False     # была ли подсказка на первые подброшенные кирпичи?
    def try_generate_bricks(self):
        # закомментил, потому как на начало уровня строить действительно еще нельзя, но подкидывание уже нужно
        #if not BMgr.CanBeBuilded('granitemine1'):    # если нельзя его строить, то
        #    return                                 # сразу выходим
        #if self.helper_bricks_item_uid and BMgr.get_item(self.helper_bricks_item_uid):    # если сгенерированный элемент еще на поле
        #    return
        # ---
        self.helper_bricks_item_uid = ''
        _items = BMgr.get_buildings_simple(['granitemine'])
        for b in _items:
            _has_access = b.HasAccess()
            if _has_access:   # если здание доступно
                return
        _items = BMgr.get_items_by_subtype('granite')
        _has_granites = False
        for b in _items:
            _has_access = b.HasAccess()
            if _has_access:   # если ресурс доступен
                _has_granites = True
        if _has_granites:   # если есть ресурсы, то нужно проверить наличие quarry
            _items = BMgr.get_buildings_simple(['quarry'])
            for b in _items:
                _has_access = b.HasAccess()
                if _has_access:   # если здание доступно
                    if not b.lock_shown:    # и не заброшено
                        return
        self.bricks_pause_time = self.PAUSE_TIME
        _total = UStats.GetCurRes()
        _total_bricks = _total.Get('bricks')    # сколько всего древесины
        if _total_bricks < 40:
            self.generate_bricks()  # генерим ресурс
# ============================================================
    def generate_bricks(self):
        self.bricks_pause_time = self.PAUSE_TIME * 2    # дольше проверка должна быть
        _generators = BMgr.get_items_by_subtype('bricks_helper', False, True)
        _gen_count = len(_generators)
        if _gen_count > 0:
            _has_up_towers = self._has_upgrading_towers()   # если башня ремонтируется или апгрейдится, то камни не подбрасываем
            if _has_up_towers:
                return
            _max_index = -1
            _generator = None
            for _g in _generators:
                _index = _g.GetAttrInt('index')
                if _index > _max_index:
                    _max_index = _index
                    _generator = _g
            #index = tools.Rand(0, _gen_count-1)
            #_generator = _generators[index]
            _items = _generator.generate_res()
            if _items:
                _item = _items[0]
                self.helper_bricks_item_uid = _item.get('uid')
                if not self.bricks_help_shown:
                    if LBoard.episode <= 1:
                        UI.show_top_text('BRICKS_HELPER_GENERATED', _generator.get('uid'))
                    elif LBoard.episode == 2:
                        UI.show_top_text('BRICKS_HELPER_GENERATED_2', _generator.get('uid'))
                    self.bricks_help_shown = True
                    Camera.MoveTo(_item)
                    _item.show_arrow()
# ============================================================
# ============================================================
# ============================================================
# #######################################################
# #######################################################
# #######################################################
