# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_build
import task_build_place
# ============================================================
# ============================================================
class BuildingsContainer(object):
    def __init__(self):
        self.arrays = {}

    def append(self, building):
        if not building: return
        stype = building.get('stype')
        if stype not in self.arrays:
            self.arrays[stype] = []
        self.arrays[stype].append(building)

    def remove(self, building):
        if not building: return
        stype = building.get('stype')
        if stype not in self.arrays:
            return
        self.arrays[stype].remove(building)

    def get(self, stype):
        if not stype: return []
        if stype not in self.arrays:
            return []
        return self.arrays[stype]
# ============================================================
# ============================================================
# ============================================================
# ============================================================
class BuildingsMgr(BuildingsMgr_class):
    all_buildings = []
    FBC = None
    all_units = []
    all_items = []
    uids_dict = {}
    nests = []
# ============================================================
    def __init__(self, cdata):
        BuildingsMgr_class.__init__(self, cdata)
        self.FBC = BuildingsContainer()
# ============================================================
    def on_msg(self, msg_type, msg_param1="", msg_param2="", msg_dict={}):
        if msg_type == "on_create":
            self.on_create()
        elif msg_type == "state_machine":
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == "signal_msg":
            self.on_signal(msg_param1)
        elif msg_type == "test":
            mbox("test succesful")
        elif msg_type == 'refresh_next_res': # аналогично реагируем и при изменении ресуров в on_event
            # обновить переменные всех зданий
            for b in self.all_buildings:
                b.SendMsg('refresh_next_res')
# ============================================================
    def on_signal(self, signal):
        if signal == 'exit_menu':
            dlg.Open('ExitMenu', 'dlg_exit')
        elif signal == 'complete_init':
            self.CompleteInitialize()   #@!!! если время где-то затормозится, то пиши пропало!
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == "enter":
            return
        elif type == "exit":
            return
        elif type == "change":
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_change(self, cur_state, next_state):
        #trace(cur_state.append(next_state))
        return
# ============================================================
    def Add_build_item(self, template_name, cell):
        TMgr.add_task( task_build.BuildingTask("build_building", template_name, "place", cell ) )
        UStats.ApplyReserveRes()
# ============================================================
    def Build_fixed_item(self, map_place_element):
        TMgr.add_task( task_build_place.BuildingPlaceTask("build_building", map_place_element ) )
        UStats.ApplyReserveRes()
# ============================================================
    def cancel_build_mode(self):
        UStats.UndoReserveRes()
# ============================================================
    def on_create(self):
        trace('BuildingsManager on_create')
        EMgr.add_event_listener(self)
        self.nests = []
        for item in self.all_items: # выгребаем все гнезда варваров
            _stype = item.get('stype')
            if _stype == 'nest':
                self.nests.append(item)
# ============================================================
    def on_event(self, name, obj, extra):
        '''Обработчик игровых событий, позже перенести в LevelBoard'''
        if name == 'game_win':
            # отменить режим строительства если включен
            dlg.Open('StatsMenu', 'dlg_stats', {'single_mode':'1', 'final':''})
        elif name == 'resources_changed':
            # обновить переменные всех зданий
            for b in self.all_buildings:
                b.SendMsg('refresh_next_res')
        pass
# ============================================================
    def on_update(self, ms):                # данный метод должен вызываться из C++ каждый квант
        pass
# ============================================================
    def on_rare_update(self, ms):
        for nest in self.nests:
            nest.on_rare_update(ms)
# #######################################################
# #######################################################
# #######################################################
    def find_nearest_unit_building(self, target, cell_shift_name = "", with_units = True, vacant_only = False):
        buildings = self.GetHomeBuildings(with_units, vacant_only) # true - with free units
        pathes = []
        cells = []
        for b in buildings:
            cells.append(b.start_cell_pos)
        result = BMgr.FindNearestCell(target, cell_shift_name, cells)
        if result:  # найдем здание по ячейке
            for b in buildings:
                _b_cell = b.start_cell_pos
                if self.__cells_equal(_b_cell, result):
                    return b
        trace('find_nearest_unit_building: no pathes')
        return None

        #for b in buildings:
        #    map_path = self.CalcPathTo(b.start_cell_pos, target, cell_shift_name, True) # считаем путь от одного из домиков до объекта
        #    if map_path.is_path_exist:
        #        pathes.append( (map_path,b) )
        #if not pathes:
        #    trace("find_nearest_unit_building: no pathes")
        #    return None
        # # need to find nearest building:
        # # in-place compare function:
        #def cmp_fnc(a, b):
        #    path1, d = a
        #    path2, d = b
        #    return path1.cost - path2.cost
        #path, nearest_building = min_ex(cmp_fnc, pathes)
        #return nearest_building
# ============================================================
    def __cells_equal(self, c1, c2):
        if c1.col == c2.col:
            if c1.row == c2.row:
                return True
        return False
# ============================================================
    def get_buildings(self, subtype, for_task_type = '', skip_status = []):
        '''Возвращает список зданий указанного типа, причем здания могут принимать ресурсы (и есть другие доп. условия).
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома с вакантными местами, __main__ - главное здание'''
        _is_broken_in = 'broken' in skip_status
        result = []
        if subtype == '__store__':
            for building in self.all_buildings:
                _is_store = building.is_store()
                if _is_store:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        _is_broken = False
                        if building.status in skip_status: _is_broken = True
                        elif _is_broken_in:
                            if building.is_under_attack: _is_broken = True
                        _stype = building.get('stype')
                        if _stype not in ['hq', 'hq_small']:
                            if not building.worker_units:
                                _is_broken = True       # потому что внутри еще нет юнита
                        if not _is_broken:
                            result.append(building)
            trace('STORE BUILDINGS: >>>')
            for b in result:    trace(b.get('name'))
            trace('STORE BUILDINGS <<<')
        elif subtype == '__home__':
            for building in self.all_buildings:
                _is_home = building.is_home()
                if _is_home:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        _is_broken = False
                        if building.status in skip_status: _is_broken = True
                        elif _is_broken_in:
                            if building.is_under_attack: _is_broken = True
                        if not _is_broken:
                            _vacant_places_count = building.Get_vacant_places_count()
                            if _vacant_places_count:  # если есть свободные места
                                result.append(building)
        elif subtype == '__main__': # возвращает главное здание, но в списке
            for building in self.all_buildings:
                _stype = building.get('subtype')
                if _stype in ['hq', 'hq_small']:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        # не бывает поломанным
                        #_is_broken = False
                        #if building.status in skip_status: _is_broken = True
                        #elif _is_broken_in:
                        #    if building.is_under_attack: _is_broken = True
                        #if not _is_broken:
                            result.append(building)
        elif subtype == '__generating__':   # здания, которые могут производить ресурсы
            # вообще данная возможно здесь - это больше ХАК, потому как условия отбора здесь сильно отличаются от других вариантов
            for building in self.all_buildings:
                _need_base_resource = building.need_base_resource()
                if _need_base_resource:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        if building.status not in skip_status:
                            #if building.status == 'normal': # если здание не сломано и не в режиме постройки
                                #_state = building.SM.GetCurrentState()
                                #if _state in ['', 'normal']: # и не апгрейдится
                                #_is_participating = building.is_task_participating(for_task_type)
                                #if not _is_participating:    # не генерирует ресурс и не является участником такого же задания
                                result.append(building)
        else:
            _buildings = []
            if subtype == '':
                _buildings = self.all_buildings
            else:
                _buildings = self.FBC.get(subtype)
            for building in _buildings:
                _is_active_raw = building.is_active_raw()
                if _is_active_raw:
                    #if building.status not in skip_status:
                        if building.status == 'normal': # если здание не сломано и не в режиме постройки
                            _state = building.SM.GetCurrentState()
                            if _state in ['', 'normal']: # и не апгрейдится
                                _is_participating = building.is_task_participating(for_task_type)
                                if not _is_participating:    # не генерирует ресурс и не является участником такого же задания
                                    result.append(building)
        return result
# ============================================================
    def get_buildings_raw(self, subtypes):
        '''Возвращает список зданий указанного типа, c меньшим числом доп. условий
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома с вакантными местами, __main__ - главное здание'''
        if not subtypes:
            return []
        _dict_stypes = {}
        for stype in subtypes: _dict_stypes[stype] = 0  # оптимизация для поиска
        result = []
        first = subtypes[0]
        if first == '__store__':
            for building in self.all_buildings:
                _is_store = building.is_store()
                if _is_store:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        result.append(building)
        elif first == '__home__':
            for building in self.all_buildings:
                _is_home = building.is_home()
                if _is_home:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        result.append(building)
        elif first == '__main__': # возвращает главное здание, но в списке
            for building in self.all_buildings:
                if building.get('subtype') in ['hq', 'hq_small']:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        result.append(building)
        else:
            for building in self.all_buildings:
                _stype = building.get('subtype')
                if _stype in _dict_stypes:
                    if building.status not in ['broken', 'map_place']: # если здание не сломано и это не положение на карте
                        _is_active_raw = building.is_active_raw()
                        if _is_active_raw:
                            result.append(building)
        return result
# ============================================================
    def get_buildings_simple(self, subtypes):
        '''Возвращает список зданий указанного типа, вообще без доп. условий
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома, __main__ - главное здание'''
        if not subtypes:
            return []
        #---
        _is_string = istype(subtypes, 'string')
        if _is_string: subtypes = [subtypes]
        #---
        _dict_stypes = {}
        for stype in subtypes: _dict_stypes[stype] = 0  # оптимизация для поиска
        result = []
        first = subtypes[0]
        if first == '__store__':
            for building in self.all_buildings:
                _is_store = building.is_store()
                if _is_store:
                    #if building.is_active_raw():
                    result.append(building)
        elif first == '__home__':
            for building in self.all_buildings:
                _is_home = building.is_home()
                if _is_home:
                    #if building.is_active_raw():
                    result.append(building)
        elif first == '__main__': # возвращает главное здание, но в списке
            for building in self.all_buildings:
                _stype = building.get('subtype')
                if _stype in ['hq', 'hq_small']:
                    #if building.is_active_raw():
                    result.append(building)
        else:
            for building in self.all_buildings:
                _stype = building.get('subtype')
                if _stype in _dict_stypes:
                    #if building.status not in ['broken', 'map_place']: # если здание не сломано и это не положение на карте
                    #    if building.is_active_raw():
                    result.append(building)
        return result
# ============================================================
    def get_items_by_subtype(self, subtype, skip_activity = False, only_accessible = False):
        '''Возвращает DiffItem'ы по их subtype'''
        result = []
        for item in self.all_items:
            #if item and item.IsSubTypeOf(subtype):
            #    if item.is_active():
            #        result.append(item)
            if item:
                #trace('item UID:', item.uid)
                our_stype = item.IsSubTypeOf(subtype)
                if our_stype:
                    _acceptable = False
                    if not skip_activity:
                        _is_active = item.is_active()
                        if _is_active:
                            _acceptable = True
                    else:
                        _acceptable = True
                    if _acceptable:
                        if only_accessible:
                            _has_access = item.HasAccess()
                            if _has_access:
                                result.append(item)
                        else:
                            result.append(item)
                        #result.append(item)
        return result
# ============================================================
    def find_nearest(self, buildings, buildings_shift_cell_name, target, target_shift_cell_name):
        pathes = []
        cells = []
        for b in buildings:
            _cell = None
            _has_attr = b.HasAttr(buildings_shift_cell_name)
            if _has_attr:
                _cell = b.GetAttrCellFixed(buildings_shift_cell_name)
            else:
                _cell = b.start_cell_pos # если у здания запрашиваемой клетки, то стартовая подменяет ее
            cells.append(_cell)
        result = BMgr.FindNearestCell(target, target_shift_cell_name, cells)
        if result:  # найдем здание по ячейке
            for b in buildings:
                _cell = None
                _has_attr = b.HasAttr(buildings_shift_cell_name)
                if _has_attr:
                    _cell = b.GetAttrCellFixed(buildings_shift_cell_name)
                else:
                    _cell = b.start_cell_pos # если у здания запрашиваемой клетки, то стартовая подменяет ее
                _is_cells_equal = self.__cells_equal(_cell, result)
                if _is_cells_equal:
                    return b
        trace('find_nearest: no pathes')
        trace(' find_nearest: target(from) =', target.__str__())
        trace(' find_nearest: buildings_shift_cell_name =', buildings_shift_cell_name)
        trace(' find_nearest: target_shift_cell_name =', target_shift_cell_name)
        _str = ''
        for b in buildings: _str += b.__str__() + ', '
        trace(' find_nearest: buildings = [', _str, ']')
        return None
# ============================================================
    def remove_building(self, building):
        uid = building.get('uid')
        if uid in self.uids_dict:
            del self.uids_dict[uid]
        if building in self.all_buildings:
            self.all_buildings.remove(building)
        self.FBC.remove(building)
        building.dispose()
# ============================================================
    def Remove_item(self, item):
        uid = item.get('uid')
        if uid in self.uids_dict:
            del self.uids_dict[uid]
        if item in self.all_buildings:
            self.all_buildings.remove(item)
        self.FBC.remove(item)
        if item in self.all_units:
            self.all_units.remove(item)
        if item in UMgr._total_workers:         # вроде тоже нужно
            UMgr._total_workers.remove(item)
        if item in self.all_items:
            self.all_items.remove(item)
        item.dispose()
# ============================================================
    def remove_uid(self, uid):
        if uid in self.uids_dict:
            del self.uids_dict[uid]
# ============================================================
    def get_item(self, uid):
        if uid in self.uids_dict:
            return self.uids_dict[uid]
        return None
# ============================================================
    def get_items(self, uids_string):
        '''Возвращает список элементов по строке UIDов (из редактора)'''
        if not uids_string or len(uids_string) <= len('uids:'):
            return []
        naked_uids = uids_string[len('uids:'):]
        uids = naked_uids.split(';')
        result = []
        for uid in uids:
            result.append(self.get_item(uid))
        return result
# ============================================================
    def get_UIDs_list(self, uids_string):
        '''Возвращает список UIDов (из редактора)'''
        _uids_len = len('uids:')
        _uids_string_len = len(uids_string)
        if not uids_string or _uids_string_len <= _uids_len:
            return []
        naked_uids = uids_string[_uids_len:]
        uids = naked_uids.split(';')
        return uids
# ============================================================
    def Is_build_action_possible(self, template_name, need_worker):
        '''Проверяет - достаточно ли юнитов для строительства'''
        #trace('Is_build_action_possible:', template_name)
        _free_units = UMgr.Get_free_real_units_count()
        _real_units_enough = UMgr.is_free_real_units_enough()
        if need_worker:
            if not _real_units_enough:
                if template_name in ['hq1', 'hq_small1', 'house1']:
                    return True
                elif template_name == 'watchtower1':
                    if _free_units > 1:
                        return True
                return False
        return True
# ============================================================
    def Has_accessible_building(self, template_name, status=''):
        for b in self.all_buildings:
            _name = b.get('name')
            if _name == template_name:
                _is_accessible = b.HasAccess()
                if _is_accessible:
                    if b.status != 'map_place':
                        if not status or status == b.status:
                            return True
        return False
# ============================================================
    active_states = {'awaiting_action':0, 'build':0, 'upgrade':0}
    def Has_active_building(self, template_name):
        '''Есть ли доступное, активное, несломанное, либо начавшее ремонт, здание'''
        #trace('Has_active_building:', template_name)
        for b in self.all_buildings:
            _name = b.get('name')
            if _name == template_name:
                _is_accessible = b.HasAccess()
                _b_state = b.SM.GetState()
                #trace(' Has_active_building:', _is_accessible, _b_state)
                if _is_accessible:
                    if not b.is_deactivated:
                        #trace(' Has_active_building: ', b.status)
                        if b.status != 'map_place':
                            #trace(' Has_active_building: ', b.lock_shown)
                            if b.status != 'broken':
                                #trace(' Has_active_building: TRUE')
                                return True
                            elif b.lock_shown:
                                return True
                            #elif _b_state in ['awaiting_action', 'build', 'upgrade']:
                            elif _b_state in self.active_states:
                                return True
        #trace(' Has_active_building: False')
        return False
# ============================================================
    def Has_buildings_in_build(self, template_name):
        '''Есть ли доступное, активное, несломанное, либо начавшее ремонт, здание'''
        for b in self.all_buildings:
            _name = b.get('name')
            if _name == template_name:
                _is_accessible = b.HasAccess()
                _b_state = b.SM.GetState()
                if _is_accessible:
                    if not b.is_deactivated:
                        #if _b_state in ['awaiting_action', 'build', 'upgrade']:
                        if _b_state in self.active_states:
                            return True
        return False
# ============================================================
    non_attack_states = {'build':0, 'upgrade':0, 'repair':0, 'awaiting':0}
    non_attack_statuses = {'broken':0, 'map_place':0, 'place':0}
    def get_nest_attack_buildings(self, item, cell_shift_name, radius):
        '''Возвращает список зданий доступных для атаки варварами в указанном радиусе'''
        result = []
        buildings = self.GetUserBuildingsInRadius(item, cell_shift_name, radius)
        for b in buildings:
            _stype = b.get('stype')
            _need_no_worker = b.GetAttrBool('need_no_worker')   # эти здания вроде как все не нужно атаковать (но ниже они перечислены)
            if _need_no_worker:
                continue
            if _stype not in b.not_for_attack:
                if b.status not in self.non_attack_statuses:
                    state = b.SM.GetState()
                    if state not in self.non_attack_states:
                        if b.energy_hit > 0:
                            #if not b.is_under_attack:   # совместная атака?
                            result.append(b)
        return result
# ============================================================
    def is_shop_red(self):
        '''Проверяем - есть ли что строить в магазине'''
        _has_bridge = False
        _can_anything_to_build = self.CanAnythingToBuild()
        if not _can_anything_to_build:
            return True
        templates = self.GetBuildableList()
        #tmp_t = []
        for item in templates:
            item.cannot = ''
            if item.is_visible:
                if item.subtype == 'bridge':
                    if _has_bridge: continue
                    else: _has_bridge = True
                _can_build = item.can_build
                _is_prior = self.CanBuildPriority(item.name)
                if _can_build:
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    if item.depends_from:
                        if _can_build:
                            for dep in item.depends_from:
                                _has = self.HasActiveBuilding(dep)
                                _can_build = False
                                item.cannot = 'NO_DEPENDANT'
                                if _has:
                                    item.cannot = ''
                                    _can_build = True
                                    break
                    if _can_build:
                        _has_builders = self.HasEnoughBuilders(item.name)
                        _has_mplaces = self.HasActiveMapPlaces(item.name, item.subtype)
                        if not _has_builders:
                            _can_build = False
                            item.cannot = 'NO_BUILDERS'
                        elif not _has_mplaces:
                            _can_build = False
                            _has_deactive_mplaces = self.HasOnlyDeactivatedMapPlaces(item.name, item.subtype)
                            item.cannot = ('NO_MAP_PLACE', 'NO_ACTIVE_MAP_PLACE')[_has_deactive_mplaces == True]
                        if item.cannot == '':
                            if item.name == 'quarry1':
                                _has_granite = self.HasGranite()
                                if not _has_granite:
                                    _can_build = False
                                    item.cannot = 'NO_GRANITE'
                else:
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    else:
                        item.cannot = 'CANNOT_BUILD'    # from C++
                if _can_build:
                    return False
            #tmp_t.append(item.cannot)
        return True
# ============================================================

