# -*- coding: utf-8 -*-


import sys
if not "tinypy" in sys.version:
    from extra import *

# #######################################################
# - может таски разделять на те, что запущены и те, что нет?
# #######################################################
import quest
import quest_normal
import quest_giveres
import quest_tut_build
import quest_tut_break
import quest_tut_upgrade

class QuestsMgr(QuestsMgr_class):
    def __init__(self, cdata):
        QuestsMgr_class.__init__(self, cdata)
        self.events = []
        self.all_quests = {}        # здесь все квесты хранятся
        self.started_quests = []
        self.dlg_quest = None       # квест, для которого сейчас показывается диалог
        self.completed_quests = []
        self.awaiting_quests = []
        self.tutorial_quests = []
        self.deactivated_tutorial_quests = []  # туториальные квесты, которые пока не активированы
        self.time_without_quest = 0 # время, которое прошло без запущенного квеста
        self.ticks = 0
        # смотреть on_create !!!
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'global_event':
            self.on_global_event(msg_param1, msg_param2)
        elif msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == "on_create":
            self.on_create()
# ============================================================
    def on_signal(self, signal):
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == "enter":
            return
        elif type == "exit":
            return
        elif type == "change":
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def on_create(self):
        trace("QuestsManager on_create")
        self.load_all_quests()  # загружаем все квесты
        EMgr.add_event_listener(self)   # добавляем себя в слушатели событий
        pass
# ============================================================
    def on_event(self, name, obj, extra):
        '''Рассылает события запущенным квестам'''
        for quest in self.started_quests:
            quest.on_quest_event(name, obj, extra)
# ============================================================
    def on_global_event(self, event, param):
        if event == 'extra_event':
            #if not self.has_started_quests():
            for q in self.awaiting_quests:  # пробегаем по индейским квестам
                q.on_extra_event(param)
        #elif event == 'global_unit_talk':
        #    for q in self.started_quests:  # пробегаем по индейским квестам
        #        q.on_quest_event(param, None, {})
# ============================================================
    def on_update(self, ms):                # данный метод должен вызываться из C++ каждый квант
        '''Вызываем обновление у всех запущенных квестов. Если запущенных нет, то пытаемся запустить туториальный, если есть'''
        self.ticks += 1
        _completed_quests = []
        _has_started = self.has_started_quests()
        if not _has_started:
            self.time_without_quest += ms   # увеличим время, проведенное без квеста
            if not (self.ticks % 50):   #! каждую секунду (25)
                _barb_attack_count = UStats.GetLevelValue('barbarian_attack')
                if _barb_attack_count == 0:
                    self.try_start_tutorial()       # пытаемся запустить туториал
        else:
            for started_quest in self.started_quests:
                started_quest.on_update(ms)
                if started_quest.is_complete:
                    _completed_quests.append(started_quest)  # сохраняем во временный список, потому что 'move_quest_to_completed' удаляет элементы списка self.started_quests
                    if started_quest.is_final:
                        UStats.AddLevelValue('final_quest_complete', 1) # финальный квест завершен
                        _is_time_up = UStats.GetLevelValue('time_is_up') > 0
                        if not _is_time_up:
                            UStats.StopTime()   # время на уровне больше не идет
            for completed_quest in _completed_quests:
                self.move_quest_to_completed( completed_quest )
# ============================================================
    def on_rare_update(self, ms):
        pass
# #######################################################
# #######################################################
    def load_all_quests(self):
        '''Загружает данные квестов. Создает квесты и размещает их в необходимые списки'''
        count = self.GetQuestsCount()
        trace("Quests count: ", count)
        for i in range(count):
            name, data = self.GetQuestData(i)
            q = None
            if data.is_tutorial:
                trace("tutorial quest:", name)
                if data['task1_action'] in ['build_building', 'repair', 'repair_boat']:   # если это обычный туториальный квест
                    q = quest_tut_build.QuestTutBuild(name, data)
                elif data['task1_action'] == 'general_break':   # если квест на уничтожение горшка
                    q = quest_tut_break.QuestTutBreak(name, data)
                elif data['task1_action'] == 'building_upgrade':   # если квест на апгрейд здания
                    q = quest_tut_upgrade.QuestTutUpgrade(name, data)
                elif data['task1_action'] == 'do_action':   # если квест на куст или подобное
                    q = quest_normal.QuestNormal(name, data)
                if data.deactivated:
                    self.deactivated_tutorial_quests.append(q)
                else:
                    self.tutorial_quests.append(q)
            else:
                trace("non tutorial quest:", name)
                if data['task1_action'] == 'give_resources':
                    q = quest_giveres.QuestGiveRes(name, data)
                else:
                    q = quest_normal.QuestNormal(name, data)
                self.awaiting_quests.append(q)
            self.all_quests[name] = q
        trace("load quests complete")
# ============================================================
    def try_start_tutorial(self):
        '''Пробует запустить какой-нибудь туториальный квест'''
        completed_quests = []
        _is_current_loc_active = self.IsCurrentLocationActive()
        if not _is_current_loc_active:  # пока локация неактивна - не запускаем туториальные квесты на ней
            return
        _is_units_critical = UMgr.Is_units_count_critical() # если критическое кол-во юнитов - туториальные квесты не запускаем
        # проверка на критич.кол-во юнитов ниже, чтобы пропускать квесты с домиками
        _has_started = self.has_started_quests()
        if not _has_started:   # квесты запускать лишь тогда, когда других нет
            for q_tut in self.tutorial_quests:
                if not q_tut: continue
                trace('QMgr: awaiting tut-quest start-possibility "' + str(q_tut.name) + '"')
                _q_start_possible = q_tut.is_start_possible()
                if _q_start_possible:   # если запуск квеста возможен
                    _q_needed_items_exist = q_tut.are_needed_items_exist()
                    if not _q_needed_items_exist:
                        # фейковый старт и завершение квеста:
                        self.is_started = True
                        q_tut.complete()
                        completed_quests.append(q_tut)
                        break  # потому что удалили из списка
                    # --- затычка для того, чтобы не запускать туториальные квесты
                    # когда мало работников, исключение - квест на ремонт домика
                    if _is_units_critical:
                        while True:
                            qtd = q_tut.get_first_qtd()
                            if qtd.type == 'repair':
                                if qtd.task_items:
                                    building = qtd.task_items[0]
                                    stype = building.get('stype')
                                    if stype == 'house':
                                        break   # этот квест можно запускать
                            return              # остальные - нет
                    # ---
                    if q_tut.need_visible_items:
                        if q_tut.extra_wait_time == 0:
                            trace('mgr_q: q_tut.extra_wait_time = 2000')
                            q_tut.extra_wait_time = 2000
                            q_tut.params.time = 0   # чтобы не влияло больше
                            QMgr.time_without_quest = 0 # сбросим счетчик "без квестов"
                            break
                        elif QMgr.time_without_quest < q_tut.extra_wait_time:
                            break       # ждем!
                    trace('QMgr: quest "' + str(q_tut.name) + '" started')
                    self.dlg_quest = q_tut
                    #self._start_tut_quest(q_tut)
                    self.tutorial_quests.remove(q_tut)
                    _tasks = q_tut.get_tasks_text()
                    _dlg_settings = {'nameid': '__unit_name__', \
                                     'textid': q_tut.params.text, \
                                     'tasks_text': _tasks, \
                                     'task_uid':str(-1), \
                                     'single_mode':str(True),# диалог должен выводиться в гордом одиночестве
                                     'general_cloud':str(True), # не показывать, пока висит облако генерала
#                                     'showin_effect':'task_fly_mirror_in',
                                     }
                    qtd_type = q_tut.quest_tasks_datas[0].type
#                    if qtd_type != 'give_resources':
#                        _dlg_settings['need_task_effect'] = str(1)  # и эффект нужно показать
                    #UI.show_tut_text(q_tut.params.text)     # покажем туториальную подсказку
                    dlg.Open("GeneralMenuOk", "dlg_general_ok", _dlg_settings )
                    break   # запускаем только один квест
                elif q_tut.is_complete:        # возможно, если игрок действует не по "плану" (is_complete может изменяться в is_start_possible())
                    trace('Tutorial quest is complete', q_tut.name)
                    completed_quests.append(q_tut)
                else:
                    break   # квесты запускаются строго по одному и по очереди
        for q in completed_quests:          # удаляем уже выполненные квесты
            self.tutorial_quests.remove(q)
            q.on_after_complete()
            self.completed_quests.append(q) # нужно ли туториальные квесты хранить?
# ============================================================
    def on_dlg_callback(self, result, extra_dict):
        '''Вызывается диалогом генерала'''
        self._start_tut_quest(self.dlg_quest)
        self.dlg_quest = None
# ============================================================
    def move_quest_to_completed(self, quest):
        '''Перемещает квест в список выполненных, кроме туториальных. Удаляет квест из списка запущенных. Извещает объекты о прекращении.'''
        self.started_quests.remove(quest)
        if not quest.is_tutorial:       # туториальные не храним
            self.completed_quests.append(quest)
        # рассылать сообщения зданиям пока нет никакого смысла
        #for item_b in BMgr.all_buildings:
        #    item_b.SendMsg('quest_complete', quest.name)
        for item_u in BMgr.all_units:
            _stype = item_u.get('subtype')
            if _stype not in ['worker', 'woodcutter', 'qman']:  # им данные о квестах не нужны
                item_u.SendMsg('quest_complete', quest.name)
        # рассылать сообщения объектам пока нет никакого смысла
        #for item_d in BMgr.all_items:
        #    item_d.SendMsg('quest_complete', quest.name)
# ============================================================
    def has_started_quests(self):
        '''Есть ли запущенные квесты.'''
        return len(self.started_quests)
# ============================================================
    def has_started_nonartefact_quests(self):
        '''Есть ли запущенные квесты НЕ на артефакт.'''
        _allowed_quests = ['give_object', 'talk']
        if self.dlg_quest:
            if self.dlg_quest not in _allowed_quests:
                return True
        for q in self.started_quests:
            qtds = q.quest_tasks_datas
            if qtds:
                if qtds[0].type not in _allowed_quests:   # если это не артефакт и не разговор
                    return True
        return False
# ============================================================
    def activate_quest(self, name):
        '''активирует туториальный квест'''
        _q = self.get_deactivated_quest(name)
        if _q:  # если данный квест присутствует среди деактивированных:
            self.deactivated_tutorial_quests.remove(_q)
            self.tutorial_quests.append(_q)
# ============================================================
    def _start_tut_quest(self, quest):
        '''Внутренний метод запуска туториального квеста, он запускает квест не проверяя необходимых условий'''
        if not quest:
            mbox('No quest info in dialog callback')
        else:
            trace('Starting tutorial quest:', quest.name)
            quest.start()
            self.started_quests.append(quest)
        self.time_without_quest = 0 # время сбросить не забудем
        # известим все объекты на поле о запуске квеста:
# ============================================================
    def start_quest(self, quest):
        '''Запускает обычный квест, рассылая необходимые сообщения'''
        trace('Starting quest:', quest.name)
        quest.start()       #  запускаем сам квест
        self.started_quests.append(quest)   # добавляем в список запущенных квестов
        self.time_without_quest = 0 # время сбросить не забудем, хотя это уже не должно быть важно - это только для туториальных квестов было
        # известим все объекты на поле о запуске квеста:
# ============================================================
    def get_quest(self, name):
        if name in self.all_quests:
            return self.all_quests[name]
        #trace(self.all_quests)
        return None
# ============================================================
    def get_deactivated_quest(self, name):
        for _q in self.deactivated_tutorial_quests:
            if _q.name == name:
                return _q
        return None
# ============================================================
    def is_quest_started(self, name):
        q = self.get_quest(name)
        if q:
            return q.is_started
        return False
# ============================================================
    def is_quest_complete(self, name):
        q = self.get_quest(name)
        if q:
            return q.is_complete
        return False
# ============================================================
    def has_tutorial_quests(self):
        return len(self.tutorial_quests) or self.dlg_quest
# ============================================================
    def has_started_tut_quest(self):
        for q in self.started_quests:
            if q.is_tutorial:
                return True
        if self.dlg_quest:
            return True
        _count = len(self.tutorial_quests)
        if _count > 0:
            for q_tut in self.tutorial_quests:
                if q_tut.is_start_possible():
                    return True
        return False
# ============================================================
