# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *


import quest
import game_task

# ============================================================
class QuestUniversalTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, event_name, qtdata, items):
        game_task.GameTask.__init__(self, qtdata)
        self.events_count = len(items)     #^ кол-во событий (game_event), необходимое для выполнения задания
        self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = event_name     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.items = items

    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        #mbox(name, self.game_event)
        if name == self.game_event:     # наше событие завершения
            if obj in self.items:
                self.events_remain -= 1
                self.update_count()
                if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                    self.set_complete()                     # значит уже все сделали
# ============================================================

class QuestDoActionTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, qtdata, items):
        game_task.GameTask.__init__(self, qtdata)
        #self.task_template = qtdata.task_template
        self.events_count = len(items)     #^ кол-во событий (game_event), необходимое для выполнения задания
        self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = 'do_action_complete'     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.items = items

    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == self.game_event:     # наше событие завершения
            if obj in self.items:
                self.events_remain -= 1
                self.update_count()
                if self.events_remain <= 0:      # учитываем, что действий выполнено достаточное кол-во
                    self.set_complete()                     # значит уже все сделали
# ============================================================
class QuestInventoryTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, qtdata, quest_item_name, quest_item_count = 1):
        game_task.GameTask.__init__(self, qtdata)
        #self.task_template = qtdata.task_template
        self.events_count = quest_item_count     #^ кол-во событий (game_event), необходимое для выполнения задания
        self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = 'inventory_add_qitem'     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.quest_item_name = quest_item_name
        self.update_events_remain_count()  # здесь нельзя это вызывать
        UStats.AddLevelValue('give_obj_quest_started')
        QMgr.SendGlobalMsg('give_obj_quest_started', '')    # уведомим всех, что квест запущен... Только останутся те юниты, что созданы после.. (на подлокациях)

    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        #mbox(name, self.game_event)
        if name == self.game_event:     # наше событие завершения
            self.update_events_remain_count()
            if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                self.set_complete()                     # значит уже все сделали

    def update_events_remain_count(self):
        '''Проверка на звершение задания'''
        has_count = UStats.Inventory.GetQuestItemCount(self.quest_item_name)
        if has_count:
            self.events_remain = self.events_count - has_count
            self.update_count()

    def force_completeness_check(self):
        self.update_events_remain_count()
        if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
            self.set_complete()                     # значит уже все сделали
# ============================================================
class QuestTalkTask(game_task.GameTask):
    '''Задание квеста для разговора с юнитом(у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, qtdata, quest_npc_names):
        game_task.GameTask.__init__(self, qtdata)
        #self.task_template = qtdata.task_template
        self.events_count = len(quest_npc_names)     #^ кол-во событий (game_event), необходимое для выполнения задания
        self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = 'unit_talk'     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.quest_npc_names = quest_npc_names[:]   # нужна полная копия списка, чтобы из нее можно было удалять
        # покажем значки у говорящих юнитов
        UStats.AddLevelValue('talk_quest_was_started')
        QMgr.SendGlobalMsg('talk_quest_started', '')    # уведомим всех, что квест запущен... Только останутся те юниты, что созданы после.. (на подлокациях)

    def on_event(self, name, obj, extra_dict):
        if name == self.game_event:     # наше событие завершения
            if 'param' in extra_dict:
                param = extra_dict.param
                if param in self.quest_npc_names:
                    self.quest_npc_names.remove(param)
                    self.events_remain -= 1
                    self.update_count()
                    if self.events_remain <= 0:      # учитываем, что действий выполнено достаточное кол-во
                        self.set_complete()                     # значит уже все сделали
# ============================================================

class QuestNormal(quest.Quest):
    def __init__(self, name, data_dict):
        quest.Quest.__init__(self, name, data_dict)
# ============================================================
    # @override
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        # нужно проверить, что данный квест еще не выполнен:
        #qtd = self.quest_tasks_datas[0]     #! пока работаем только с одним, но потом будет и больше
        #_items = qtd.task_items
        #_items_count = len(_items)
        #_visible_items = self.need_visible_items
        #_visible_count = len(_visible_items)
        if not quest.Quest.is_start_possible(self, skip_ne_cases):
            return False
        if self.is_tutorial:
            if not QMgr.has_started_quests():   # если нет запущенных квестов
                if self.quest_tasks_datas:
                    _qtd = self.quest_tasks_datas[0]
                    if _qtd.type == 'do_action':
                        if _qtd.task_items:
                            _item = _qtd.task_items[0]
                            if _item.had_action:        # если над объектом производили действия, то квест скипуем
                                #self.is_complete = True
                                self.complete()
                                return False
                if not self.are_needed_items_exist():   # чтобы быстрее его же закончить
                    return True
                if QMgr.time_without_quest >= self.params.time: # если прошло уже достаточно времени
                    for _qtd in self.quest_tasks_datas:
                        for _item in _qtd.task_items:
                            if _item:
                                if not _item.HasAccess(): # если объект недоступен, то:
                                    return False
                else:
                    return False
        return True     # для тасков от сидячих индейцев - всегда возможно запустить. С ходящими будет сначала генерация идущего индейца
# ============================================================
    # @override
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        quest.Quest.start(self)
        self.add_game_tasks()
        self.is_started = True  #! строго в конце метода
        pass
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод (ну или start - позже будет ясно) и
        #игроку даются игровые задания'''
        if not self.is_started:
            for qtd in self.quest_tasks_datas:
                #qtd = self.quest_tasks_datas[0]     # QuestTaskData
                _task = None
                qtd_type = qtd.type
                if qtd_type == 'repair':
                    _task = QuestUniversalTask('build_repair_complete', qtd, qtd.task_items)
                elif qtd_type == 'repair_boat':
                    _task = QuestUniversalTask('build_repair_complete', qtd, qtd.task_items)
                    #_task = QuestUniversalTask('build_repair_boat_complete', qtd, qtd.task_items)
                elif qtd_type == 'build_building':
                    _task = QuestUniversalTask('build_building_complete', qtd, qtd.task_items)
                elif qtd_type == 'break':
                    pass
                elif qtd_type == 'do_action':
                    _task = QuestDoActionTask(qtd, qtd.task_items)
                elif qtd_type == 'give_object':
                    _task = QuestInventoryTask(qtd, qtd.quest_item_name, qtd.quest_item_count)
                elif qtd_type == 'talk':
                    _quest_npc_names = qtd.quest_npc_unique_names
                    _task = QuestTalkTask(qtd, _quest_npc_names)
                elif qtd_type == '':
                    pass
                if not _task:
                    mbox("Task was not created. Type=" + qtd_type)
                self.quest_tasks.append(_task)
                _uid = '-1'
                _task_tips = tools.ListToStr(self.get_tasks_text_for_panel(qtd))
                _items_count = len(qtd.task_items)
                if _items_count == 1:
                    _uid = qtd.task_items[0].get('uid')
                _npc_uid = '-1'
                if self.npc: _npc_uid = self.npc.get('uid')
                self._dbg_npc_uid = _npc_uid
                _task.slot = UI.AddGameTask(_task.task_image, _task.task_image_frame, _task.task_template, {'uid':_uid, 'npc_uid':_npc_uid, 'tips':_task_tips})

                UI.SendMessage(_task.slot, 'set_task_count', str(_task.events_remain))
                _task_text = ''
                if qtd_type in ['repair', 'repair_boat']:
                    _task_text = 'TASK_REPAIR'
                elif qtd_type == 'build_building':
                    _task_text = 'TASK_BUILD'
                elif qtd_type == 'do_action':
                    _task_text = self.get_qtd_action_text(qtd)
                elif qtd_type == 'give_object':
                    _task_text = 'TASK_BRING'
                UI.SendMessage(_task.slot, 'set_task_text', _task_text)
                UI.SendMessage(_task.slot, 'set_task_tips_text', _task_tips )
                _task.force_completeness_check()
        pass
# ============================================================
    # @override
    def complete(self):
        '''Завершает квест и считает его выполненным'''
        _qtd_count = len(self.quest_tasks_datas)
        if _qtd_count:
            if self.quest_tasks_datas[0].type == 'give_object': # квест "принести артефакт"
                _quest_item_name = self.quest_tasks_datas[0].quest_item_name    # сработает только если один объект
                _quest_item_count = self.quest_tasks_datas[0].quest_item_count
                UStats.Inventory.RemoveQuestItem(_quest_item_name, _quest_item_count)
        types = [qtd.type for qtd in self.quest_tasks_datas]    # типы завершаемых заданий
        if 'talk' in types: # если завершаем говорящий квест
            UStats.SubtractLevelValue('talk_quest_was_started') # если остались говорящие - отключим их значки
            UMgr.SendGlobalMsg('talk_quest_completed')          # если остались говорящие - отключим их значки
        if 'give_object' in types:
            UStats.SubtractLevelValue('give_obj_quest_started')
            UMgr.SendGlobalMsg('give_obj_quest_completed')
        quest.Quest.complete(self)
# ============================================================
    # @override
    def on_update(self, ms):
        if not self.is_precomplete:
            _can_complete = True
            for task in self.quest_tasks:
                if not task.is_complete:
                    _can_complete = False
                    break
            if _can_complete:
                self.is_precomplete = True
                if self.is_tutorial:    # если это туториальный квест
                    self.complete()
                self.is_precomplete = True
                if self.npc: self.npc.activate()
                if self.name == 'quest_07_repair':
                    if LBoard.Core.episode == 1:
                        if LBoard.Core.level == 1:
                            UI.show_tut_text('TUT_HELP_05') # хак для первого квеста первого уровня
                    #UI.Signals.Restart('tut_task_1', 1000)
# ============================================================
    #def __str__(self):
    #    extra = to_str("quest: ", self.uid, "is_complete", self.is_complete, "is_started", self.is_started, "is_tut", self.is_tutorial)
    #    return to_str("{", extra, "}", str(self))
# #######################################################
# #######################################################
# #######################################################
# #######################################################
# #######################################################
