# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes


# #######################################################
# #######################################################
class BuildingPlaceTask(task_classes.Task):
    '''Класс-задание для постройки здания, которое имеет предустановленное место'''
    def __init__(self, type, map_place_element):
        task_classes.Task.__init__(self, type)

        self.priority = 50
        self.building = map_place_element
        self.is_building_started = False
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append("free_unit") # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        '''Проверка необходимых условий начала таска, а так же минимальная инициализации при достаточности условий'''
        #if self.type == "build_building" and self.building == None:    #если строительство здания
        #    return False
        self.building.show_clock()  # песочные часы ожидания
        b = self.building
        self.resources_need = b.get_needs_res('level_1')
        b.switch_level_to_place()           # переключим здание на шаблон, который можно строить
        b.start_task_participation(self)
        self.building.SM.SetState('awaiting_action')
        return True
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            self.is_started = True
            trace("BuildingPlaceTask: started", self.building.get('name'))   #unit.get('name'),
            #mbox(to_str("task:", self.uid, self._units))
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        if self.building:
            build_cell_name = self.building.get_action_cell_name()
            if build_cell_name == None: return False
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, build_cell_name, 'builder' )
            if unit == None:
                return False
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype('builder') # нам нужен строитель
            success = unit.run_to_target(self.building, None, build_cell_name)
            if not success:
                trace('BuildingPlaceTask::try_call_unit: run_to_target failed', unit.get('uid'))
                if removed: UMgr.add_free_unit(unit)
                return False
            self.building.reserve_action_cell(build_cell_name)
            unit.task = self
            unit.task_type = self.type
            self._units.append(unit)
            trace("BuildingPlaceTask: started", unit.get('name'), self.building.get('name'))
            #mbox(to_str("task:", self.uid, self._units))
            self.create_unit_signal()   # позже позовем еще одного
            return True
        return False
# ============================================================
    def _complete_task(self):
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace("BuildingPlaceTask task complete")
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.task = None
            unit.task_type = ""
            #unit.SM.SetState("run")
            UMgr.add_free_unit(unit)
        self.building.SM.SetState('normal')
        self.building.switch_level('builded')   # переключаемся на следующий вариант
        self.building.stop_task_participation(self)
        self.building = None
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if not self.is_building_started:
            self.building.show_clock(False)  # песочные часы ожидания
            self.building.on_before_building_upgrade()
            self.building.SM.SetState('build')
            #self.create_unit_signal()   # если нужно, то позже позовем еще одного
        self.is_building_started = True
        sender.SM.SetState('build')
        sender.Drawer.PlayAnimationOnce('build', 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
# ============================================================
    def on_update(self, ms):
        if self.is_building_started: # если строительство уже началось, то:
            # нужно готовность здания увеличивать пропорционально кол-ву работающих юнитов
            b = self.building
            count = self._get_building_units_count()
            quant = ms * count * b.GetBoostMultiplier()
            b.build_progress += quant
            if b.build_progress >= b.build_time:
                self._complete_task()
            else:
                b.Drawer.SetFramesProgress('build', b.build_progress/b.build_time) # сменяет кадры постройки, если изображение указано как анимированное
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        b = self.building
        b.show_clock(False)  # песочные часы ожидания
        if not self.is_building_started:        # если юнит еще не начал действие, то
            UStats.AddRes(self.resources_need)  # нужно возвращать стоимость апгрейда
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace('BuildingPlaceTask task canceled')
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.task = None
            unit.task_type = ''
            UMgr.add_free_unit(unit)
        b.unreserve_action_cells()
        b.SM.SetState('normal')
        #self.building.switch_level()   # переключаемся на следующий вариант
        b.stop_task_participation(self)
        b.cancel_switch_level_to_place()
        self.building = None
# ============================================================
    def _get_building_units_count(self):
        count = 0
        for unit in self._units:
            if unit.SM.GetCurrentState() == 'build':
                count += 1
        return count
# ============================================================
    def on_external_signal(self, signal_id):
        if signal_id == 'try_more_units':
            if self.try_call_unit():
                self.create_unit_signal()   # если нужно, то позже позовем еще одного
            else:   # юнита позвать не удалось
                self.create_unit_signal(500)   # попробоуем еще раз через полсекунды
        pass
# ============================================================
    def on_bump_need(self):
        if self.building:
            self.building.scale_bump()
# #######################################################
# #######################################################
# #######################################################
