# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *


class Subtask(object):
    '''Мини-задание в основном для действий, после достижения цели юнитом'''
    def __init__(self, parent_task, actor):
        self.parent_task = parent_task
        self.actor = actor
        self.actor_action_animation = ''    # анимация для юнита при выполнении задания, задается в каждом таске
# ============================================================
    def start_action(self, item):
        #item.action_started = True
        self.on_action_complete()   # complete immediately
        return True
# ============================================================
    def on_action_complete(self):
        self.parent_task.complete_task()
# ============================================================
    def on_callback(self, result, extra_dict):
        pass

# ============================================================
# ============================================================
# ============================================================
#@statemachine
class Task(object):
    def __init__(self, type):
        self.uid = -1
        self.is_complete = False
        self.is_started = False
        self.is_action_started = False
        self.item = None
        self.is_event_task = False
        self._event_triggers = []
        self.type = type
        self.extra = ''
        self.is_unique_task = False # для того, чтобы таск сделать уникальным, если требуется, т.е. не плодить одинаковых тасков, если не хватает ресурсов или юнитов
        self.unique_id = ''
        self.priority = 0   # пока не используется, но чем больше приоритет, тем приоритетнее задание
        self.subtask = None
# ============================================================
    def on_update(self):
        pass
# ============================================================
    def do_pre_action(self):
        '''Делает предварительные проверки на предмет возможности создания задания, а также подготовительные расчеты'''
        pass
# ============================================================
    def try_do_action(self):
        '''Пытается выполнить задание'''
        pass
# ============================================================
    def complete_task(self):
        self._complete_task()
# ============================================================
    def _complete_task(self):
        pass
# ============================================================
    def cancel_action(self):
        '''Должно будет отменять начатое задание'''
        pass
# ============================================================
    def cancel(self, force=False):
        '''Отмена задания, force - когда отменяются задания системой, а не через клик'''
        pass
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if self.subtask != None:
            if self.item != None:
                if not self.is_action_started:
                    # сначала сменим анимацию юнита для действия над объектом:
                    self.subtask.actor.current_action_animation = self.subtask.actor_action_animation
                    self.item.on_before_action_started()    # перед тем, как начнется действие над объектом
                    _started = self.subtask.start_action(self.item)
                    if _started:
                        self.is_action_started = True
                        self.on_subtask_started()
        else:
            self.complete_task()
# ============================================================
    def on_subtask_started(self):
        pass
# ============================================================
    def is_event_trigger(self, event_name):
        '''Возвращает True, если данное событие привязано к событию event_name'''
        for trig in self._event_triggers:
            if event_name == trig:
                return True
        return False
# ============================================================
    def is_working(self):
        return (self.is_started and not self.is_complete)
# ============================================================
    def was_not_started(self):
        return (not self.is_started and not self.is_complete)
# ============================================================
    def on_callback(self, result, extra_dict):
        pass
# ============================================================
    def __str__(self):
        extra = to_str("task: ", self.uid, self.type, "is_complete", self.is_complete, "is_started", self.is_started)
        return to_str("{", extra, "}", str(self))
# #######################################################
# #######################################################
# ============================================================
    def create_unit_signal(self, time = -1):
        b = self.building
        if b:
            _cur_units = len(self._units)
            _max_units = b.max_build_units
            if _max_units > _cur_units:
                _next_unit_ms = time
                if time < 0:
                    _next_unit_ms = b.GetAttrInt('next_unit_ms') # время до следующего юнита
                    if not _next_unit_ms:
                        _next_unit_ms = 650
                EMgr.add_signal(self, 'try_more_units', _next_unit_ms)   # через две секунды позвать нового
# ============================================================
    def on_external_signal(self, signal_id):
        pass
# #######################################################
# #######################################################
# #######################################################
