# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class UpgradeTask(task_classes.Task):
    def __init__(self, building):
        type = 'upgrade_building'
        task_classes.Task.__init__(self, type)
        self.extra = building.get('name')

        self.priority = 50
        if not UMgr.is_free_real_units_enough():    # если мало юнитов, то:
            _stype = building.get('stype')
            if _stype in ['hq', 'hq_small', 'house']:
                self.priority = 100
        self.extra += ' p=' + str(self.priority)
        self.building = building
        self.is_building_started = False
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append("free_unit") # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        b = self.building
        if b.is_next_level_possible():
            self.building.show_clock()  # песочные часы ожидания
            next_level_name = b.get_next_level_name()   # название варианта следующего уровня
            self.resources_need = b.get_needs_res(next_level_name)   # необходимые ресурсы
            UStats.SubRes(self.resources_need)      # вычитаем затраченные ресурсы
            self.building.on_before_building_upgrade()
            self.building.start_task_participation(self)
            self.building.SM.SetState('awaiting_action')
            return True
        return False
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            self.is_started = True
            trace('UpgradeTask: started', self.building.get('name'))   #unit.get('name'),
            #mbox(to_str("task:", self.uid, self._units))
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        '''Вызывает юнита на апгрейд'''
        if self.building:
            build_cell_name = self.building.get_action_cell_name()
            if build_cell_name == None: return False
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, build_cell_name, 'builder' )
            if unit == None:
                return False
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype('builder') # нужен строитель
            success = unit.run_to_target(self.building, None, build_cell_name)
            if not success:
                trace('UpgradeTask::try_call_unit: run_to_target failed', unit.get('uid'))
                if removed: UMgr.add_free_unit(unit)
                return False
            self.building.reserve_action_cell(build_cell_name)
            unit.task = self
            unit.task_type = self.type
            self._units.append(unit)
            trace("UpgradTask: started", unit.get('name'), self.building.get('name'))
            #mbox(to_str("task:", self.uid, self._units))
            self.create_unit_signal()   # позже позовем еще одного
            return True
        return False
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if not self.is_building_started:
            self.building.on_upgrade_started()
        #    self.create_unit_signal()   # если нужно, то позже позовем еще одного
        self.building.show_clock(False)  # песочные часы ожидания
        self.is_building_started = True
        sender.SM.SetState('build') # юнита переключаем в режим молотка
        sender.Drawer.PlayAnimationOnce('build', 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
        self.building.SM.SetState('upgrade')
        if self.building.status == 'normal':
            if self.building.hack_for_upgrade:
                sender.hide()
                self.building.show_unit = True
# ============================================================
    def _complete_task(self):
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace("task complete")
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.show()
            unit.task = None
            unit.task_type = ""
            UMgr.add_free_unit(unit)
        b = self.building
        b.show_unit = False
        b.SM.SetState('normal')
        _prev_status = self.building.status
        if _prev_status == 'broken':
            b.switch_level('repaired')   # переключаемся на следующий вариант (после того, как состояние здания перевели в normal !)
            trace('Building repair complete')
        else:
            b.switch_level('upgraded')
            trace('Building upgrade complete')
        b.stop_task_participation(self)
        self.building = None
# ============================================================
    def on_update(self, ms):
        if self.is_building_started: # если строительство уже началось, то:
            # нужно готовность здания увеличивать пропорционально кол-ву работающих юнитов
            b = self.building
            count = self._get_building_units_count()
            quant = ms * count * b.GetBoostMultiplier()
            b.build_progress += quant
            if b.build_progress >= b.build_time:
                self._complete_task()
            else:
                b.Drawer.SetFramesProgress("build", b.build_progress/b.build_time) # сменяет кадры постройки, если изображение указано как анимированное
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        trace('Cancel building upgrade')
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.building.show_clock(False)  # песочные часы ожидания
        if not self.is_building_started:        # если юнит еще не начал действие, то
            UStats.AddRes(self.resources_need)  # нужно возвращать стоимость апгрейда
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace("task canceled")
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.task = None
            unit.task_type = ""
            UMgr.add_free_unit(unit)
        b = self.building
        b.unreserve_action_cells()
        b.SM.SetState("normal")
        #b.switch_level()   # переключаемся на следующий вариант
        b.stop_task_participation(self)
        b.on_upgrade_cancel()
        self.building = None
# ============================================================
    def _get_building_units_count(self):
        count = 0
        for unit in self._units:
            _state = unit.SM.GetCurrentState()
            if _state == 'build':
                count += 1
        return count
# ============================================================
# ============================================================
    def on_external_signal(self, signal_id):
        if signal_id == 'try_more_units':
            if not self.try_call_unit():
                #self.create_unit_signal()   # если нужно, то позже позовем еще одного
            #else:   # юнита позвать не удалось
                self.create_unit_signal(500)   # попробуем еще раз через полсекунды
        pass
# ============================================================
    def on_bump_need(self):
        if self.building:
            self.building.scale_bump()
# #######################################################
# #######################################################
# #######################################################
