# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *
# ============================================================
# ============================================================
# ============================================================
class BInfo(object):
# ============================================================
    additive = ['functional', 'broken', 'attacked', 'survived', 'map_places', 'thrown',\
                'broken_active', 'upgrading', 'build', 'allowed_level_3']   # поля, которые числовые и их можно складывать с разных уровней
    def __init__(self):
        self.fields = {}
        self.fields['subtype'] = ''
        for name in self.additive:
            self.fields[name] = 0
        self.fields['can_upgrade'] = False
        self.fields['can_build'] = False
        self.fields['can_repair'] = False
        self.fields['res_enough_for_upgrade'] = False
        self.fields['res_enough_for_repair'] = False
        self.fields['min_eff'] = -1
        self.fields['max_eff'] = -1
# ============================================================
    def take_into_account(self, building):
        self.fields.subtype = building.get('stype')
        _b_state = building.SM.GetState()
        _has_access = building.HasAccess()
        _is_functional = building.is_functional()
        _is_broken_active = False
        self.fields.functional += int(_is_functional)
        if not building.is_thrown:  # если это не брошенное здание
            if _b_state != 'upgrade':
                _is_broken = int(building.is_broken())
                self.fields.broken += _is_broken
                if not building.is_deactivated:
                    if _is_broken:
                        self.fields.broken_active += _is_broken
                        _is_broken_active = True
        self.fields.attacked += int(building.is_under_attack)
        self.fields.survived += int(building.is_deactivated and building.status == 'normal')
        self.fields.map_places += int(building.status == 'map_place' and not building.is_deactivated)
        self.fields.thrown += int(building.is_thrown)
        self.fields.upgrading += int(_b_state == 'upgrade' and building.status == 'normal')
        #self.fields.build += int(_b_state == 'build' or (_b_state == 'awaiting_action' and building.status == 'place'))
        self.fields.build += int(_b_state == 'build' or building.status == 'place')
        _is_active = building.is_active()
        if _is_active:
            if _has_access:
                if _b_state not in ['upgrade', 'awaiting_action', 'build']:   # пока идет апгрейд - мы не можем определить сможет ли здание проапгрейдиться на следующий уровень
                    if _is_functional:
                        if not self.fields.can_upgrade:
                            self.fields.can_upgrade = True
                            _can_build = building.can_build()
                            if not _can_build:
                                self.fields.can_upgrade = False
                            if self.fields.can_upgrade:
                                self.fields.res_enough_for_upgrade = building.button_res_enough
                            _vname = building.get('variant_name')
                            if _vname == 'level_2':
                                if building.button_res_enough:
                                    if self.fields.can_upgrade:
                                        self.fields.allowed_level_3 += 1    # для того, чтобы знать, что здание можно апгрейдить до 3 уровня
                    elif _is_broken_active:
                        if not self.fields.can_repair:
                            self.fields.can_repair = True
                            _can_build = building.can_build()
                            if not _can_build:
                                self.fields.can_repair = False
                            if self.fields.can_repair:
                                self.fields.res_enough_for_repair = building.button_res_enough
                        #_vname = building.get('variant_name')
                        #if _vname == 'level_2' and building.button_res_enough and self.fields.can_upgrade:
                        #    self.fields.allowed_level_3 += 1    # для того, чтобы знать, что здание можно апгрейдить до 3 уровня
        _name = building.get('name')
        #_need_worker = not building.GetAttrBool('need_no_worker')
        #self.fields.can_build = base_building.BaseBuilding.can_build_static(_name, _need_worker)
        self.fields.can_build = BMgr.CanBuildPriority(_name)
        _eff = int(building.get_effectiveness_coeff() * 100)
        if self.fields.min_eff >= 0:
            if _eff >= 0:
                self.fields.min_eff = min(_eff, self.fields.min_eff)
            self.fields.max_eff = max(_eff, self.fields.max_eff)
        else:
            self.fields.min_eff = _eff
            self.fields.max_eff = _eff
# ============================================================
    def take_into_account_item(self, item):
        self.fields.subtype = item.get('stype')
        if not item.is_deactivated:
            if not item.is_action_complete:
                self.fields.functional += 1
# ============================================================
    def get_clone(self):
        bi = BInfo()
        bi.fields = copy(self.fields)
        return bi
# ============================================================
    def serialize(self, dest):
        dest.append('BInfo')
        dest.append(self.fields.subtype)
        for name in self.additive:
            dest.append(self.fields[name])
        dest.append(self.fields.can_upgrade)
        dest.append(self.fields.can_repair)
        dest.append(self.fields.res_enough_for_upgrade)
        dest.append(self.fields.res_enough_for_repair)
        dest.append(self.fields.can_build)
        dest.append(self.fields.min_eff)
        dest.append(self.fields.max_eff)
# ============================================================
    def deserialize(self, source):
        _marker = source.pop(0)
        if _marker != 'BInfo': mbox('Error in BInfo data (TutHelper) ' + str(_marker))
        self.fields.subtype = source.pop(0)
        for name in self.additive:
            self.fields[name] = source.pop(0)
        self.fields.can_upgrade = source.pop(0)
        self.fields.can_repair = source.pop(0)
        self.fields.res_enough_for_upgrade = source.pop(0)
        self.fields.res_enough_for_repair = source.pop(0)
        self.fields.can_build = source.pop(0)
        self.fields.min_eff = source.pop(0)
        self.fields.max_eff = source.pop(0)
# ============================================================
    def add(self, binfo):
        '''Складывает данные для одного типа здания с разных подуровней'''
        #trace(str(self.subtype))
        #trace(to_str(binfo))
        if self.fields.subtype == binfo.fields.subtype:
            for name in self.additive:
                self.fields[name] += int(binfo.fields[name])
            if binfo.fields.can_upgrade: self.fields.can_upgrade = True   # правда на разных уровнях могут быть разные требования...
            if binfo.fields.can_repair: self.fields.can_repair = True   # правда на разных уровнях могут быть разные требования...
            # ## self.res_enough_for_upgrade
            if binfo.fields.min_eff < 0:
                self.fields.min_eff = binfo.fields.min_eff
                self.fields.max_eff = binfo.fields.max_eff
            else:
                if binfo.fields.min_eff >= 0:
                    self.fields.min_eff = min(binfo.fields.min_eff, self.fields.min_eff)
                    self.fields.max_eff = max(binfo.fields.max_eff, self.fields.max_eff)
            # can_build не складываем - он на разных уровнях разный
# ============================================================
# ============================================================
# ============================================================
class GameInfoContainer(object):
# ============================================================
    def __init__(self):
        self.binfos = {}
        self.resources = {}
        self.units_count = 0
        self.is_units_count_critical = False
        self.has_started_quests = False
        self.has_tutorial_quests = False
# ============================================================
    def serialize(self, dest):
        dest.append('BI')
        _binfos_count = len(self.binfos)
        dest.append(_binfos_count)
        for n in self.binfos:
            bi = self.binfos[n]
            bi.serialize(dest)
        dest.append('OTHER')
        dest.append(self.units_count)
        dest.append(self.is_units_count_critical)
        dest.append(self.has_started_quests)
        dest.append(self.has_tutorial_quests)
# ============================================================
    def deserialize(self, source):
        if not source: return   # если нет данных, то ничего и не читаем
        if source.pop(0) != 'BI': mbox('Error in GameInfoContainer data (TutHelper)')
        _binfos_count = source.pop(0)
        for i in range(0, _binfos_count):
            bi = BInfo()
            bi.deserialize(source)
            self.binfos[bi.fields.subtype] = bi
        if source.pop(0) != 'OTHER': mbox('Error in GameInfoContainer data 2 (TutHelper)')
        self.units_count = source.pop(0)
        self.is_units_count_critical = source.pop(0)
        self.has_started_quests = source.pop(0)
        self.has_tutorial_quests = source.pop(0)
# ============================================================
    def read_info(self):
        # читаем данные по зданиям:
        self.read_info_buildings()
        self.read_info_resources()
        self.read_info_other()
# ============================================================
    def read_info_buildings(self):
        '''Читаем данные о всех зданиях на данном подуровне'''
        _buildings = BMgr.all_buildings
        for b in _buildings:
            if b:   # на всякий случай
                _has_access = b.HasAccess()     # иногда здесь падает!!! Но как?!
                _is_passable = b.IsPassable()
                if _has_access and _is_passable:     # недоступные нас не интересуют
                    if not b.is_indians and b.subtype != 'bridge' or\
                        b.subtype in ['totem', 'totem_stone']:      # без мостов и индейских зданий! Но с тотемами
                            _stype = b.subtype
                            _bi = None
                            if _stype not in self.binfos:   # если такое здание еще не встречалось
                                _bi = BInfo()
                                self.binfos[_stype] = _bi
                            else:
                                _bi = self.binfos[_stype]
                            _bi.take_into_account(b)
        _items = BMgr.all_items
        for item in _items:
            if item:   # на всякий случай
                _has_access = b.HasAccess()
                _is_passable = b.IsPassable()
                if _has_access and _is_passable:     # недоступные нас не интересуют
                    if item.subtype == 'granite':
                        _has_access = item.HasAccess()
                        if _has_access:
                            _is_active = item.is_active()
                            if _is_active:
                                _stype = item.get('subtype')
                                _bi = None
                                if _stype not in self.binfos:   # если такой объект еще не встречался
                                    _bi = BInfo()
                                    self.binfos[_stype] = _bi
                                else:
                                    _bi = self.binfos[_stype]
                                _bi.take_into_account_item(item)
# ============================================================
    res_names = ['food', 'boards','bricks', 'coins', 'gold', 'gem_red', 'gem_green',
		              'gem_blue', '_wood', '_stones', '_granite', '_flour',
		              '_bread', '_ore', '_berries', '_fish', '_fish_sm']
    def read_info_resources(self):
        '''Читаем данные о текущих ресурсах'''
        _res = UStats.GetCurRes()
        for name in self.res_names:
            self.resources[name] = _res.Get(name)
# ============================================================
    def read_info_other(self):
        '''Читаем прочую полезную информацию'''
        self.units_count = UMgr.get_free_real_units_count()
        self.is_units_count_critical = UMgr.is_free_real_units_enough()
        self.has_started_quests = QMgr.has_started_quests()
        self.has_tutorial_quests = QMgr.has_started_tut_quest()
# ============================================================
    def add(self, gic):
        for n in gic.binfos:
            _bi = gic.binfos[n]
            if n in self.binfos:
                bi = self.binfos[n]
                bi.add(_bi)
            else:
                bi = BInfo()
                bi.fields.subtype = n   # из-за этого и был баг
                self.binfos[n] = bi
                bi.add(_bi)
        # больше вроде и не требуется
        if not self.has_tutorial_quests:
            if gic.has_tutorial_quests:
                self.has_tutorial_quests = gic.has_tutorial_quests
# ============================================================
    def get_clone(self):
        gic = GameInfoContainer()
        for n in self.binfos:
            gic.binfos[n] = self.binfos[n].get_clone()
        #for n in self.resources: gic.resources[n] = self.resources[n]
        #gic.binfos = copy(self.binfos)
        gic.resources = copy(self.resources)
        gic.units_count = self.units_count
        gic.is_units_count_critical = self.is_units_count_critical
        gic.has_started_quests = self.has_started_quests
        return gic
# ============================================================
# ============================================================
# ============================================================
class EventsContainer(object):
    def __init__(self):
        self.events = {}
        self.vars = {}      # просто некие временные переменные
        self.cur_ms = 0
# ============================================================
    def on_create(self):
        pass
# ============================================================
    def serialize(self, dest):
        dest.append('EC')
        dest.append(self.cur_ms)
        _events_count = len(self.events)
        dest.append(_events_count)
        for n in self.events:
            dest.append(n)
            dest.append(self.events[n])
        _vars_count = len(self.vars)
        dest.append(_vars_count)
        for n in self.vars:
            dest.append(n)
            dest.append(self.vars[n])
# ============================================================
    def deserialize(self, source):
        if not source: return   # если нет данных, то ничего и не читаем
        if source.pop(0) != 'EC': mbox('Error in EventsContainer data (TutHelper)')
        self.cur_ms = source.pop(0)
        _events_count = source.pop(0)
        for i in range(0, _events_count):
            n = source.pop(0)
            v = source.pop(0)
            self.events[n] = v
        _vars_count = source.pop(0)
        for i in range(0, _vars_count):
            n = source.pop(0)
            v = source.pop(0)
            self.vars[n] = v
# ============================================================
    def add_var(self, _var, _val = 0):
        '''Добавить событие с текущим временем'''
        self.vars[_var] = _val
# ============================================================
    def remove_var(self, _var):
        '''Удалить событие'''
        if _var in self.vars:
            del self.vars[_var]
# ============================================================
    def get_var(self, _var):
        '''Удалить событие'''
        if _var in self.vars:
            return self.vars[_var]
        else:
            return 0
# ============================================================
    def has_var(self, _var):
        '''Есть ли запрашиваемое событие'''
        return (_var in self.vars)
# ============================================================
    def add(self, event):
        '''Добавить событие с текущим временем'''
        self.events[event] = self.cur_ms
# ============================================================
    def remove(self, event):
        '''Удалить событие'''
        if event in self.events:
            del self.events[event]
# ============================================================
    def has(self, event):
        '''Есть ли запрашиваемое событие'''
        return (event in self.events)
# ============================================================
    def is_event_time_more(self, event, time):
        '''Есть ли запрашиваемое событие'''
        if event in self.events:
            if (self.cur_ms - self.events[event]) >= time:
                return True
        return False
# ============================================================
    def is_event_spoiled(self, event, time):
        return not self.has(event) or self.is_event_time_more(event, time) # если нет сообщения или прошло достаточно времени
# ============================================================
    def get_clone(self):
        ec = EventsContainer()
        ec.cur_ms = self.cur_ms
        ec.events = copy(self.events)
        return ec
# ============================================================

# ============================================================
# ============================================================
# ============================================================
class TutHelper(object):
# ============================================================
    # - нужно с определенной периодичностью собирать данные, сохраняя предыдущие
    # -
    def __init__(self):
        self.ticks = 0
        self.level_settings = None
        self.local_info = None
        self.global_info = None
        self.local_prev_info = None
        self.global_prev_info = None
        self.ec = EventsContainer()
        self.other_ec = []
        self._li = None
        self._lip = None
        self._gi = None
        self._gip = None
        self._binfos = None
        self.is_effectivness_cloud_started = False
# ============================================================
    def has_functional(self, stype):
        if stype in self._binfos and self._binfos[stype].fields.functional > 0:
            return True
        return False
# ============================================================
    def on_create(self):
        self.level_settings = BMgr.GetLevelSettings()
        if not self.is_cave():
            self.refresh_info() # чтобы избавиться от None значений в self._lip и self._gip
        self.ec.add('msg_13')   # чтобы показывать периодическое сообщение
        self.ec.add('msg_6')    # для апгрейдов, чтобы не ранее, чем через 20 секунд
        if LBoard.Core.subindex == 0:   # только для главных уровней
            if LBoard.Core.episode == 1:
                self.ec.add('msg_22')
                self.ec.add('msg_23')
            elif LBoard.Core.episode == 2:
                self.ec.add('msg_24')
                self.ec.add('msg_25')
# ============================================================
    def on_update(self, ms):
        if LBoard.Core.episode == 2 and LBoard.Core.level == 6:  # здесь никаких подсказок
            return
        self.ec.cur_ms += ms
        if TMgr.IsCurrentLocationActive():  # остальные будут обновляться текущим активным уровнем
            _is_cave = self.is_cave()
            if not _is_cave:
                #if self.ticks > 1000000: self.ticks = 0
                self.ticks += ms
                if self.ticks >= 3000:
                    self.ticks = 0
                    _progress = UI.calc_cur_progress()
                    if _progress < 0.9:  # когда осталось 10% генеральского бара соревнования, то показывать подсказки не нужно
                        self.refresh_info()
                        self.generate_messages()
# ============================================================
    def is_cave(self):
        '''Текущий уровень является пещерой?'''
        if 'level_type' in self.level_settings and \
            self.level_settings.level_type == 'underlevel':
            return True
        return False
# ============================================================
    def on_signal(self, signal):
        '''Своей сигнальной машины у TutHelper нет, поэтому используем от TMgr,
        все сигналы должны начинаться на "th_"! Тогда они будут сюда перенаправлены'''
        if signal == 'th_effectiveness':
            _mode = LBoard.GetUserInfo('game_mode')
            if _mode == 'hard':
                return
            TMgr.Signals.Restart('th_effectiveness', 13*60*1000)    # каждые 13 минут
            _is_cur_active = TMgr.IsCurrentLocationActive()
            if _is_cur_active:
                UI.show_top_text('KEEP_INMIND_EFFECTIVENESS')
# ============================================================
    def on_global_event(self, event, param):
        #_is_cur_active = TMgr.IsCurrentLocationActive()
        #if _is_cur_active:
        if event == 'effectiveness_80': # чтобы отследить, когда эффективность здания упадет ниже 80%
            if not self.is_effectivness_cloud_started:
                self.is_effectivness_cloud_started = True
                TMgr.Signals.Restart('th_effectiveness', 1000)
# ============================================================
    def on_event(self, event_name, extra):
        if event_name == 'priority_complete':
            _has_priority = self.ec.has_var('priority_msg_complete')
            if not _has_priority:
                self.ec.add('priority_msg')
        elif event_name in ['build_building_complete', 'build_repair_complete']:
            if 'subtype' in extra:
                if extra.subtype == 'woodcutter':
                    self.ec.add('woodcutter_functional')
                    self.ec.add('woodcutter_functional2')
        if event_name in ['build_building_complete', 'build_repair_complete', 'build_upgrade_complete']:    # намеренно IF, а не ELIF
            if 'subtype' in extra:
                self.ec.add('msg_6')    # обновим время для апгрейдов
# ============================================================
    def serialize(self, result):
        if self.local_info:
            self.local_info.serialize(result)
            self.ec.serialize(result)
# ============================================================
    def refresh_info(self):
        self.other_ec = []
        self.local_prev_info = self.local_info
        self.local_info = GameInfoContainer()       # каждый раз создаем новый объект
        self.local_info.read_info()
        self.other_ec = []
        # нужно глобальные данные еще прочитать
        _is_cur_loc_active = TMgr.IsCurrentLocationActive()
        if _is_cur_loc_active:
            game_infos = TMgr.GetOtherLevelsGameInfo('tut_helper')
            self.global_prev_info = self.global_info
            # нужно склонировать данные из local_info в global_info и
            self.global_info = self.local_info.get_clone()
            for _l in game_infos:
                gic = GameInfoContainer()
                gic.deserialize(_l)
                ec = EventsContainer()
                ec.deserialize(_l)
                self.other_ec.append(ec)
                self.global_info.add(gic)   # добавить данные других уровней
# ============================================================
    def show_msg(self, text_id, extra = {}):
        '''Метод-фильтр, чтобы на скрытых уровнях не показывать подсказки'''
        _is_cur_active = TMgr.IsCurrentLocationActive()
        if _is_cur_active:
            UI.show_tut_text(text_id, False, '', extra)
# ============================================================
    def has_enough_res_for_build(self, template_name):
        '''Достаточно ли ресурсов для постройки объекта'''
        _res = BMgr.GetTemplateRes(template_name, 'normal_needs_1')
        return UStats.IsResEnough(_res)
# ============================================================
    def is_build_priority_allowed(self, template_name):
        '''Можно ли строить данный шаблон по приоритету'''
        _can_build_priority = BMgr.CanBuildPriority(template_name)
        return _can_build_priority
# ============================================================
    def generate_messages(self):
        self._li = self.local_info
        self._lip = self.local_prev_info
        self._gi = self.global_info
        self._gip = self.global_prev_info
        self._binfos = self._gi.binfos
        #if 'garden' in self._binfos:
        #    trace(self._binfos.garden.fields)
        if self.local_info.has_tutorial_quests: # когда запущены туториальные квесты - сообщения не выводим
            return False
        _mode = LBoard.GetUserInfo('game_mode')
        if _mode == 'hard':
            return
        _functions = [self.msg_6, self.msg_7, self.msg_8, self.msg_9, \
                        self.msg_10, self.msg_11, self.msg_12, self.msg_13,\
                        self.msg_14, self.msg_15, self.msg_16, self.msg_17, \
                        self.msg_18, self.msg_19,\
                        self.msg_20, self.msg_21, self.msg_22, self.msg_23, \
                        self.msg_24, self.msg_25, self.msg_26, self.msg_27, \
                        self.msg_28, self.msg_29, self.msg_30, self.msg_31, \
                        self.msg_32, self.msg_34, self.msg_35, self.msg_36, \
                        self.msg_37 ]
        for func in _functions:
            _result = func()
            if _result: return
# ============================================================
    def msg_6(self):
        '''6 any building - upgrade'''
        _msg = 'msg_6'
        _msg2 = _msg + '_time'
        _can_run = ( not self.ec.has(_msg2) and not self.ec.has(_msg) )
        if not _can_run:
            if self.ec.has(_msg2):
                _can_run = self.ec.is_event_time_more(_msg2, 600000)
            else:
                _can_run = self.ec.is_event_time_more(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            if self.ec.is_event_spoiled(_msg, 20000):   # на случай, когда было событие апгрейда/постройки/ремонта, но давно, либо не было такого
                for stype in self._li.binfos:      # пробегаем все здания
                    if stype != 'watchtower':       # башни скипуем
                        _bi = self._binfos[stype]
                        if _bi.fields.functional > 0 and \
                            _bi.fields.can_upgrade and \
                            _bi.fields.res_enough_for_upgrade:# если можно апгрейдить и достаточно средств
                                trace('>msg_6>', stype)
                                self.show_msg('TUT_HELP_06', {'id':6})
                                self.ec.add(_msg2)
                                return True
            self.ec.remove(_msg2)
            self.ec.add(_msg)
        return False
# ============================================================
    lvl_skip_msg_7 = [(1,1,0), (2,4,3)]
    def msg_7(self):
        '''7 house - build'''
        for lvl in self.lvl_skip_msg_7: # пропускаем некоторые уровни:
            e,lindex,s = lvl
            if LBoard.Core.episode == e:
                if LBoard.Core.level == lindex:
                    if LBoard.Core.subindex == s:
                        return False
        _msg = 'msg_7'
        _msg_once = 'msg_7_once'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if self._li.units_count > 10:
                self.ec.remove_var(_msg_once)   # удалим переменную
            if self._li.units_count <= 10 and self._li.units_count > 7: # в диапазоне (7,11]
                _already_shown = self.ec.has_var(_msg_once)
                if _already_shown: return False     # если уже показывали сообщение
                if 'watchtower' in self._li.binfos:
                    if self._li.binfos.watchtower.fields.functional > 0:
                        _show = False
                        if 'house' not in self._li.binfos:
                            _show = True
                        else:
                            _house = self._li.binfos.house
                            if (_house.fields.upgrading+_house.fields.build) == 0:
                                _show = True
                        _can_build_priority = self.is_build_priority_allowed('house1')
                        if _show:
                            if _can_build_priority:
                                self.show_msg('TUT_HELP_07', {'id':7, 'blink_build_button':True})
                                self.ec.add_var(_msg_once)      # чтобы показать только один раз
                                return True
        return False
# ============================================================
    def msg_8(self):
        '''8 woodcutter - build'''
        _msg = 'msg_8'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if self._li.resources._wood < 10 and self._li.resources.boards < 50:
                _show = False
                if 'woodcutter' not in self._binfos:
                    _show = False   # потому что добавили условие заброшенного woodcutter
                else:
                    _woodcutter = self._binfos.woodcutter
                    if _woodcutter.fields.functional == 0 \
                        and (_woodcutter.fields.broken + _woodcutter.fields.survived) == 0:
                            if _woodcutter.fields.thrown > 0:
                                _show = True
                if _show:
                    self.show_msg('TUT_HELP_08', {'id':8, 'blink_build_button':True})
                    return True
        return False
# ============================================================
    def msg_9(self):
        '''9 garden - build'''
        _msg = 'msg_9'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if self._li.resources.food < 50:
                if 'garden' in self._binfos:
                    _garden = self._binfos.garden
                    #if _garden.fields.functional == 1:
                    if _garden.fields.max_eff < 60:
                        if not _garden.fields.can_upgrade:
                            self.show_msg('TUT_HELP_09', {'id':9, 'blink_build_button':True})
                            trace('!>! :', _garden.fields.max_eff, _garden.fields)
                            return True
        return False
# ============================================================
    def msg_10(self):
        '''10 sawmill - build'''
        _msg = 'msg_10'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if self._li.resources.boards < 100:
                if self._li.resources.boards >= 15 and self._li.resources.food >= 15:
                #_is_res_enough = self.has_enough_res_for_build('woodcutter1')
                #if _is_res_enough:
                    if 'sawmill' in self._binfos and 'woodcutter' in self._binfos:
                        _sawmill = self._binfos.sawmill
                        _woodcutter = self._binfos.woodcutter
                        _diff = _woodcutter.fields.functional - _sawmill.fields.functional
                        if _diff >= 2 and not _sawmill.fields.can_upgrade:
                            self.show_msg('TUT_HELP_10', {'id':10, 'blink_build_button':True})
                            return True
        return False
# ============================================================
    def msg_11(self):
        '''11 sawmill - build'''
        _msg = 'msg_11'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if self._li.resources._wood >= 5 and self._li.resources.boards < 100:
                if self._li.resources.boards >= 15 and self._li.resources.food >= 15:
                #_is_res_enough = self.has_enough_res_for_build('sawmill1')
                #if _is_res_enough:
                    if 'sawmill' in self._binfos:
                        _sawmill = self._binfos.sawmill
                        #trace('<>EFFECTIVENESS:', _sawmill.fields.min_eff, _sawmill.fields.max_eff)
                        if _sawmill.fields.max_eff < 100 and _sawmill.fields.min_eff < 100:
                            self.show_msg('TUT_HELP_11', {'id':11, 'blink_build_button':True})
                            return True
        return False
# ============================================================
    def msg_12(self):
        '''12 granitemine - build'''
        _msg = 'msg_12'
        if LBoard.Core.episode == 1 and LBoard.Core.level == 2:
            return False
        _can_run = self.ec.is_event_spoiled(_msg, 300000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            _show = False
            _has_res_source = False
            if 'granitemine' in self._li.binfos:
                if self._li.binfos.granitemine:
                    _granitemine = self._li.binfos.granitemine
                    if _granitemine.fields.map_places > 0:
                        _has_res_source = True
                    if _granitemine.fields.functional:
                        if _granitemine.fields.functional == 1:     # только одна шахта
                            if _granitemine.fields.broken == 0:     # сломанных шахт нет
                                if _granitemine.fields.min_eff < 50:# эффективность < 50%
                                    _show = True
            if not _has_res_source:
                if 'granite' in self._li.binfos:
                    if self._li.binfos.granite.fields.functional > 0:
                        _has_res_source = True
            if _show:
                if 'quarry' not in self._li.binfos:
                    pass
                else:
                    _quarry = self._li.binfos.quarry
                    if _quarry.fields.functional > 0 or _quarry.fields.broken > 0:
                        return False
            if _show:
                if  _has_res_source:
                    _can_priority = self.is_build_priority_allowed('granitemine1')
                    if _can_priority:
                        self.show_msg('TUT_HELP_12', {'id':12, 'blink_build_button':True})
                        return True
        return False
# ============================================================
    def msg_13(self):
        '''13 warehouse - build'''
        _msg = 'msg_13'
        #_can_run = self.ec.is_event_time_more(_msg, 30000)
        if LBoard.Core.episode == 1:
            if LBoard.Core.level == 1:
                return False
        _can_run = self.ec.is_event_time_more(_msg, 900000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            _show = False
            if 'warehouse' not in self._binfos:
                _show = True
            else:
                _w = self._binfos.warehouse
                if (_w.fields.functional + _w.fields.broken + _w.fields.survived) == 0:
                    if _w.fields.can_build:
                        _show = True
            self.ec.remove(_msg)
            self.ec.add(_msg)   # на случай если стало возможно строить склад
            if _show:
                #_is_res_enough = self.has_enough_res_for_build('warehouse1')
                #if _is_res_enough:
                    self.ec.add('msg_14')   # чтобы через некоторое время показать 14 сообщение
                    self.show_msg('TUT_HELP_13', {'id':13, 'blink_build_button':True})
                    return True
        return False
# ============================================================
    def msg_14(self):
        '''14 house - build'''
        _msg = 'msg_14'
        _can_run = self.ec.is_event_time_more(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.show_msg('TUT_HELP_14', {'id':14, 'blink_build_button':True})
            return True
        return False
# ============================================================
    def msg_15(self):
        '''15 watchtower - build'''
        _msg = 'msg_15'
        if self.ec.is_event_spoiled(_msg, 40000):   # если готово событие или его не было
            self.ec.remove(_msg)                    # удалим событие
            if 'watchtower' in self._li.binfos:        # если есть на карте башня
                #! добавить проверку на достаточночть ресурсов для постройки
                _r = self._li.resources
                if _r.food >= 15 and _r.bricks >= 40 and _r.coins >= 40:    # если есть ресурсы на строительство
                    _wt = self._li.binfos.watchtower
                    if _wt.fields.map_places > 0:       # если есть места для строительства
                        if _wt.fields.can_build:        # если строительство допустимо сейчас
                            if self.ec.has('pre_' + _msg):
                                if self.ec.is_event_time_more('pre_' + _msg, 20000):
                                    self.ec.remove('pre_' + _msg)
                                    self.ec.add(_msg)
                                    _fcount = self.ec.get_var('var_15_fcount')
                                    self.ec.remove_var('var_15_fcount')
                                    if _fcount != _wt.fields.functional:    # если за 20 секунд была построена хоть одна башня
                                        return False
                                    else:
                                        self.show_msg('TUT_HELP_15', {'id':15, 'blink_build_button':True})
                                        return True
                            else:
                                self.ec.add('pre_' + _msg)
                                self.ec.add(_msg)
                                self.ec.add_var('var_15_fcount', _wt.fields.functional)
        return False
# ============================================================
    def msg_16(self):
        '''16 watchtower - upgrade'''
        _msg = 'msg_16'
        if self.ec.is_event_spoiled(_msg, 40000):   # если готово событие или его не было
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if 'watchtower' in self._binfos:
                _wt = self._binfos.watchtower
                _ready = False
                _totem_disallow = False
                if _wt.fields.broken == 0 and _wt.fields.survived == 0 and \
                    _wt.fields.map_places == 0 and _wt.fields.functional > 0:
                    if 'totem' in self._binfos:
                        _totem = self._binfos.totem
                        if _totem.fields.broken == 0:
                            _ready = True
                        else:
                            _totem_disallow = True
                    if 'totem_stone' in self._binfos and not _totem_disallow:
                        _totem = self._binfos.totem_stone
                        if _totem.fields.broken == 0:
                            _ready = True
                        else:
                            _ready = False
                    if _ready:
                        if _wt.fields.can_upgrade and _wt.fields.res_enough_for_upgrade:
                            if self.ec.has_var('var_16_functional_count'):
                                _f_count = self.ec.get_var('var_16_functional_count')
                                self.ec.remove_var('var_16_functional_count')
                                if _f_count == _wt.fields.functional:   # за 20 секунд не было построено/получено новых башен
                                    self.show_msg('TUT_HELP_16', {'id':16})
                                    return True
                            else:
                                self.ec.add_var('var_16_functional_count', _wt.fields.functional)   # сохраняем
                    else:
                        self.ec.remove_var('var_16_functional_count')   # на всякий случай
                else:
                    self.ec.remove_var('var_16_functional_count')   # на всякий случай
        return False
# ============================================================
    def msg_17(self):
        '''17 watchtower - upgrade level_3'''
        _msg = 'msg_17'
        if self.ec.is_event_spoiled(_msg, 40000):   # если готово событие или его не было
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if 'watchtower' in self._li.binfos:
                _wt = self._li.binfos.watchtower
                if _wt.fields.allowed_level_3 > 0:  # есть что апгрейдить до 3 уровня
                    if self.ec.has_var('var_17_functional_count'):
                        _f_count = self.ec.get_var('var_17_functional_count')
                        self.ec.remove_var('var_17_functional_count')
                        if _f_count == _wt.fields.functional:   # за 20 секунд не было построено/получено новых башен
                            self.show_msg('TUT_HELP_17', {'id':17})
                            return True
                    else:
                        self.ec.add_var('var_17_functional_count', _wt.fields.functional)   # сохраняем
                else:
                    self.ec.remove_var('var_17_functional_count')   # на всякий случай
        return False
# ============================================================
    def msg_18(self):
        '''18 stonecutter - build'''
        _msg = 'msg_18'
        _can_run = self.ec.is_event_spoiled(_msg, 180000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            _has_f = self.has_functional('granitemine') or self.has_functional('quarry')
            if _has_f:
                _show = False
                if 'stonecutter' not in self._binfos:
                    _show = True
                else:
                    _s = self._binfos.stonecutter
                    if (_s.fields.functional + _s.fields.broken + _s.fields.survived) == 0:
                        _show = True
                    #if not _s.fields.res_enough_for_upgrade: _show = False  # затычка от недостатка ресурсов
                if _show:
                    self.show_msg('TUT_HELP_18', {'id':18, 'blink_build_button':True})
                    self.ec.add(_msg)
                    return True
        return False
# ============================================================
    def msg_19(self):
        '''19 mint - build'''
        _msg = 'msg_19'
        _can_run = self.ec.is_event_spoiled(_msg, 180000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            if 'silvermine' in self._binfos and self._binfos.silvermine:    # наличие глобально
                if self._binfos.silvermine.fields.functional:
                    _show = False
                    if 'mint' not in self._binfos:   # наличие тоже глобально на подуровне
                        _show = True
                    else:
                        _mint = self._binfos.mint
                        if _mint.fields.functional == 0:
                            if _mint.fields.broken == 0:
                                _show = True
                    if _show:
                        self.show_msg('TUT_HELP_19', {'id':19, 'blink_build_button':True})
                        self.ec.add(_msg)
                        return True
        return False
# ============================================================
    def msg_20(self):
        '''20 bakery - build'''
        _msg = 'msg_20' # repeat msg
        _msg_wait = _msg + '_wait'  # first wait msg
        if self.ec.is_event_spoiled(_msg, 5000):   # если готово событие или его не было
            self.ec.remove(_msg)
            if self.ec.is_event_spoiled(_msg_wait, 180000):   # если готово событие или его не было
                self.ec.remove(_msg_wait)
                if 'farm' in self._binfos:
                    _farm = self._binfos.farm
                    _ready = False
                    if _farm.fields.functional > 0: # есть рабочая ферма
                        _show = False
                        if 'bakery' not in self._binfos: _show = True
                        else:
                            _bakery = self._binfos.bakery
                            _sum = _bakery.fields.broken+_bakery.fields.survived+_bakery.fields.functional
                            if _sum == 0: _show = True
                            else: self.ec.remove('pre_' + _msg)
                        if _show:
                            if self.ec.has('pre_' + _msg):
                                self.ec.remove('pre_' + _msg)
                                self.show_msg('TUT_HELP_20', {'id':20, 'blink_build_button':True})
                                self.ec.add(_msg)       # 5s
                                return True
                            else:
                                self.ec.add('pre_' + _msg)
                                self.ec.add(_msg_wait)  # 20s
        return False
# ============================================================
    def msg_21(self):
        '''21 smokery - build'''
        _msg = 'msg_21' # repeat msg
        _msg_wait = _msg + '_wait'  # first wait msg
        if self.ec.is_event_spoiled(_msg, 5000):   # если готово событие или его не было
            self.ec.remove(_msg)
            if self.ec.is_event_spoiled(_msg_wait, 180000):   # если готово событие или его не было
                self.ec.remove(_msg_wait)
                if 'fisherman' in self._binfos:
                    _fisherman = self._binfos.fisherman
                    _ready = False
                    if _fisherman.fields.functional > 0: # есть рабочая ферма
                        _show = False
                        if 'smokery' not in self._binfos: _show = True
                        else:
                            _smokery = self._binfos.smokery
                            _sum = _smokery.fields.broken+_smokery.fields.survived+_smokery.fields.functional
                            if _sum == 0: _show = True
                            else: self.ec.remove('pre_' + _msg)
                        if _show:
                            if self.ec.has('pre_' + _msg):
                                self.ec.remove('pre_' + _msg)
                                self.show_msg('TUT_HELP_21', {'id':21, 'blink_build_button':True})
                                self.ec.add(_msg)       # 5s
                                return True
                            else:
                                self.ec.add('pre_' + _msg)
                                self.ec.add(_msg_wait)  # 20s
        return False
# ============================================================
    def msg_22(self):
        '''22 garden - build'''
        _msg = 'msg_22'
        _has_msg = self.ec.has(_msg)
        if _has_msg:    # генерим единожды и только когда есть сообщение с начала уровня
            _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                _show = False
                if 'garden' not in self._binfos:
                    _show = True
                else:
                    _g = self._binfos.garden
                    if (_g.fields.functional + _g.fields.broken + _g.fields.survived) == 0:
                        _show = True
                if _show:
                    _has_quest = QMgr.has_started_quests()
                    if _has_quest:
                        self.ec.add(_msg)
                        return False
                    self.show_msg('TUT_HELP_22', {'id':22, 'blink_build_button':True})
                    self.ec.add(_msg)   # чтобы повторить через время
                    return True
        return False
# ============================================================
    def msg_23(self):
        '''23 garden - repair'''
        _msg = 'msg_23'
        _has_msg = self.ec.has(_msg)
        if _has_msg:    # генерим единожды и только когда есть сообщение с начала уровня
            _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                _show = False
                if 'garden' in self._li.binfos:
                    _g = self._li.binfos.garden
                    if _g.fields.broken_active > 0 and _g.fields.functional == 0:
                        _show = True
                if _show:
                    #_has_quest = QMgr.has_started_quests()
                    #if _has_quest:
                    #    self.ec.add(_msg)
                    #    return False
                    self.show_msg('TUT_HELP_23', {'id':23})
                    return True
        return False
# ============================================================
    def msg_24(self):
        '''24 fisherman - build'''
        _msg = 'msg_24'
        _has_msg = self.ec.has(_msg)
        if _has_msg:    # генерим единожды и только когда есть сообщение с начала уровня
            _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                _show = False
                if 'fisherman' not in self._binfos:
                    _show = True
                else:
                    _f = self._binfos.fisherman
                    if (_f.fields.functional + _f.fields.broken + _f.fields.survived) == 0 and\
                        _f.fields.map_places > 0:
                        _show = True
                if _show:
                    _has_quest = QMgr.has_started_quests()
                    if _has_quest:
                        self.ec.add(_msg)
                        return False
                    self.show_msg('TUT_HELP_24', {'id':24, 'blink_build_button':True})
                    return True
        return False
# ============================================================
    def msg_25(self):
        '''25 mint - repair'''
        _msg = 'msg_25'
        _has_msg = self.ec.has(_msg)
        if _has_msg:    # генерим единожды и только когда есть сообщение с начала уровня
            _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                _show = False
                if 'fisherman' in self._li.binfos:
                    _f = self._li.binfos.fisherman
                    if _f.fields.broken_active > 0 and _f.fields.functional == 0:
                        _show = True
                if _show:
                    #_has_quest = QMgr.has_started_quests()
                    #if _has_quest:
                    #    self.ec.add(_msg)
                    #    return False
                    self.show_msg('TUT_HELP_25', {'id':25})
                    return True
        return False
# ============================================================
    def msg_26(self):
        '''26 woodcutter - build'''
        _msg = 'priority_msg'
        _complete = self.ec.has_var('priority_msg_complete')
        if _complete:
            return False
        _has_event = self.ec.has(_msg)
        if _has_event:
            _can_run = self.ec.is_event_time_more(_msg, 20000)
            if _can_run:
                _show = False
                if 'woodcutter' in self._binfos:
                    if self._binfos.woodcutter.fields.functional > 0:
                        self.ec.add_var('priority_msg_complete', 0)
                        self.ec.remove(_msg)
                        return False
                if 'woodcutter' not in self._binfos:
                    _show = True
                else:
                    _w = self._binfos.woodcutter
                    if (_w.fields.broken+_w.fields.functional+_w.fields.survived) == 0:
                        _show = True
                if 'woodcutter' in self._binfos:
                    if not self._binfos.woodcutter.fields.can_build:
                        if _w.fields.broken == 0:   # чтобы не сбрасывать время для msg_17
                            self.ec.remove(_msg)
                            self.ec.add(_msg)
                        return False
                if _show:
                    self.show_msg('TUT_HELP_26', {'id':26, 'blink_build_button':True})
                    self.ec.remove(_msg)
                    self.ec.add(_msg)
                    return True
        return False
# ============================================================
    def msg_27(self):
        '''27 woodcutter - repair'''
        _msg = 'priority_msg'
        _complete = self.ec.has_var('priority_msg_complete')
        if _complete:
            return False
        _has_event = self.ec.has(_msg)
        if _has_event:
            _can_run = self.ec.is_event_time_more(_msg, 20000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                _show = False
                if 'woodcutter' in self._binfos and self._binfos.woodcutter.fields.functional > 0:
                    self.ec.add_var('priority_msg_complete', 0)
                    self.ec.remove(_msg)
                    return False
                if 'woodcutter' in self._binfos:
                    _w = self._binfos.woodcutter
                    if _w.fields.broken_active > 0:
                        _show = True
                    if not _w.fields.can_repair:
                        self.ec.remove(_msg)
                        self.ec.add(_msg)
                        return False
                if _show:
                    self.show_msg('TUT_HELP_27', {'id':27})
                    self.ec.remove(_msg)
                    self.ec.add(_msg)
                    return True
        return False
# ============================================================
    def msg_28(self):
        '''28 sawmill - build'''
        _msg = 'woodcutter_functional'
        _msg2 = 'sawmill_functional'
        _complete = self.ec.has_var('woodcutter_functional_complete')
        if _complete:
            return False
        _has_event = self.ec.has(_msg) or self.ec.has(_msg2)
        if _has_event:
            _can_run = self.ec.is_event_time_more(_msg, 20000) or\
                        self.ec.is_event_time_more(_msg2, 180000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                self.ec.remove(_msg2)
                _show = False
                if 'sawmill' not in self._binfos:
                    _show = True
                else:
                    _sawmill = self._binfos.sawmill
                    if _sawmill.fields.functional == 0 and _sawmill.fields.broken_active == 0:
                        _show = True
                    elif _sawmill.fields.functional > 0:
                        self.ec.remove(_msg2)
                        self.ec.add_var('woodcutter_functional_complete', 0)
                        UI.remove_tut_text('TUT_HELP_28', {'id':28})
                if _show:
                    self.show_msg('TUT_HELP_28', {'id':28})
                    self.ec.add(_msg2)   # чтобы повтор был
                    #self.ec.add_var('woodcutter_functional_complete', 0)
                    return True
        return False
# ============================================================
    def msg_29(self):
        '''29 sawmill - repair'''
        _msg = 'woodcutter_functional2'
        _msg2 = 'sawmill_functional2'
        _complete = self.ec.has_var('woodcutter_functional_complete')
        if _complete:
            return False
        _has_event = self.ec.has(_msg) or self.ec.has(_msg2)
        if _has_event:
            _can_run = self.ec.is_event_time_more(_msg, 20000) or\
                        self.ec.is_event_time_more(_msg2, 180000) # если нет сообщения или прошло достаточно времени
            if _can_run:
                self.ec.remove(_msg)
                self.ec.remove(_msg2)
                self.ec.add(_msg2)   # чтобы повтор был
                _show = False
                if 'sawmill' in self._li.binfos:
                    _sawmill = self._li.binfos.sawmill
                    if _sawmill.fields.functional == 0 and _sawmill.fields.broken_active == 1:
                        _show = True
                    elif _sawmill.fields.functional > 0:
                        self.ec.remove(_msg2)
                        self.ec.add_var('woodcutter_functional_complete', 0)
                        UI.remove_tut_text('TUT_HELP_29', {'id':29})
                if _show:
                    self.show_msg('TUT_HELP_29', {'id':29})
                    self.ec.add(_msg2)
                    return True
        return False
# ============================================================
    def msg_30(self):
        '''30 granitemine - build'''
        _msg = 'msg_30'
        if self.ec.is_event_spoiled(_msg, 40000):   # если готово событие или его не было
            self.ec.remove(_msg)                    # удалим событие
            if 'granitemine' in self._binfos:        # если есть на карте шахта
                #! добавить проверку на достаточночть ресурсов для постройки
                #_r = self._li.resources
                #if _r.food >= 15 and _r.bricks >= 40 and _r.coins >= 40:    # если есть ресурсы на строительство
                _g = self._binfos.granitemine
                if _g.fields.map_places > 0 and _g.fields.can_build:       # если есть места для строительства
                    if _g.fields.can_build:        # если строительство допустимо сейчас
                        _can_priority = self.is_build_priority_allowed('granitemine1')
                        if _can_priority:
                            if self.ec.has('pre_' + _msg):  # если есть уже "ожидающее событие"
                                if self.ec.is_event_time_more('pre_' + _msg, 40000):    # прошло 40 секунд после постройки
                                    self.ec.remove('pre_' + _msg)
                                    self.ec.add(_msg)
                                    _fcount = self.ec.get_var('var_30_fcount')
                                    self.ec.remove_var('var_30_fcount')
                                    if _fcount != _g.fields.functional:    # если за 20 секунд была построена хоть одна шахта
                                        return False
                                    else:
                                        self.show_msg('TUT_HELP_30', {'id':30, 'blink_build_button':True})
                                        return True
                            else:
                                self.ec.add('pre_' + _msg)
                                self.ec.add(_msg)
                                self.ec.add_var('var_30_fcount', _g.fields.functional)
        return False
# ============================================================
    def msg_31(self):
        '''31 silvermine - build'''
        _msg = 'msg_31'
        if self.ec.is_event_spoiled(_msg, 40000):   # если готово событие или его не было
            self.ec.remove(_msg)                    # удалим событие
            if 'silvermine' in self._binfos:        # если есть на карте шахта
                #! добавить проверку на достаточночть ресурсов для постройки
                #_r = self._li.resources
                #if _r.food >= 15 and _r.bricks >= 40 and _r.coins >= 40:    # если есть ресурсы на строительство
                _g = self._binfos.silvermine
                if _g.fields.map_places > 0:       # если есть места для строительства
                    if _g.fields.can_build:        # если строительство допустимо сейчас
                        if self.ec.has('pre_' + _msg):  # если есть уже "ожидающее событие"
                            if self.ec.is_event_time_more('pre_' + _msg, 40000):    # прошло 40 секунд после постройки
                                self.ec.remove('pre_' + _msg)
                                self.ec.add(_msg)
                                _fcount = self.ec.get_var('var_31_fcount')
                                self.ec.remove_var('var_31_fcount')
                                if _fcount != _g.fields.functional:    # если за 20 секунд была построена хоть одна шахта
                                    return False
                                else:
                                    self.show_msg('TUT_HELP_31', {'id':31, 'blink_build_button':True})
                                    return True
                        else:
                            self.ec.add('pre_' + _msg)
                            self.ec.add(_msg)
                            self.ec.add_var('var_31_fcount', _g.fields.functional)
        return False
# ============================================================
    def msg_32(self):
        '''32 granitemine - repair'''
        _msg = 'msg_32'
        _can_start = (not self.ec.has(_msg) and not self.ec.has('pre_' + _msg))
        if not _can_start:
            _can_start = self.ec.is_event_time_more('pre_' + _msg, 40000) or\
                    self.ec.is_event_time_more(_msg, 40000)   # если готово событие или его не было
        if _can_start:
            self.ec.remove(_msg)                    # удалим событие
            if 'granitemine' in self._li.binfos:        # если есть на карте шахта
                _g = self._li.binfos.granitemine
                if _g.fields.broken_active > 0:       # если есть сломанные
                    if _g.fields.can_build:        # если строительство допустимо сейчас
                        if self.ec.has('pre_' + _msg):  # если есть уже "ожидающее событие"
                            if self.ec.is_event_time_more('pre_' + _msg, 40000):    # прошло 40 секунд
                                self.ec.remove('pre_' + _msg)
                                self.ec.add(_msg)
                                _fcount = self.ec.get_var('var_32_fcount')
                                self.ec.remove_var('var_32_fcount')
                                if _fcount != _g.fields.functional:    # если за 20 секунд была построена/отремонтирована хоть одна шахта
                                    return False
                                else:
                                    self.show_msg('TUT_HELP_32', {'id':32})
                                    self.ec.add(_msg)
                                    return True
                        else:
                            self.ec.add('pre_' + _msg)
                            self.ec.add_var('var_32_fcount', _g.fields.functional)
        return False
# ============================================================
    def msg_34(self):
        '''34 quarry - build'''
        _msg = 'msg_34'
        _can_run = self.ec.is_event_spoiled(_msg, 20000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            if 'granite' in self._li.binfos:    # если есть гранит в зоне проходимости
                _show = False
                if 'quarry' not in self._li.binfos:
                    _show = True
                else:
                    _quarry = self._li.binfos.quarry
                    if _quarry.fields.functional == 0 and _quarry.fields.broken_active == 0:
                        _show = True
                if _show:
                    self.show_msg('TUT_HELP_34', {'id':34, 'blink_build_button':True})
                    return True
        return False
# ============================================================
    def msg_35(self):
        '''35 stonecutter - build'''
        _msg = 'msg_35'
        if LBoard.Core.episode == 1 and LBoard.Core.level in [1,2]:
            return False
        _can_run = self.ec.is_event_spoiled(_msg, 180000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            _stonecutters = 0
            _granite_gens = 0
            if 'stonecutter' in self._binfos:    # если есть гранит в зоне проходимости
                _s = self._binfos.stonecutter
                if _s.fields.broken > 0:
                    return False
                _stonecutters = _s.fields.functional
                if _stonecutters == 0:
                    return False
                if _s.fields.min_eff < 60 and _s.fields.max_eff < 80:
                    if self._li.resources._granite > 5 and self._li.resources.bricks < 100:
                        if _s.fields.functional < 3:
                            self.show_msg('TUT_HELP_35', {'id':35})
                            return True
            else:
                return False
            # далее идет альтернативное условие:
            if 'granitemine' in self._binfos:
                _granite_gens += self._binfos.granitemine.fields.functional
            if 'quarry' in self._binfos:
                _granite_gens += self._binfos.quarry.fields.functional
            if (_granite_gens-_stonecutters) >= 1:
                self.show_msg('TUT_HELP_35', {'id':35, 'blink_build_button':True})
                return True
        return False
# ============================================================
    def msg_36(self):
        '''36 smokery - build'''
        _msg = 'msg_36'
        _can_run = self.ec.is_event_spoiled(_msg, 180000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            _smokerys = 0
            _fishermans = 0
            if 'smokery' in self._binfos:
                _s = self._binfos.smokery
                if _s.fields.broken > 0:
                    return False
                _smokerys = _s.fields.functional
            if _smokerys == 0:
                return False
            if self._binfos.smokery.fields.max_eff < 60:
                if self._li.resources.food < 70:
                    if self._li.resources._fish > 5:
                        return True
            if 'fisherman' in self._binfos:
                _f = self._binfos.fisherman
                _fishermans = _f.fields.functional
            if _fishermans == 0:
                return False
            if (_fishermans-_smokerys) >= 1:
                self.show_msg('TUT_HELP_36', {'id':36, 'blink_build_button':True})
                return True
        return False
# ============================================================
    def msg_37(self):
        '''37 bakery - build'''
        _msg = 'msg_37'
        _can_run = self.ec.is_event_spoiled(_msg, 180000) # если нет сообщения или прошло достаточно времени
        if _can_run:
            self.ec.remove(_msg)
            self.ec.add(_msg)
            _bakerys = 0
            _farms = 0
            if 'bakery' in self._binfos:
                _b = self._binfos.bakery
                if _b.fields.broken > 0:
                    return False
                _bakerys = _b.fields.functional
            if _bakerys == 0:
                return False
            if self._binfos.bakery.fields.max_eff < 60:
                if self._li.resources.food < 70:
                    if self._li.resources._flour > 5:
                        return True
            if 'farm' in self._binfos:
                _f = self._binfos.farm
                _farms = _f.fields.functional
            if _farms == 0:
                return False
            if (_farms-_bakerys) >= 1:
                self.show_msg('TUT_HELP_37', {'id':37, 'blink_build_button':True})
                return True
        return False
# ============================================================
# ============================================================
# ============================================================