# -*- coding: utf-8 -*-

import sys
if not 'tinypy' in sys.version:
    from _globals import *
import base_unit


class UnitWorker(base_unit.BaseUnit):
    resource_animation_dict = {
        "": "run",
        "_wood": "run_wood",
        "boards": "run_boards",
        "_stones": "run_stone",
        "_ore": "run_ore",
        "_fish": "run_fish",
        "_fish_sm": "run_fish",
        "coins": "run_coin",
        "bricks": "run_brick",
        "_berries": "run_basket",
        "_bread": "run_basket",
        "food": "run_basket",   # для подстраховки
        "_granite": "run_stone",
        "_flour": "run_bag",
        "gold": "run_bag"
        }
    run_animations = ['run', 'run_w_cut', 'run_w_wood', 'run_bag', 'run_basket',\
                        'run_boards', 'run_brick', 'run_coin', 'run_fish',\
                        'run_ore', 'run_stone', 'run_wood', 'run_builder',\
                        'run_quarry', 'run_soldier']
# ============================================================
    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.resource = None
        self.SM.SetState('stay')
        self.current_action_animation = 'run'   # данное поле выставляется при взаимодействии с тем или иным объектом
        default_unit_speed = Globals.GFloat("DEFAULT_UNIT_SPEED") # default speed
        cur_unit_speed = Globals.GFloat("UNIT_SPEED")
        self.Drawer.SetAnimBoost(self.run_animations,\
                                    1000*cur_unit_speed/default_unit_speed)
        EMgr.add_event_listener(self)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_event_listener(self)
# ============================================================
    def on_message(self, msg_type, msg_param1="", msg_param2="", msg_dict={}):
        if msg_type == 'hit_moment':
            if self.task:
                self.task.on_bump_need()
            else:
                self.PlaySoundRandom(['SND_HAMMER_HIT_1', 'SND_HAMMER_HIT_2', 'SND_HAMMER_HIT_3'])
        elif msg_type == 'build_hit_complete':
            self.Drawer.PlayAnimationOnce('build', 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
        elif msg_type == 'at_target':
            if self.home_building: # running home and already at home
                # if not istype(self.target, "list"): # and self.target.__class__ == "base_building.BaseBuiding":
                # if "__class__" in self.target: # and self.target.__class__ == "base_building.BaseBuiding":
                if 'col' not in self.target:
                    if 'row' not in self.target: # and self.target.__class__ == "base_building.BaseBuiding":
                        _is_target_home = self.target.is_home()
                        if _is_target_home:
                            self.target.in_unit(self)    # hide unit in building
                        else:
                            self.give_resource(self.target) # отдадим ресурс зданию
                            self.target.remove_walking_home_unit(self)  # освободимся от здания
                            self.go_home()  # теперь можно идти домомй
                self.switch_animation()
                return
            else:       # at target
                trace(self.task_type)
                if self.task:
                    self.task.target_reached(self)
                #else:
                #    self.go_home()   # hack - just for test - running to the nearest vacant home
                self.switch_animation()
                return
# ============================================================
    def on_signal(self, signal):
        base_unit.BaseUnit.on_signal(self, signal)
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        #if name == 'resources_changed':            # изменилось кол-во ресурсов у игрока
        #    self.switch_animation()     # чтобы знак над индейцем обновить
        if name in ['barbarian_attack_started', 'barbarian_attack_completed', 'barbarian_attack_canceled']:
            self.switch_animation()
# ============================================================
    def on_state_enter(self, state_name, additional):
        self.switch_animation()
# ============================================================
    def on_state_exit(self, state_name, additional):
        #if state_name == "build":
        #    self.go_home()   # hack - just for test - running to nearest vacant home
        pass
# ============================================================
    def on_state_change(self, cur_state, next_state):
        if cur_state == "run":
            if next_state == "stay":
                self.StopMove()
# ============================================================
    def on_create(self):
        self.SetDirection(self.GetAttrInt('start_direction'))
        self.switch_animation()
# ============================================================
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)
# ============================================================
# ============================================================
# ============================================================
    simple_dict = {"build": "build", "run": "run", "stay": 'default'}
    builder_animation_dict = {"run" : 'run_builder', 'build' : 'build', "action" : 'build', 'stay':'run_builder'}#<<
    breaker_animation_dict = {"run" : 'run_breaker', 'action' : 'action_breaker', 'stay':'run_breaker'}#<<
    woodcutter_animation_dict = {'run': 'run_w_cut', 'action': 'action_w_cut', 'stay': 'run_w_cut'}
    soldier_animation_dict = {'run': 'run_soldier', 'action': 'run_soldier', 'stay': 'run_soldier'}
    quarry_animation_dict = {'run': 'run_quarry', 'action': 'action_quarry', 'stay': 'run_quarry'}#<<
# ============================================================
    def switch_animation(self):
        _is_partial_fog = self.Drawer.IsPartiallyInFog()
        if _is_partial_fog:     # для выживших?
            self.Drawer.ShowImages('', False)
            return
        new_animation_name = 'run'
        state_name = self.SM.GetState()
        #if self.resource and not self.action_item:
        _unit_with_res = False
        if self.resource:
            if not self.action_item:
                _unit_with_res = True
        if _unit_with_res:
            new_animation_name = self.resource_animation_dict[self.resource.GetSingleName()]
            if new_animation_name == 'run_wood':
                if self.current_subtype == 'dt_wcutter':    # если лесоруб с бревном
                    new_animation_name = 'run_w_wood'
        elif self.current_subtype == 'builder': # строитель с молотком
            if state_name in self.builder_animation_dict:
                new_animation_name = self.builder_animation_dict[state_name]
        elif self.current_subtype == 'dt_wcutter': # с топором
            if state_name in self.woodcutter_animation_dict:
                new_animation_name = self.woodcutter_animation_dict[state_name]
        elif self.current_subtype == 'soldier':
            if state_name in self.soldier_animation_dict:
                new_animation_name = self.soldier_animation_dict[state_name]
        elif self.current_subtype == 'quarryman':   # ломатель киркой
            if state_name in self.quarry_animation_dict:
                new_animation_name = self.quarry_animation_dict[state_name]
        elif self.current_subtype == 'breaker': # ломатель киркой
            if state_name in self.breaker_animation_dict:
                new_animation_name = self.breaker_animation_dict[state_name]
        elif self.current_subtype == 'simple':  # обычный юнит
            if state_name in self.simple_dict:
                new_animation_name = self.simple_dict[state_name]
            elif state_name == 'action':
                # нужно выбирать анимацию в зависимости от объекта и действия
                new_animation_name = self.current_action_animation
        dir = self.GetDirection()
        self.Drawer.ShowImages('', False)
        self.Drawer.ShowImages(new_animation_name, True)
        self.Drawer.SetSequence(new_animation_name, dir)
        self.Drawer.ShowImages('shadow', True)
        _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        if not _has_barbarians:
            _has_access = self.HasAccess()
            if _has_access:
                if self.is_survived:
                    if not self.is_deactivated:
                        if self.GetAttrBool('is_talking'):
                            self.Drawer.ShowImages('sign_e', True)
# ============================================================
    def has_sign(self):
        return (self.is_survived and not self.is_deactivated and self.GetAttrBool('is_talking'))
# ============================================================




